/**********************************************************************************
**
**		jQuery Tristate Checkbox Plugin
**		version: 1.0
**
**		Dual licensed under the MIT and GPL licenses:
**   		http://www.opensource.org/licenses/mit-license.php
**   		http://www.gnu.org/licenses/gpl.html
**
**		author: Ben Belcourt
**		creation date: 04.11.2010
**		dependencies: "jquery-1.3.2.js" or higher
**
**		This file contains the functionality for implementing 3 state checkboxes.
**
**********************************************************************************/

(function ($){
	$.fn.tristate = function(options) {  
	
		var config = {
			checkedClass: 'checked',
			fullClass: 'full',
			heading: 'span.label',
			partialClass: 'partial',
			showOriginalInputs: false,
            labelCallBack: function(){},
            app: ''
		};
		var opts = $.extend(config,options);
		
		return this.each(function() {  
			obj = $(this);

			var triState = {
				init: function () {
					// Add proxy checkbox for each heading
					
					var $inputs = obj.find('a.checkbox');
					for (var i=0, y=$inputs.length; i<y; i++) {
						// sets up the input fields
                        
                        $($inputs[i]).bind('click', function (e) {
                            e.preventDefault();
                            e.stopPropagation();
                            triState.initProxyClick($(this));
                        });
                        
                        $($inputs[i]).bind('update', function(){
                            triState.updateProxyClick($(this));
                        });
                        
                        $inputs[i] = null;
					}	

                    $inputs = null;
				},
				
				checkSiblings: function ($el, bChecked, callback) {
					
					var $parentList = $el.parent('div').parent('li').parent('ul');
					var sResult='';
					
                    if(bChecked){
                        sResult = opts.partialClass;
                    }else{
                        $parentList.children('li').each(function(i,x){
                            if($(x).find('div>a.checkbox.'+opts.checkedClass+', div>a.checkbox.'+opts.partialClass).size() > 0){
								sResult = opts.partialClass;
                                return false;
                            }                            
                        });
						
                        if(opts.app=='local'){
                            var filePath = $parentList.parent('li').children('div').find('input:checkbox').val();
                            for(var i=0; i<global_obj.selectedFilesList.length; i++){
                                if(global_obj.selectedFilesList[i].indexOf(filePath) == 0){
                                    sResult = opts.partialClass;
                                    break;
                                }
                            }
                        }else if(opts.app=='mozy'){
                            var filePath = $parentList.parent('li').children('div').find('input:checkbox').val();
                            for(var i=0; i<global_obj.selectedRestoreFilesList.length; i++){
                                if(global_obj.selectedRestoreFilesList[i].indexOf(filePath) == 0){
                                    sResult = opts.partialClass;
                                    break;
                                }
                            }                            
                        }
						
                    }
					
					// Set the appropriate class to the parent list
					$parentList.removeClass(opts.partialClass + ' ' + opts.fullClass).addClass(sResult);
                    
                    if($j.isFunction(callback)){
                        callback.apply();
                    }
                    
                    parentList = null;
                    $el = null;
				},
				
                updateProxyClick: function($el){
					var bChecked;
                    
                    ($($el).hasClass(opts.checkedClass) === true || $($el).hasClass(opts.partialClass) === true) ? bChecked = true : bChecked = false ;

					// Check for 'checked' siblings
                    triState.checkSiblings($el, bChecked, function(){
                        // Check/uncheck this element's ancestors appropriately
                        triState.setAncestors($el, bChecked);
                    });
                    
                },
                
				initProxyClick: function ($el) {
					var bChecked;
					triState.setInputField($el);
					
					// Initialize the input field
					($($el).hasClass(opts.checkedClass) === true || $($el).hasClass(opts.partialClass) === true) ? bChecked = true : bChecked = false ;		
					
											
					// Check for 'checked' siblings
                    triState.checkSiblings($el, bChecked, function(){
                        // Check/uncheck this element's ancestors appropriately
                        triState.setAncestors($el, bChecked);
                        // Check/uncheck this elements descendants appropriately
                        triState.setDescendants($el, bChecked);
                        // Check/uncheck this elements associated label
                        triState.setLabel($el, bChecked, 'origin');
                    });
                        
                    $el = null;
                    
					// triState.updateProxyStates();
				},
				
				updateProxyStates: function () {
					//var $inputs = $('input:checked', obj);
                    var $inputs = obj.find('input:checked');
					var $proxyLink;
					for (var i=0, y=$inputs.length; i<y; i++) {
						$proxyLink = $($inputs[i]).siblings('a.checkbox');
						triState.setInputField($proxyLink, 'checked');
						triState.checkSiblings($($inputs[i]), true);
						triState.setAncestors($($inputs[i]), true);
					}
				},
				
				setAncestors: function ($el, bChecked) {
					var bAncestorCheck;
					var $parentList = $el.parent('div').parent('li').parent('ul');
					var $ancestorInput = $parentList.parent('li').children('div').children('a.checkbox');
					
					// Check the class on the parentList and check the ancestorInput appropriately
					if (!$parentList.hasClass(opts.partialClass) && !$parentList.hasClass(opts.fullClass)) {
						$ancestorInput.removeClass(opts.checkedClass +' '+ opts.partialClass);
						triState.setInputField($ancestorInput, 'unchecked');
						triState.setLabel($ancestorInput, bChecked, 'ancestor');
						bAncestorCheck = false;
					} else {
						triState.setInputField($ancestorInput, 'checked');
						if ($parentList.hasClass(opts.partialClass)) { $ancestorInput.removeClass(opts.checkedClass).addClass(opts.partialClass); }
						if ($parentList.hasClass(opts.fullClass)) { $ancestorInput.removeClass(opts.partialClass).addClass(opts.checkedClass); }
						triState.setLabel($ancestorInput, bChecked, 'ancestor');
						bAncestorCheck = true;
					}
					
					// As long as we're not at the root element then keep traversing up the chain to check each input appropriately
					if ($ancestorInput.length > 0) {
						triState.checkSiblings($ancestorInput, bAncestorCheck);
						triState.setAncestors($ancestorInput, $ancestorInput.hasClass(opts.checkedClass));
					}
                    
                    $parentList = null;
                    $ancestorInput = null;
				},
				
				setDescendants: function ($el, bChecked) {
					var $parentItem = $el.parent('div').parent('li').find('ul');
					
					if (bChecked) {
						$parentItem.removeClass(opts.partialClass + ' ' + opts.fullClass).addClass(opts.fullClass);
                        $parentItem.find('a.checkbox').each(function(){
							triState.setInputField($(this), 'checked');
							triState.setLabel($(this), bChecked, 'descendant');
                        });
					} else {
						$parentItem.removeClass(opts.partialClass + ' ' + opts.fullClass);
                        $parentItem.find('a.checkbox').each(function(){
							triState.setInputField($(this), 'unchecked');
							triState.setLabel($(this), bChecked, 'descendant');
                        });
					}
                    
                    $parentItem = null;
				},
				
				setInputField: function ($el, state) {
					var sState = state || '';
					switch (sState) {
						case '':
							($el.hasClass(opts.checkedClass) === true) ? triState.uncheckInput($el) : triState.checkInput($el);
							break;
						case 'checked':
							triState.checkInput($el);
							break;
						case 'unchecked':
							triState.uncheckInput($el);
							break;
					}
				},
				
				setLabel: function ($el, state, position) {
					$label = $el.siblings('label');
					(state) ? triState.checkLabel($label) : triState.uncheckLabel($label);
                    
                    //added by johnson
                    opts.labelCallBack.call(this, $label, state, position);
                    
				},
				
				checkInput: function ($el) {
					$el.removeClass(opts.partialClass).addClass(opts.checkedClass).siblings('input:checkbox').attr('checked', 'checked');
				},
				
				uncheckInput: function ($el) {
					$el.removeClass(opts.checkedClass +' '+ opts.partialClass).siblings('input:checkbox').attr('checked', '');
				},
				
				checkLabel: function ($el) {
					$el.removeClass(opts.partialClass).addClass(opts.checkedClass);
				},
				
				uncheckLabel: function ($el) {
					$el.removeClass(opts.checkedClass +' '+ opts.partialClass);
				}
			};
			
			triState.init();			

		});  
	};
})(jQuery);