// some configuration
var config = {
	max_criteria: 10, // set to 0 for no limit
	sort_fields_by_name: false,
	debug: false
}

// the criteria object
var criteria = {
	// all criteria are 3 parts: a field, an operator, and a value
	PART_FIELD:    1,
	PART_OPERATOR: 2,
	PART_VALUE:    3,

	// field types
	FIELD_BOOL:    4,
	FIELD_INT:     5,
	FIELD_STRING:  6,
	FIELD_DATE:    7,

	// configurations of value input fields
	VALUE_INPUT:   8, // (int, string) a text input
	VALUE_LIST:    9, // (int, string) a limited list of values (codectype, for example)
	VALUE_SPAN:   10, // (date)        a text input and a unit (day, week, ...)
//	VALUE_RANGE:  11, // (date)        two text inputs (date1 to date2)

	// a running count of criteria; used to generate a unique ID for a new criterion
	count: 0,
	new_count: 0,
	// each criterion's current field type is stored so that any entered value can be preserved when changing between type compatible fields
	old_types: new Array(),

	fields:    null, // an array of fields
	operators: null  // an array of operators for each field type
}

var field_options = ''; // the field <option> tag HTML
//var target = null;      // where the playlist text will end up
var old_value = '';     // the original value of target

/**
 * initialization
 *
 * @return void
 */
function main() {
	// populate the criteria object's field array
	criteria.fields = new Array(
		// boolean fields
//		{ value: 'compilation',  label: '',   type: criteria.FIELD_BOOL },

		// integer fields
		{ value: 'bitrate',      label: NIC_ITUNES_STR022, type: criteria.FIELD_INT, index: 5 },
//		{ value: 'bpm',          label: 'BPM',          type: criteria.FIELD_INT },
//		{ value: 'data_kind',    label: '',             type: criteria.FIELD_INT },	// what the heck is this? (it's only ever 1 or 0)
//		{ value: 'disc',         label: 'Disc Number',  type: criteria.FIELD_INT },
//		{ value: 'file_size',    label: '',             type: criteria.FIELD_INT },
//		{ value: 'force_update', label: '',             type: criteria.FIELD_INT },
//		{ value: 'id',           label: 'ID',           type: criteria.FIELD_INT },
//		{ value: 'idx',          label: '',             type: criteria.FIELD_INT },
//		{ value: 'item_kind',    label: '',             type: criteria.FIELD_INT }, // this appears to be hard coded to '2', and as such not yet useful
//		{ value: 'play_count',   label: '',             type: criteria.FIELD_INT },
//		{ value: 'rating',       label: '',             type: criteria.FIELD_INT },
//		{ value: 'samplerate',   label: 'Sample Rate',  type: criteria.FIELD_INT },
//		{ value: 'sample_count', label: '',             type: criteria.FIELD_INT },
//		{ value: 'song_length',  label: '',             type: criteria.FIELD_INT },
//		{ value: 'totaldiscs',   label: 'Total Discs',  type: criteria.FIELD_INT },
//		{ value: 'total_tracks', label: '',             type: criteria.FIELD_INT },
//		{ value: 'track',        label: 'Track Number', type: criteria.FIELD_INT },
		{ value: 'year',         label: NIC_ITUNES_STR021, type: criteria.FIELD_INT, index: 4 },

		// string fields
		{ value: 'album',        label: NIC_ITUNES_STR017, type: criteria.FIELD_STRING, index: 0 },
		{ value: 'artist',       label: NIC_ITUNES_STR018, type: criteria.FIELD_STRING, index: 1 },
//		{ value: 'comment',      label: '',            type: criteria.FIELD_STRING },
//		{ value: 'composer',     label: '',            type: criteria.FIELD_STRING },
//		{ value: 'description',  label: '',            type: criteria.FIELD_STRING },
//		{ value: 'fname',        label: 'File Name',   type: criteria.FIELD_STRING },
		{ value: 'genre',        label: NIC_ITUNES_STR019, type: criteria.FIELD_STRING, index: 2 },
//		{ value: 'grouping',     label: '',            type: criteria.FIELD_STRING },
//		{ value: 'orchestra',    label: '',            type: criteria.FIELD_STRING },
		{ value: 'path',         label: NIC_ITUNES_STR020, type: criteria.FIELD_STRING, index: 3 },
//		{ value: 'title',        label: NIC_ITUNES_STR024, type: criteria.FIELD_STRING, index: 2},
//		{ value: 'type',         label: 'File Type',   type: criteria.FIELD_STRING },
//		{ value: 'url',          label: 'URL',         type: criteria.FIELD_STRING },

		// codectype has a limited number of possible values
/*		{ value: 'codectype',    label: 'Codec Type',  type: criteria.FIELD_STRING,
		  values: new Array(
		  	{ value: 'alac', label: 'Apple Lossless' },
		  	{ value: 'flac', label: 'FLAC'           },
		  	{ value: 'mp4a', label: 'AAC'            },
		  	{ value: 'mpc',  label: 'Musepack'       },
		  	{ value: 'mpeg', label: 'MP3'            },
		  	{ value: 'ogg',  label: 'OGG'            },
		  	{ value: 'wav',  label: 'WAV'            },
		  	{ value: 'wma',  label: 'WMA'            }
		  ) },
*/
		// date fields
//		{ value: 'db_timestamp',  label: 'Last Indexed',  type: criteria.FIELD_DATE },
		{ value: 'time_added',    label: NIC_ITUNES_STR023, type: criteria.FIELD_DATE, index: 6 }
//		{ value: 'time_played',   label: 'Last Played',   type: criteria.FIELD_DATE },
//		{ value: 'time_modified', label: 'Date Modified', type: criteria.FIELD_DATE }
	);

	// populate the operators by field type
	criteria.operators = new Array();
	criteria.operators[ criteria.FIELD_BOOL ] = new Array(
			{ value: 'is 1', label: 'is true',  type: null },
			{ value: 'is 0', label: 'is false', type: null }
		);
	criteria.operators[ criteria.FIELD_INT ] = new Array(
			{ value: 'is',     label: NIC_ITUNES_STR009, type: criteria.VALUE_LIST },
//			{ value: 'not is', label: NIC_ITUNES_STR010, type: criteria.VALUE_LIST },
			{ value: '>',     label: NIC_ITUNES_STR011, type: criteria.VALUE_INPUT },
			{ value: '<',     label: NIC_ITUNES_STR012, type: criteria.VALUE_INPUT }
		);
	criteria.operators[ criteria.FIELD_STRING ] = new Array(
			{ value: 'includes',     label: NIC_ITUNES_STR007, type: criteria.VALUE_INPUT},
			{ value: 'not includes', label: NIC_ITUNES_STR008, type: criteria.VALUE_INPUT},
			{ value: 'is',            label: NIC_ITUNES_STR009, type: criteria.VALUE_LIST},
			{ value: 'not is',        label: NIC_ITUNES_STR010, type: criteria.VALUE_LIST}
		);
	criteria.operators[ criteria.FIELD_DATE ] = new Array(
//			{ value: '=',    label: 'is',                 type: criteria.VALUE_INPUT },
			{ value: '<',    label: NIC_ITUNES_STR013, type: criteria.VALUE_INPUT },
			{ value: '>',    label: NIC_ITUNES_STR014, type: criteria.VALUE_INPUT },
			{ value: 'after',    label: NIC_ITUNES_STR015, type: criteria.VALUE_SPAN },
			{ value: 'before',    label: NIC_ITUNES_STR016, type: criteria.VALUE_SPAN }
//			{ value: 'from', label: 'is in the range',    type: criteria.VALUE_RANGE }
		);
		
	criteria.fields = criteria.fields.sort( field_sort_callback_ex );
/*
	
	for( var i in criteria.fields ) {
		// populate any empty field labels with a capitalized version of the field name
		if( ! criteria.fields[i].label ) {
			
			if( criteria.fields[i].value.match( /_/ ) ) {
				var words = criteria.fields[i].value.split( "_" );
				for( var j in words ) {
					words[j] = words[j].substring( 0, 1 ).toUpperCase() + words[j].substring( 1 );
				}
				criteria.fields[i].label = words.join( " " );
			} else {
				criteria.fields[i].label = criteria.fields[i].value.substring( 0, 1 ).toUpperCase() + criteria.fields[i].value.substring( 1 );
			}
		}
			
		// sort values by name on any field with a list of values
		if( criteria.fields[i].values ) {
			criteria.fields[i].values = criteria.fields[i].values.sort( field_sort_callback );
		}
	}

	// sort fields by label if desired
	if( config.sort_fields_by_name ) {
		criteria.fields = criteria.fields.sort( field_sort_callback );
	}
*/
	// build the field <option> tags
	for( var i in criteria.fields ) {
		field_options += '<option value="' + criteria.fields[i].value + '">' + criteria.fields[i].label + '</option>';
	}

	// set the target
/*	target = ( window.opener ? window.opener : window ).document.getElementsByName( "playlist_spec" )[0];
	if( ! target ) {
		alert( "The playlist criteria textarea could not be found.  (That's bad.)" );
	} else {
		old_value = target.value;
	}
*/
	// add the initial criterion form on load
	if( ! window.addEventListener && document.attachEvent ) { // an IE fix
	    window.addEventListener = function( type, listener, useCapture ) { this.attachEvent( "on" + type, listener ); }
	}
//	window.addEventListener( "load", addCriterion, false );

	return true;
}

/**
 * add a new criterion line to the builder form (triggered by a click of the '+' button)
 *
 * @param string the ID of a criterion DIV that the new DIV will be inserted after
 *
 * @return bool success
 */
function addCriterion( id , add_div) {
	// get the criteria wrapper <div>
	var criteria_div = document.getElementById( add_div );
	
	// if the criteria wrapper <div> is missing something's really broken
	if( ! criteria_div ) {
		return false;
	}

	// if an id was passed, get the div with that id
	if( id && id.match && id.match( /c[0-9]+/ ) ) {
		var ref_criterion_div = document.getElementById( id );
	}

	// make sure there aren't too many criteria already	
	var num_criteria = criteria_div.childNodes.length;
	if( config.max_criteria && num_criteria >= config.max_criteria ) {
		return false;
	}

	// generate the id for the new criterion
	if(add_div=="criteria"){
		criteria.count++;
		var id = 'c' + ( criteria.count );
	}
	else if(add_div=="criteria_new"){
		criteria.new_count++;
		var id = 'ec' + ( criteria.new_count );
	}
	// create the criterion wrapper node and it's contents
	var criterion_div = document.createElement( "div" );
	criterion_div.id = id;
	criterion_div.className = "criterion";
//	alert(id + 'p' + criteria.PART_FIELD + 's');
//	alert(id + 'p' + criteria.PART_OPERATOR);
	criterion_div.innerHTML = 
		'<select id="' + id + 'p' + criteria.PART_FIELD + 's" onchange="populateCriterionOperators(\'' + id + '\');setting_change();" class="p_field">' + field_options + '</select>' +
		'<span id="' + id + 'p' + criteria.PART_OPERATOR + '" class="p_field"></span>' +
		'<span id="' + id + 'p' + criteria.PART_VALUE + '" class="p_field"></span>' +
		'&nbsp;&nbsp;&nbsp;<input type="button" id="' + id + 'plusb" value=" + " class="button" onclick="addCriterion(\'' + id + '\', \'' + add_div + '\');"/>' +
		'<input type="button" id="' + id + 'minorb" value=" - "' + ( num_criteria == 0 ? ' disabled="disabled"' : '' ) + ' class="button" onclick="setting_change();delCriterion(\'' + id + '\', \'' + add_div + '\');"/>&nbsp;';
		
	// if there's a reference criterion, insert immediately after it
	if( ref_criterion_div ) {
		criteria_div.insertBefore( criterion_div, ref_criterion_div.nextSibling );
	}
	// otherwise put it at the end
	else {
		criteria_div.appendChild( criterion_div );
	}

	// update the criterion's operator <select> box
	populateCriterionOperators( id );

	// enable the minus button for the first criterion
	if( num_criteria > 0 ) {
		getElementsByClassName( criteria_div.childNodes[0], "button" )[1].disabled = false;
	}

	if(add_div == "criteria"){
		var divop = document.getElementById('crit_op');
		var divops = document.getElementById('crit_op_single');
	}
	else if(add_div == "criteria_new"){
		var divop = document.getElementById('crit_op_new');
		var divops = document.getElementById('crit_op_single_new');
	}
		if(divop){
			if(num_criteria==0){
				divop.style.display='none';
				divops.style.display='block';
			}
			if( num_criteria > 0 ) {
				divop.style.display='block';
				divops.style.display='none';
			}
			else{
				divop.style.display='none';
//				divops.style.display='none';
			}
		}  
	return id;
}

/**
 * delete an existing criterion
 *
 * @param string the id of the criterion being deleted
 *
 * @return bool success
 */
function delCriterion( id, del_div) {
	// get the criteria wrapper <div>
	var criterion_div = document.getElementById( id );
	
	// if the criteria wrapper <div> is missing something's really broken
	if( ! criterion_div ) {
		return false;
	}
	
	// get the criterion <div> to be removed
	var criteria_div = criterion_div.parentNode;
	
	// remove it
	criteria_div.removeChild( criterion_div );

	// if we're now down to a single criterion, disable its minus button
	if( criteria_div.childNodes.length == 1 ) {
		getElementsByClassName( criteria_div.childNodes[0], "button" )[1].disabled = true;
	}

	// the playlist has changed, so update the text in the opener's <textarea>
	var num_criteria = criteria_div.childNodes.length;

	if(num_criteria == 1){
		if(del_div == "criteria"){
			var divop = document.getElementById('crit_op');
			var divops = document.getElementById('crit_op_single');
		}
		else if(del_div == "criteria_new"){
			var divop = document.getElementById('crit_op_new');
			var divops = document.getElementById('crit_op_single_new');
		}
		
		divop.style.display="none";
		divops.style.display="block";
	}
	
	return true;
}

/**
 * update the operator <select> of a given criterion based on the field type
 *
 * @param string the id of the criterion form div (it'll be in the form c[0-9]+)
 *
 * @return bool success
 */
function populateCriterionOperators( id ) {

	var field_type = getCriterionType( id, criteria.PART_FIELD );
	var criterion_operator = document.getElementById( id + 'p' + criteria.PART_OPERATOR );

	if( ! field_type || ! criterion_operator || ! criteria.operators[ field_type ] ) {
		return false;
	}

	if( field_type != criteria.old_types[ id + 'p' + criteria.PART_OPERATOR ] ) {
		var operator_html = '<select id="' + id + 'p' + criteria.PART_OPERATOR + 's" onchange="populateCriterionValue(\'' + id + '\');setting_change();" class="p_operator">';
		for( var i in criteria.operators[ field_type ] ) {
			operator_html += '<option value="' + criteria.operators[ field_type ][i].value + '">' + criteria.operators[ field_type ][i].label + '</option>';
		}
		operator_html += '</select>';
		criterion_operator.innerHTML = operator_html;
	}

	criteria.old_types[ id + 'p' + criteria.PART_OPERATOR ] = field_type;

	return populateCriterionValue( id );
}

/**
 * update the value field(s) of a given criterion based on the operator type
 *
 * @param string the id of the criterion form div (it'll be in the form c[0-9]+)
 *
 * @return bool success
 */
function populateCriterionValue( id ) {
	var operator_type = getCriterionType( id, criteria.PART_OPERATOR );
	var criterion_value_span = document.getElementById( id + 'p' + criteria.PART_VALUE );
	var value_html = '';

	if( ! operator_type || ! criterion_value_span ) {
		if( ! operator_type ) {
			criteria.old_types[ id + 'p' + criteria.PART_VALUE ] = null;
			criterion_value_span.innerHTML = '';
		}
//		updatePlaylist();
		return false;
	}
	var isyear = 0;
	var istimeadded = 0;
	var isbitrate = 0;
	// build the value input HTML based on the value type
	switch( operator_type ) {
		case criteria.VALUE_LIST:
			var field_values = getCriterionValues( id );
			if( field_values && field_values.length ) {
				var value_html = '<select class="p_value" onchange="setting_change();">';
				for( var i in field_values ) {
					value_html += '<option value="' + field_values[i].value + '">' + field_values[i].label + '</option>';
				}
				value_html += '</select>&nbsp;';
				break;
			} else {
				operator_type = criteria.VALUE_INPUT;
			}
		case criteria.VALUE_INPUT:
			var mycriterion = document.getElementById( id + 'p' + criteria.PART_FIELD + 's' );
			if(mycriterion.options[ mycriterion.selectedIndex ].value.toString()=="time_added"){
				var mycritop = document.getElementById( id + 'p' + criteria.PART_OPERATOR + 's' );
					if(mycritop.options[ mycritop.selectedIndex ].value.toString() == "<" ||
							mycritop.options[ mycritop.selectedIndex ].value.toString() == ">"){
						istimeadded=1;
						var mDate = new Date();
						var month = mDate.getMonth()+1
						var aDate = mDate.getDate();
						if(month<10)
							month = '0'+month;
						if(aDate < 10)
							aDate = '0'+aDate;
						var value_html = '<input type="text" id="' + id + 'p' + criteria.PART_VALUE + 't" name="' + id + 'p' + criteria.PART_VALUE + 't" value="'+ mDate.getFullYear() + '-' + month + '-'+ aDate + '"maxlength=64 class="p_text p_value" style="WIDTH:110px;" onKeydown="setting_change();"/>';
				}
			}
			else if(mycriterion.options[ mycriterion.selectedIndex ].value.toString()=="year"){
				isyear=1;
				var mDate = new Date();
				var value_html = '<input type="text" id="' + id + 'p' + criteria.PART_VALUE + 't" name="' + id + 'p' + criteria.PART_VALUE + 't" value="'+ mDate.getFullYear() + '" class="p_text p_value" onKeydown="setting_change();"/>';
			}
			else if(mycriterion.options[ mycriterion.selectedIndex ].value.toString()=="bitrate"){
				isbitrate=1;
				var value_html = '<input type="text" maxlength=64 id="' + id + 'p' + criteria.PART_VALUE + 't" name="' + id + 'p' + criteria.PART_VALUE + 't"  class="p_text p_value" style="WIDTH:110px;" onKeydown="setting_change();"/>&nbsp; kbps';
			}
			else{
				var value_html = '<input type="text" id="' + id + 'p' + criteria.PART_VALUE + 't" name="' + id + 'p' + criteria.PART_VALUE + 't" maxlength=64 class="p_text p_value" onKeydown="setting_change();"/>';
			}
			break;
		case criteria.VALUE_SPAN:
			var value_html =
				'<input type="text" id="' + id + 'p' + criteria.PART_VALUE + 't" name="' + id + 'p' + criteria.PART_VALUE + 't" maxlength=64 class="p_sm_text p_value" onKeydown="setting_change();"/>' + 
				'<select class="sm_p_value p_value" id="' + id + 'p' + criteria.PART_VALUE + 's" name="' + id + 'p' + criteria.PART_VALUE + 's" onchange="setting_change();">' +
					'<option value="day">' + NIC_ITUNES_STR057 + '</option>' +
					'<option value="week">' + NIC_ITUNES_STR058 + '</option>' +
					'<option value="month">' + NIC_ITUNES_STR059 +'</option>' +
					'<option value="year">' + NIC_ITUNES_STR060 + '</option>' +
				'</select> &nbsp;';
			break;
/*			case criteria.VALUE_RANGE:
				var value_html = 
					'<input type="text" maxlength=64 name="' + id + '" class="sm_text p_value"  />' + 
					'<span class="p_value_spacer">to</span>' +
					'<input type="text" maxlength=64 name="' + id + '" class="sm_text p_value" />';
				break;
*/		case null:
			var value_html = '';
			break;
		default:
			config.debug && alert( 'unknown operator type: ' + operator_type );
			break;
	}
	
	// only set the new value inputs if the old value is a different type
	if( operator_type != criteria.old_types[ id + 'p' + criteria.PART_VALUE ] || isyear || istimeadded || isbitrate) {
		criterion_value_span.innerHTML = value_html;
	}
	
	// save the value type
	criteria.old_types[ id + 'p' + criteria.PART_VALUE ] = operator_type;
	
//	updatePlaylist();

	return true;
}

/**
 * get a field object by value
 *
 * @param string the value of the criterion field
 *
 * @return bool|object either false on error or an item from the critera.fields array
 */
function getCriterionFieldByLabel( value ) {
	for( var i in criteria.fields ) {
		if( criteria.fields[i].value.toLowerCase() == value.toLowerCase() ) {
			return criteria.fields[i];
		}
	}
	
	return false;
}

/**
 * get the list of values for a given criterion
 *
 * @param string the id of the criterion form div (it'll be in the form c[0-9]+)
 *
 * @return bool|array either false on error, or an array of values
 */
function getCriterionValues( id ) {
	var criterion_select = document.getElementById( id + 'p' + criteria.PART_FIELD + 's' );

	if( ! criterion_select ) {
		return false;
	}

	var field = getCriterionFieldByLabel( criterion_select.value );
	
	if( field && field.values ) {
		return field.values;
	}
	
	return false;
}

/**
 * get a field or operator type for a given criterion
 * 
 * @param string the id of the criterion form div (it'll be in the form c[0-9]+)
 * @param int the part to get the type of (one of the criteria.PART_* constants)
 *
 * @return bool|int either false on error, or one of the criteria.(FIELD|OPERATOR)_* type constants
 */
function getCriterionType( id, part ) {
	var criterion_select = document.getElementById( id + 'p' + part + 's' );

	if( ! criterion_select ) {
		return false;
	}

	return getPartTypeByLabel( id, part, criterion_select.options[ criterion_select.selectedIndex ].text );
}

/**
 * get the the type of a criterion part by one of the <select> box labels
 *
 * @param string the id of the criterion form div (it'll be in the form c[0-9]+)
 * @param int the part to get the type of (one of the criteria.PART_* constants)
 * @param string the label of the part
 *
 * @return bool|int either false on error, or one of the criteria.(FIELD|OPERATOR)_* type constants
 */
function getPartTypeByLabel( id, part, label ) {
	if( part == criteria.PART_FIELD ) {
		for( var i in criteria.fields ) {
			if( criteria.fields[i].label == label ) {
				return criteria.fields[i].type;
			}
		}
		return false;
	} else if( part == criteria.PART_OPERATOR && criteria.operators[ getCriterionType( id, criteria.PART_FIELD ) ] ) {
		var field_type = getCriterionType( id, criteria.PART_FIELD );
		for( var i in criteria.operators[ field_type ] ) {
			if( criteria.operators[ field_type ][i].label == label ) {
				return criteria.operators[ field_type ][i].type;
			}
		}
	}

	return false;
}

/**
 * 
 */
function field_sort_callback( a, b ) {
	if( a.label < b.label ) {
		return -1
	} else if( a.label > b.label ) {
		return 1;
	} else {
		return 0;
	}
}

/**
 * 
 */
function field_sort_callback_ex( a, b ) {
	if( a.index < b.index ) {
		return -1
	} else if( a.index > b.index ) {
		return 1;
	} else {
		return 0;
	}
}

/**
 * update the target (probably a <textarea> with the current playlist as text
 */

function updatePlaylist() {
	alert("UpdatePlaylist");
//	target.value = generatePlaylist();
}

/**
 * convert the builder form into daapd playlist format
 *
 * @return string the playlist text
 */
 /*
function generatePlaylist() {
	var playlist_text = "";

	if( document.getElementById( "and_or" ) ) {
		var and_or = " " + ( document.getElementById( "and_or" ).value == "and" ? "AND" : "OR" );
	}

	var criteria_divs = document.getElementById( 'criteria' ).getElementsByTagName( 'div' );

	for( var i = 0, j = 0; i < criteria_divs.length; i++ ) {
		if( criteria_divs[i].id && criteria_divs[i].id.match( /^c[0-9]+$/ ) ) {
			playlist_text += ( j++ > 0 ? and_or + "\n" : "" );

			var criterion_id = criteria_divs[i].id;

			// get the field name and comparison operator
			playlist_text += getElementsByClassName( criteria_divs[i], "p_field" )[0].value;
			playlist_text += " " + getElementsByClassName( criteria_divs[i], "p_operator" )[0].value;

			switch( getCriterionType( criteria_divs[i].id, 1 ) ) {
				case criteria.FIELD_BOOL:
					break;
				case criteria.FIELD_DATE:
					switch( getCriterionType( criteria_divs[i].id, 2 ) ) {
						case criteria.VALUE_INPUT:
							playlist_text += " " + getElementsByClassName( criteria_divs[i], "p_value" )[0].value;
							break;
						case criteria.VALUE_SPAN:
							var value = getElementsByClassName( criteria_divs[i], "p_value" )[0].value;
							var unit = getElementsByClassName( criteria_divs[i], "p_value" )[1].value
							playlist_text += " " + value + " " + unit + ( value == 0 || value > 1 ? "s" : "" ) + " before today";
							break;
					}
					break;
				case criteria.FIELD_INT:
					playlist_text += " " + getElementsByClassName( criteria_divs[i], "p_value" )[0].value;
					break;
				case criteria.FIELD_STRING:
					playlist_text += " \"" + getElementsByClassName( criteria_divs[i], "p_value" )[0].value + "\"";
					break;
			}
		}
	}

	return playlist_text;
}
*/
/**
 * recursively get elements that of a certain CSS class
 *
 * @param element the node at which to start searching
 * @param string the CSS class name
 *
 * @return Array an array of nodes
 */
function getElementsByClassName( node, className ) {
	if( typeof node != "object" || ! node.nodeType || node.nodeType != 1 || ! node.childNodes ) {
		return matches;
	}
	
	var children = node.getElementsByTagName( '*' );
	var elements = new Array();

	for( var i = 0; i < children.length; i++ ) {
		var child = children[i];
		var classNames = child.className.split( ' ' );
		for( var j = 0; j < classNames.length; j++ ) {
			if( classNames[j].toLowerCase() == className.toLowerCase() ) {
				elements.push( child );
				break;
			}
		}
	}

	return elements;
}

/**
 * reset the critera back to a single criterion
 *
 * @return bool success
 */
function resetCriteria(div) {
	return ( clearCriteria(div) && addCriterion('c1',div) );
}

/**
 * clear all criteria
 */
function clearCriteria(clear_div) {
	// get the criteria wrapper <div>
	var criteria_div = document.getElementById( clear_div );

	// if the criteria wrapper <div> is missing something's really broken
	if( ! criteria_div ) {
		return false;
	} else {
		criteria_div.innerHTML = '';
		if(clear_div=="criteria"){
			criteria.count = 0;
		}
		else if(clear_div=="criteria_new"){
			criteria.new_count = 0;
		}
		criteria.old_types = new Array();
		return true;
	}
}

/**
 * load a playlist from it's text definition
 *
 * @param string the playlist definition to parse
 *
 * @return bool success
 */
 /*
function loadPlaylist( playlist_text ) {
	if( ! playlist_text || ! clearCriteria() ) {
		return false;
	}

	// split into individual criteria
	var criteria_text = playlist_text.split( /\s+(?:AND|OR|&&|\|\|)\s+/ );

	// parse each of the criteria
	var success = true;
	for( i in criteria_text ) {
		// add the criterion to the form
		var criterion_id = addCriterion();

		// get the criteria field (the first token) from the text
		var field = criteria_text[i].replace( /^([^\s]+).*$/, '$1' );
		var operator = criteria_text[i].replace( /^[^\s+]+\s([^0-9"]+).*$/, '$1' ).replace( /(^\s+|\s+$)/, '' );

		if( ! field || 
		    ! setCriterionField( criterion_id, field ) ||
		    ! populateCriterionOperators( criterion_id ) || 
		    ! setCriterionOperator( criterion_id, operator ) ) {
			success = false;
		} else {
			
		}
		//alert( field );
	}

	updatePlaylist();

	return success;
}
*/
/**
 * set a criterion's field by field name
 *
 * @param string the id of the criterion to set
 * @param string the field to select
 *
 * @return bool success
 */
function setCriterionField( id, field ) {
	// get the criterion field select box
	var criterion_select = document.getElementById( id + 'p' + criteria.PART_FIELD + 's' );
	
	if( ! criterion_select || ! criterion_select.options ) {
		return false;
	}

	// find the option and select it
	for( i in criterion_select.options ) {
		if( criterion_select.options[i].value.toLowerCase() == field.toLowerCase() ) {
			criterion_select.options[i].selected = true;
			return true;
		}
	}

	return false;
}

/**
 * set a criterion's operator by value
 *
 * @param string the id of the criterion to set
 * @param string the operator value to select
 *
 * @return bool success
 */
function setCriterionOperator( id, operator ) {
	// get the criterion field select box
	var criterion_select = document.getElementById( id + 'p' + criteria.PART_OPERATOR + 's' );
	
	if( ! criterion_select || ! criterion_select.options ) {
		return false;
	}
	
	// find the option and select it
	for( i in criterion_select.options ) {
		if( typeof criterion_select.options[i] == "object" && 
		    criterion_select.options[i].value.toLowerCase() == operator.toLowerCase() ) {
			criterion_select.options[i].selected = true;
			return true;
		}
	}
	
	return false;
}

/**
 * reset the target (probably a <textarea>) with the original playlist text and close the popup
 */
/*
function cancel() {
	// re-set the old playlist value
	target.value = old_value;
	window.opener && window.close();
}
*/
/**
 * update the target with the playlist text and close the window
 */
/*
function save() {
	updatePlaylist();
	window.opener && window.close();
}*/
