<?php
/*
 * jQuery File Upload Plugin PHP Example 5.2.2
 * https://github.com/blueimp/jQuery-File-Upload
 *
 * Copyright 2010, Sebastian Tschan
 * https://blueimp.net
 *
 * Licensed under the MIT license:
 * http://creativecommons.org/licenses/MIT/
 */

class UploadHandler
{
    private $options;

    function __construct($options=null) {
        $this->options = array(
            'script_url' => $_SERVER['PHP_SELF'],
            'upload_dir' => dirname(__FILE__).'/files/',
            'upload_url' => dirname($_SERVER['PHP_SELF']).'/files/',
            'param_name' => 'files',
            // The php.ini settings upload_max_filesize and post_max_size
            // take precedence over the following max_file_size setting:
            'max_file_size' => null,
            'min_file_size' => 1,
            'accept_file_types' => '/.+$/i',
            'discard_aborted_uploads' => true
        );
        if ($options) {
            $this->options = array_merge($this->options, $options);
        }
    }

    private function has_error($uploaded_file, $file, $error) {
        if ($error) {
            return $error;
        }
        if (!preg_match($this->options['accept_file_types'], $file->name)) {
            return 'acceptFileTypes';
        }
        if ($uploaded_file && is_uploaded_file($uploaded_file)) {
            $file_size = filesize($uploaded_file);
        } else {
            $file_size = $_SERVER['CONTENT_LENGTH'];
        }
        if ($this->options['max_file_size'] && (
                $file_size > $this->options['max_file_size'] ||
                $file->size > $this->options['max_file_size'])
            ) {
            return 'maxFileSize';
        }
        if ($this->options['min_file_size'] &&
            $file_size < $this->options['min_file_size']) {
            return 'minFileSize';
        }
        return $error;
    }

    private function get_upload_dir($index, $filename) {
        if ( count($this->options['upload_dir']) == 1 ) {
            $path = $this->options['upload_dir'][0];
        } else if ( isset($this->options['upload_dir'][$index] ) ) {
            $path = $this->options['upload_dir'][$index];
        } else {
            return false;
        }
        return $path.$filename;
    }

    private function handle_file_upload($index, $uploaded_file, $name, $size, $type, $error) {
        $file = new stdClass();
        $file->name = basename(stripslashes($name));
        $file->size = intval($size);
        $file->type = $type;
        $error = $this->has_error($uploaded_file, $file, $error);

        if (!$error && $file->name) {
            if ($file->name[0] === '.') {
                $file->name = substr($file->name, 1);
            }

            $file_path = $this->get_upload_dir($index, $file->name);
            if (!$file_path) {
                $file->error = 'abort';
                return $file;
            }

            $count = 1;
            $file_path_new = $file_path;
            $suffix = strrchr($file_path, '.');
            $filename = str_replace($suffix, '', $file_path);
			while(file_exists($file_path_new)){
                if($suffix !== false){
                    $file_path_new = $filename.'-'.$count++.$suffix;
                }else{
                    $file_path_new = $filename.'.copy'.$count;
                    break;
                }
			}
            $file_path = $file_path_new;

            $append_file = is_file($file_path) && $file->size > filesize($file_path);
            clearstatcache();
            if ($uploaded_file && is_uploaded_file($uploaded_file)) {
                // multipart/formdata uploads (POST method uploads)
                if ($append_file) {
                    file_put_contents(
                        $file_path,
                        fopen($uploaded_file, 'r'),
                        FILE_APPEND
                    );
                } else {
                    if(@move_uploaded_file($uploaded_file, $file_path)){
						chmod($file_path,0666);
					}
					else{
						if(file_exists($uploaded_file)){
							unlink($uploaded_file);
                        }
					}
                }
            } else {
                // Non-multipart uploads (PUT method support)
                file_put_contents(
                    $file_path,
                    fopen('php://input', 'r'),
                    $append_file ? FILE_APPEND : 0
                );
            }
            $file_size = @filesize($file_path);
            if ($file_size === $file->size) {
                $file->url = $this->options['upload_url'].rawurlencode($file->name);
            } else if ($this->options['discard_aborted_uploads']) {
                @unlink($file_path);
                $file->error = 'abort';
            }
            $file->size = $file_size;
            $file->path = $file_path;
            // $file->delete_url = $this->options['script_url']
            //     .'?file='.rawurlencode($file->name);
            // $file->delete_type = 'DELETE';
        } else {
            $file->error = $error;
        }
        return $file;
    }

    public function post() {

        $upload = isset($_FILES[$this->options['param_name']]) ?
            $_FILES[$this->options['param_name']] : array(
                'tmp_name' => null,
                'name' => null,
                'size' => null,
                'type' => null,
                'error' => null
            );
        $info = array();

        if (is_array($upload['tmp_name'])) {
            foreach ($upload['tmp_name'] as $index => $value) {

            	$ErrorObj = $upload['error'][$index];

                $info[] = $this->handle_file_upload(
                    $index,
                    $upload['tmp_name'][$index],
                    isset($_SERVER['HTTP_X_FILE_NAME']) ? $_SERVER['HTTP_X_FILE_NAME'] : $upload['name'][$index],
                    isset($_SERVER['HTTP_X_FILE_SIZE']) ? $_SERVER['HTTP_X_FILE_SIZE'] : $upload['size'][$index],
                    isset($_SERVER['HTTP_X_FILE_TYPE']) ? $_SERVER['HTTP_X_FILE_TYPE'] : $upload['type'][$index],
                    $ErrorObj
                );
            }
        } else {
             $ErrorObj = $upload['error'];

            $info[] = $this->handle_file_upload(
                0,
                $upload['tmp_name'],
                isset($_SERVER['HTTP_X_FILE_NAME']) ? $_SERVER['HTTP_X_FILE_NAME'] : $upload['name'],
                isset($_SERVER['HTTP_X_FILE_SIZE']) ? $_SERVER['HTTP_X_FILE_SIZE'] : $upload['size'],
                isset($_SERVER['HTTP_X_FILE_TYPE']) ? $_SERVER['HTTP_X_FILE_TYPE'] : $upload['type'],
                $ErrorObj
            );
        }
        return $info;

    }
}

?>