#!/bin/sh
#
# This script does initialization if necessary and launches mysqld_safe.
#
# 1. Initialize mysql tables if not initialized yet
# 2. Change root password to 'qnapqnap' if not changed yet
# 3. Create user 'read' with password 'read' if not exists
# 4. Launch mysqld_safe, "which adds some safety features such as restarting 
#    the server when an error occurs"
#

if [ $# -ne 2 ]; then
	echo "Usage: $0 basedir datadir"
	exit 1
fi

#MEDIALIBRARY_INSTALL="/home/claus/cvsroot/toys/MediaLibrary/usr/local/medialibrary"
#MYSQL_BASE=$MEDIALIBRARY_INSTALL/mysql
MYSQL_BASE=$1
MYSQLD_SAFE=$MYSQL_BASE/bin/mysqld_safe
MYSQL_INSTALL_DB=$MYSQL_BASE/scripts/mysql_install_db
MYSQL_ADMIN=$MYSQL_BASE/bin/mysqladmin
MYSQL_CLIENT=$MYSQL_BASE/bin/mysql
#MYSQL_DATA=/tmp/data
MYSQL_DATA=$2/data
MYSQL_TMP=$2/tmp
MYSQL_OPTFILE=${MYSQL_OPTFILE:-$MYSQL_BASE/my-mariadb.cnf}
MYSQL_SOCKET=${MYSQL_SOCKET:-"/tmp/mysql_mediadb.sock"}
MYSQL_PORT=3310
MYSQL_PIDFILE=${MYSQL_PIDFILE:-"/var/lock/mariadb.pid"}

#test -d $MEDIALIBRARY_INSTALL 	|| ( echo "$MEDIALIBRARY_INSTALL not found";  exit 1 )
#test -d $MYSQL_BASE 			|| ( echo "$MYSQL_BASE not found"; exit 1 )
[ ! -d $MYSQL_BASE ] && echo "$MYSQL_BASE not found" && exit 1 

for i in "$MYSQL_INSTALL_DB" "$MYSQLD_SAFE" "$MYSQL_ADMIN" "$MYSQL_CLIENT" "$MYSQL_OPTFILE" ; do
	[ ! -f $i ] && echo "$i not found" && exit 1
done

LIBTINFO="/usr/lib/libtinfo.so.5" # required for 64-ALPHA
[ -e "${LIBTINFO}" ] || ln -sf libncurses.so.5 "${LIBTINFO}"

cd $MYSQL_BASE

add_user() {
	COUNTER=1
	CHECK_MYSQL_SERVER=1
	# Waiting for mysqld to start up
	while [  $COUNTER -lt 50 ] && [ $CHECK_MYSQL_SERVER -ne 0 ]; do
		echo $COUNTER
		#$MYSQL_ADMIN --socket=$MYSQL_SOCKET --port=$MYSQL_PORT ping 2>&1 >/dev/null
		$MYSQL_ADMIN --defaults-file=$MYSQL_OPTFILE  ping 2>&1 >/dev/null
		CHECK_MYSQL_SERVER=$?
		let COUNTER+=1	
		sleep 1
	done
	echo $COUNTER

	if [ $CHECK_MYSQL_SERVER -ne 0 ]; then
		echo "mysqld is not alive"
		exit 1
	fi

	$MYSQL_ADMIN --defaults-file=$MYSQL_OPTFILE -uroot password 'qnapqnap' --skip-password

	$MYSQL_CLIENT --defaults-file=$MYSQL_OPTFILE -uroot -pqnapqnap -e"GRANT SELECT on *.* to 'read'@'localhost' identified by 'read'; FLUSH PRIVILEGES"
	$MYSQL_CLIENT --defaults-file=$MYSQL_OPTFILE -uroot -pqnapqnap -e"GRANT SELECT on *.* to 'read'@'%' identified by 'read'; FLUSH PRIVILEGES"	
}

[ ! -d $MYSQL_DATA ] && mkdir -p $MYSQL_DATA
[ ! -d $MYSQL_TMP ]  && mkdir -p $MYSQL_TMP

if [ ! -d $MYSQL_DATA/mysql ]; then
	echo Running mysql_install_db...
	$MYSQL_INSTALL_DB --defaults-file=$MYSQL_OPTFILE --basedir=$MYSQL_BASE --datadir=$MYSQL_DATA
fi	

echo Running mysqld_safe...
$MYSQLD_SAFE --defaults-file=$MYSQL_OPTFILE --basedir=$MYSQL_BASE --datadir=$MYSQL_DATA --user=$USER --default-storage-engine=MyISAM --wait_timeout=2628000 --pid-file=$MYSQL_PIDFILE --tmpdir=$MYSQL_TMP &

add_user

exit 0
