<?php

require_once(__DIR__ . "/inc/url_parser.inc.php");
require_once(__DIR__ . "/inc/extern_proc.inc.php");
require_once(__DIR__ . "/playitem.php");
require_once(__DIR__ . "/CommonDevice.inc.php");
require_once(__DIR__ . "/hls.inc.php");

class AirPlayDevice extends CommonDevice {

    public static $_AP_HTTPHEADERS = array("Cache-Control: no-cache", "Connection: Keep-Alive", "Origin: airplay", "User-Agent: MediaControl/1.0", "Content-Type: application/xml", "Accept: */*");
    var $last_heartbeat;
    var $duration = 0.0;
    var $position = 0.0;
    var $connected = 0;
    var $started = 0;
    var $stopped = 1;
    var $completed = 0;
    var $pl_md5;
    var $playlist = array();
    var $current_item;
    var $idx_item;
    var $last_refresh_status = 0;
    var $ch;
    var $http_status;
    var $airplay_pwd;
    var $auth_confirmed = 0;

    function __construct($opt, $dev_version) {
        parent::__construct($opt, $dev_version);
        $this->protovers = $opt["protovers"];
        $this->hls = new hls_impl();
        $this->hls->device_id = $this->device_id;
        $this->hls->teardown();
    }

    function close_curl() {
        if (isset($this->ch)) {
            curl_close($this->ch);
            unset($this->ch, $this->http_status);
        }
    }

    function init_curl($force_rebuild = FALSE) {
        if ($force_rebuild and isset($this->ch)) {
            $ch = curl_copy_handle($this->ch);
            curl_close($this->ch);
            $this->ch = $ch;
        } if (!isset($this->ch)) {
            $this->ch = curl_init();
            $opt = array(CURLOPT_BINARYTRANSFER => TRUE, CURLOPT_FOLLOWLOCATION => TRUE, CURLOPT_AUTOREFERER => TRUE, CURLOPT_TIMEOUT => 4, CURLOPT_HTTPHEADER => self::$_AP_HTTPHEADERS);
            if (isset($this->airplay_pwd)) {
                $opt[CURLOPT_HTTPAUTH] = CURLAUTH_DIGEST;
                $opt[CURLOPT_USERPWD] = "AirPlay:{$this->airplay_pwd}";
            } 
            curl_setopt_array($this->ch, $opt);
            $this->auth_confirmed = 0;
        }
    }

    function set_airplay_pwd($pwd, &$r_pwd) {
        $this->build_connection(1);
        curl_setopt_array($this->ch, array(CURLOPT_HTTPAUTH => CURLAUTH_DIGEST, CURLOPT_USERPWD => "AirPlay:{$pwd}"));
        $this->refresh_status();
        if ($this->http_status == 200) {
            $this->airplay_pwd = $pwd;
            $r_pwd = ok();
            return 0;
        } else {
            $this->close_curl();
            if ($this->http_status == 401) {
                $r_pwd = error_appletv_need_auth();
                return -1;
            } 
            $r_pwd = error_appletv_http();
            return -1;
        }
    }

    function close() {
        echo "AirPlayDevice::close()\n";
        $this->close_curl();
        $this->last_heartbeat = null;
        $this->duration = $this->position = 0.0;
        $this->stopped = 1;
        $this->connected = $this->started = 0;
        $this->hls_teardown($r_v);
    }

    function keep_alive() {
        $now = microtime(true);
        if (is_null($this->last_heartbeat) || ($now - $this->last_heartbeat) >= 4.0) {
            $this->refresh_status();
        }
    }

    function build_connection($force_rebuild = 0) {
        if (!isset($this->ch) or $force_rebuild) {
            $this->init_curl($force_rebuild);
            $this->connected = 1;
            $this->completed = 0;
            $this->started = 0;
            $this->stopped = 1;
            unset($this->play_time);
        } 
        return 0;
    }

    public function tick() {
        $check = microtime(true) - 10;
        if (!isset($this->last_heartbeat) or $this->last_heartbeat > $check) {
            $this->refresh_status();
            return 0;
        } 
        return 1;
    }

    public function refresh_status() {
        if (!isset($this->ch))
            return 1;
        $this->build_connection(0);
        curl_setopt_array($this->ch, array(CURLOPT_RETURNTRANSFER => TRUE, CURLOPT_URL => "http://{$this->svc_host}:{$this->svc_port}/scrub", CURLOPT_HTTPGET => 1));
        $r = curl_exec($this->ch);
        $this->http_status = curl_getinfo($this->ch, CURLINFO_HTTP_CODE);
        curl_setopt($this->ch, CURLOPT_RETURNTRANSFER, FALSE);
        if ($this->http_status != 200)
            return -1;
        $this->last_refresh_status = 0;
        $i = strpos($r, "duration:");
        if (is_int($i)) {
            $dur = (float) substr($r, $i + 10);
        } else {
            $this->last_refresh_status = -1;
            return 0;
        } 
        $i = strpos($r, "position:");
        if (is_int($i)) {
            $pos = (float) substr($r, $i + 9);
        } 
        if (!isset($dur) or ! isset($pos))
            return -1;
        if ($dur == 0) {
            if ($this->duration == 0) {
                return -1;
            } 
            if (abs($this->duration - $this->position) > 2.0) {
                
            } 
            $this->stopped = $this->completed = 1;
            $this->last_heartbeat = microtime(true);
            $this->auth_confirmed = 1;
            return 0;
        } if ($pos > 0) {
            $this->started = 1;
        } 
        $end_time = $dur - 1.0;
        $stopped = ($pos === $this->position) ? 1 : 0;
        $completed = ($stopped AND $pos >= $end_time) ? 1 : 0;
        $this->stopped = $stopped;
        $this->completed = $completed;
        $this->duration = $dur;
        $this->position = $pos;
        $this->last_heartbeat = microtime(true);
        if (0 == $this->loaded && 0 == $this->load_failed) {
            if ($dur > 0) {
                $this->loaded = 1;
                $this->load_failed = 0;
                $this->loaded_time = microtime(TRUE);
            } elseif (microtime(TRUE) > ($this->play_time + 10.0)) {
                $this->loaded = 0;
                $this->load_failed = 1;
            }
        } 
        $this->auth_confirmed = 1;
        return 0;
    }

    public function scrub($pos, &$r_scrub) {
        $this->build_connection(0);
        curl_setopt_array($this->ch, array(CURLOPT_URL => "http://{$this->svc_host}:{$this->svc_port}/scrub?position={$pos}", CURLOPT_POST => 1, CURLOPT_POSTFIELDS => ""));
        curl_exec($this->ch);
        $this->http_status = curl_getinfo($this->ch, CURLINFO_HTTP_CODE);
        if ($d->http_status == 401) {
            $r_scrub = error_appletv_need_auth();
            return -1;
        } 
        
        if ($this->http_status != 200) {
            $r_scrub = error_appletv_http();
            return -1;
        } 
        $r_scrub = ok();
        return 0;
    }

    public function stop(&$r_stop = 0) {
        $this->build_connection(0);
        $this->duration = $this->position = 0;
        curl_setopt_array($this->ch, array(CURLOPT_URL => "http://{$this->svc_host}:{$this->svc_port}/stop", CURLOPT_POST => 1, CURLOPT_POSTFIELDS => ""));
        curl_exec($this->ch);
        $this->http_status = curl_getinfo($this->ch, CURLINFO_HTTP_CODE);
        if ($this->http_status != 200)
            return -1;
        unset($this->current_item);
        $this->tick_suspended = TRUE;
        $r_stop = ok();
        return 0;
    }

    public function play_photo_url($url, $transition) {
        $r = explode("?", $url);
        if (!is_array($r) or count($r) != 2)
            return $this->play_photo_url_real($url, $transition);
        parse_str_ex($r[1], $q);
        if (!is_array($q) or ! isset($q['path']))
            return $this->play_photo_url_real($url, $transition);
        return $this->play_photo("/share/{$q['path']}", $transition);
    }

    function play_photo_url_real($url, $transition) {
        $photo_path = tempnam("/tmp/", "airplay_temp_photo_");
        $esc_url = escapeshellarg($url);
        $esc_path = escapeshellarg($photo_path);
        $r = shell_exec("wget $esc_url -O $esc_path");
        $len = filesize($photo_path);
        if ($len > 0) {
            $r = $this->play_photo($photo_path, $transition);
        } else {
            $r = -1;
        } 
        unlink($photo_path);
        return $r;
    }

    function play_photo($photo_path, $transition) {
        echo "play_photo($photo_path, $transition)\n";
        $this->loaded = $this->load_failed = $this->loaded_time = 0;
        $this->build_connection(0);
        $this->duration = $this->position = 0;
        $tmap = array("none" => "None", "left" => "SlideLeft", "right" => "SlideRight", "dissolve" => "Dissolve");
        $t = isset($tmap[$transition]) ? $tmap[$transition] : "None";
        $len = filesize($photo_path);
        $infile = fopen($photo_path, "r");
        $hdr = self::$_AP_HTTPHEADERS;
        $hdr[] = "X-Apple-Transition: $t";
        curl_setopt_array($this->ch, array(CURLOPT_BINARYTRANSFER => 1, CURLOPT_TIMEOUT => 30, CURLOPT_CONNECTTIMEOUT => 10, CURLOPT_URL => "http://{$this->svc_host}:{$this->svc_port}/photo", CURLOPT_HTTPHEADER => $hdr, CURLOPT_PUT => 1, CURLOPT_INFILESIZE => $len, CURLOPT_INFILE => $infile,));
        curl_exec($this->ch);
        curl_setopt_array($this->ch, array(CURLOPT_HTTPHEADER => self::$_AP_HTTPHEADERS, CURLOPT_TIMEOUT => 4));
        $this->http_status = curl_getinfo($this->ch, CURLINFO_HTTP_CODE);
        if ($this->http_status != 200) {
            $err = curl_errno($this->ch);
            $err_str = curl_error($this->ch);
            fputs(STDERR, "fail play_photo({$this->http_status},{$r},{$err},{$err_str})\n");
            $this->loaded = 0;
            $this->load_failed = 1;
            return -1;
        } 
        $this->loaded = 1;
        $this->load_failed = 0;
        return 0;
    }

    function hls_teardown(&$r_v = 0) {
        return $this->hls->teardown($r_v);
    }

    function hls_start($src, &$r_v = 0) {
        return $this->hls->start($src, $r_v);
    }

    function hls_seek($seg, &$r_v = 0) {
        $r = $this->hls->seek($seg, $r_v);
        return $r;
    }

    function hls_status(&$r_v = 0) {
        return $this->hls->status($r_v);
    }

    function hls_play($src, $seekable, &$r_v = 0) {
        $src = parse_path_from_fileurl($src, $path);
        $r = $this->hls->start($src, $r_v);
        if ($r < 0) {
            return $r;
        } 
        $dev = $this;
        $retry_left = 20;
        dispatch(0, function($self) use($dev, $retry_left, $seekable) {
            $r = $dev->hls->status($r_status);
            if ($r < 0 or ( $r >= 0 and $dev->hls->duration <= 0 and count($r_status["segments"]) <= 0)) {
                $retry_left--;
                if ($retry_left <= 0)
                    return;
                dispatch(2, $self);
                return;
            } 
            if ($dev->hls->duration <= 0 or ! $seekable) {
                $dev->play($r_status["m3u8"]);
            } else {
                $dev->play($r_status["seekable_m3u8"]);
            }
        });
        return $r;
    }

    public function status(&$r_status) {
        $this->refresh_status();
        if ($this->http_status == 401) {
            $r_status = error_appletv_need_auth();
            return 0;
        } if ($this->last_refresh_status != 0) {
            $r_status = error_communicating_device();
            return 0;
        } 
        $dur = $this->duration;
        $pos = $this->position;
        $r_status = array("status" => 0, "duration" => floatval(sprintf("%.2f", $dur)), "position" => floatval(sprintf("%.2f", $pos)),);
        parent::status($r_status);
        return 0;
    }

    public function play($url, &$r_play = 0) {
        $svc_url = "http://{$this->svc_host}:{$this->svc_port}/play";
        $this->build_connection(1);
        $post_data = "Content-Location: $url\r\nStart-Position: 0.0\r\n";
        curl_setopt_array($this->ch, array(CURLOPT_URL => "http://{$this->svc_host}:{$this->svc_port}/play", CURLOPT_POST => 1, CURLOPT_POSTFIELDS => $post_data));
        curl_exec($this->ch);
        $this->http_status = curl_getinfo($this->ch, CURLINFO_HTTP_CODE);
        if ($this->http_status == 200) {
            $this->duration = 0;
            $this->position = 0;
            $this->started = 1;
            $this->stopped = 0;
            $this->completed = 0;
            $this->play_time = microtime(TRUE);
            $this->loaded = $this->load_failed = 0;
            $r_play = ok();
            return 0;
        } 
        if ($this->http_status == 401) {
            $r_play = error_appletv_need_auth();
            return -1;
        } 
        $r_play = make_resp(-1, "unknown error");
        return -1;
    }

    function setRate($rate, &$r_rate) {
        $this->build_connection();
        curl_setopt_array($this->ch, array(CURLOPT_URL => "http://{$this->svc_host}:{$this->svc_port}/rate?value={$rate}", CURLOPT_POST => 1, CURLOPT_POSTFIELDS => ""));
        curl_exec($this->ch);
        $this->http_status = curl_getinfo($this->ch, CURLINFO_HTTP_CODE);
        if ($this->http_status == 401) {
            $r_rate = error_appletv_need_auth();
            return -2;
        } 
        if ($rate == 1.0) {
            $this->tick_suspended = FALSE;
            $this->stopped = 0;
        } else if ($rate == 0.0) {
            $this->tick_suspended = TRUE;
            $this->stopped = 1;
        } else {
            $r_rate = error_parameter();
            return -3;
        } 
        if ($this->http_status != 200 and $this->http_status != 500) {
            $r_rate = error_appletv_http();
            return -4;
        } 
        $r_rate = ok();
        return 0;
    }
}
