<?php

require_once(__DIR__ . "/inc/extern_proc.inc.php");
require_once(__DIR__ . "/CommonDevice.inc.php");

class cc_handler extends extern_proc {

    var $dev;
    var $media_waiting = 0;
    var $media_mime_waiting = 0;
    var $media_image_url_waiting = 0;
    var $media_title_waiting = 0;
    var $playing_url;
    var $launched = 0;
    var $parse_count = 0;

    function __destruct() {
        echo "extern_proc destructor\n";
        $this->close();
    }

    function close() {
        $pc = $this->parse_count;
        echo "[cc_handler::close()] ***cc_handler::close()\n";
        flush();
        if (!isset($this->pipes))
            return;
        fwrite($this->pipes[0], "CLOSE\nQUIT\n");
        parent::close(0);
        echo "*** exitCode: {$this->rval}\n";
    }

    function stop() {
        echo "[PHP][cc_handler::stop()] STOP\n";
        fwrite($this->pipes[0], "STOP\n");
        fflush($this->pipes[0]);
    }

    function pause() {
        echo "[PHP][cc_handler::pause()] PAUSE\n";
        fwrite($this->pipes[0], "PAUSE\n");
        fflush($this->pipes[0]);
    }

    function play() {
        echo "[PHP][cc_handler::close()] PLAY\n";
        fwrite($this->pipes[0], "PLAY\n");
        fflush($this->pipes[0]);
    }

    function status() {
        echo "[cc_handler::status()] STATUS\n";
        fwrite($this->pipes[0], "STATUS\n");
        fflush($this->pipes[0]);
    }

    function load_media($url, $mime = "video/mp4", $image_url, $title) {
        echo "[PHP][cc_handler::load_media()] url:$url  mime:$mime image_url:$image_url title:$title\n";

        if (!$this->launched) {
            $this->media_waiting = $url;
            $this->media_mime_waiting = $mime;
            $this->media_image_url_waiting = $image_url;
            $this->media_title_waiting = $title;
        } else {
            echo "[cc_handler::load_media()] LOAD\n{$url}\n{$mime}\n{$image_url}\n$title\n";
            fwrite($this->pipes[0], "LOAD\n{$url}\n{$mime}\n{$image_url}\n$title\n");
            fflush($this->pipes[0]);
        }
    }

    function parse_output($s, $type) {
        $this->parse_count++;
        $s = trim($s);
        echo "[PHP][cc_handler::parse_output()] s:$s   type:$type\n";

        if ($type == 2) {
            return;
        }
        if ($s == "STATUS:DEVICE READY") {
            echo "RESP({$type}):{$s}:\n";
            fwrite($this->pipes[0], "LAUNCH\n");
        } elseif ($s == "STATUS:LAUNCH COMPLETED") {
            $this->launched = 1;
            echo "RESP({$type}):{$s}:\n";
            $this->dev->player_state = ChromecastDevice::PLAY_E;
            if (0 !== $this->media_waiting) {
                $this->load_media($this->media_waiting, $this->media_mime_waiting, $this->media_image_url_waiting, $this->media_title_waiting);
                $this->media_waiting = $this->media_mime_waiting = 0;
            }
        } elseif ("STATUS:FINISHED" == $s) {
            $this->dev->completed = 1;
            $this->dev->duration = 0;
            $this->dev->player_state = ChromecastDevice::STOP_E;
            echo "RESP({$type}):{$s}:\n";
        } elseif ("PLAYER STATE:IDLE" == $s) {
            $this->dev->player_state = ChromecastDevice::STOP_E;
            echo "RESP({$type}):{$s}:\n";
        } elseif ("PLAYER STATE:BUFFERING" == $s) {
            $this->dev->player_state = ChromecastDevice::PLAY_E;
            echo "RESP({$type}):{$s}:\n";
        } elseif ("PLAYER STATE:BUFFERING" == $s) {
            $this->dev->player_state = ChromecastDevice::PAUSE_E;
            echo "RESP({$type}):{$s}:\n";
        } elseif ("STATUS:LOADED" == $s) {
            $this->dev->loaded = 1;
            $this->dev->load_failed = 0;
            $this->dev->loaded_time = microtime(TRUE);
            $this->dev->player_state = ChromecastDevice::PLAY_E;
            echo "RESP({$type}):{$s}:\n";
        } elseif ("STATUS:LOAD FAILED" == $s) {
            $this->dev->loaded = 0;
            $this->dev->load_failed = 1;
            $this->dev->player_state = ChromecastDevice::STOP_E;
            echo "RESP({$type}):{$s}:\n";
        } elseif (FALSE !== strpos($s, "DURATION:")) {
            $v = substr($s, 9);
            $this->dev->duration = intval($v);
            echo "RESP({$type}):{$s}:\n";
        } elseif (FALSE !== strpos($s, "CURRENT TIME:")) {
            $v = substr($s, 13);
            $this->dev->position = intval($v);
            echo "RESP({$type}):{$s}:\n";
        } elseif ($s == "STATUS:CLOSED") {
            $this->launched = $this->media_waiting = $this->media_mime_waiting = 0;
            $this->dev->player_state = ChromecastDevice::STOP_E;
            echo "RESP({$type}):{$s}:\n";
        } elseif ($s == "STATUS:TERMINATED") {
            $this->dev->player_state = ChromecastDevice::STOP_E;
            echo "RESP({$type}):{$s}:\n";
        }
        flush();
    }
}

class ChromecastDevice extends CommonDevice {

    const PLAY_E = 0,
          PAUSE_E = 1,
          STOP_E = 2,
          UNKNOW_E = 4;

    var $player_state = ChromecastDevice::STOP_E;
    var $duration = 0.0;
    var $position = 0.0;
    var $completed = 0;

    function __construct($opt, $dev_version) {
        $this->player_state = ChromecastDevice::STOP_E;
        parent::__construct($opt, $dev_version);
    }

    function __destruct() {
        $this->close();
    }

    public function keep_alive() {
        
    }

    public function tick() {
        if (isset($this->cc_handler)) {
            $this->cc_handler->tick();
            return 0;
        }
        return 1;
    }

    var $cc_handler;

    public function close() {
        echo ">>>[ChromecastDevice::close()]\n";
        if (isset($this->cc_handler)) {
            $this->cc_handler->close(0);
            unset($this->cc_handler);
            return 0;
        }
        return 1;
    }

    function connect($force_rebuild = 0) {
        echo ">>>[ChromecastDevice::connect()]\n";
        if ($force_rebuild) {
            $this->close();
        }
        if (isset($this->cc_handler) and ! $this->cc_handler->isRunning()) {
            unset($this->cc_handler);
        }
        if (!isset($this->cc_handler)) {
            $this->cc_handler = new cc_handler;
            $this->cc_handler->dev = $this;
            $this->cc_handler->open(CC_HANDLER . " {$this->svc_host}");
        }
        return 0;
    }

    function setRate($rate, &$r_rate) {
        echo ">>>[ChromecastDevice::setRate()]\n";
        if (!isset($this->cc_handler)) {
            $r_rate = error_not_loaded();
            return -1;
        }
        if ($rate == 1.0) {
            $this->player_state = ChromecastDevice::PLAY_E;
            $this->cc_handler->play();
            $this->tick_suspended = FALSE;
            $this->stopped = 0;
        } else if ($rate == 0.0) {
            $this->player_state = ChromecastDevice::PAUSE_E;
            $this->cc_handler->pause();
            $this->tick_suspended = TRUE;
            $this->stopped = 1;
        } else {
            $r_rate = error_parameter();
            return -2;
        }
        $r_rate = ok();
        return 0;
    }

    public function stop(&$r_stop = 0) {
        echo ">>>[ChromecastDevice::stop()]\n";
        $this->player_state = ChromecastDevice::STOP_E;

        if (isset($this->cc_handler)) {
            $this->cc_handler->stop();
            $r_stop = ok();
            return 0;
        }
        $r_stop = ok();
        return 1;
    }

    public function play_photo_url($url, $transition) {
        echo ">>>[ChromecastDevice::play_photo_url()] url:$url\n";
        $this->playing_url = $url;
        $this->completed = $this->duration = $this->position = 0;
        $this->connect();
        $this->cc_handler->load_media($url, "image/jpeg");
        return 0;
    }

    public function play($url, &$r_play, &$f_resp, $image_url, $title) {
        echo ">>>[ChromecastDevice::play()] url:$url\n";
        $this->playing_url = $url;
        $this->completed = $this->duration = $this->position = 0;
        $this->loaded = $this->load_failed = $this->loaded_time = 0;
        $this->connect();

        // Archer 20150225
        // if the media type is not fulling in following type
        // then the mime to application/x-mpegurl
        $mime_type = "";
        /*if ((strstr($url, "mime=video/mp4") === false) &&
            (strstr($url, "mime=video/x-matroska") === false) &&
            (strstr($url, "mime=video/quicktime") === false) &&
            (strstr($url, "mime=video/3gpp") === false) &&
            (strstr($url, "mime=image") === false) &&
            (strstr($url, "mime=audio") === false)) {
            $mime_type = "application/x-mpegurl";
        }*/
        if ((strstr($url, "CHROMECAST") !== false) &&
            (strstr($url, "mime=video") !== false)) {
            $mime_type = "application/x-mpegurl";
        }else if((strstr($url, "Resized") !== false)) {
	    $mime_type = "image/jpeg";
	}else {
            $pos = strpos($url, "mime=");
            if ($pos !== false) {
                $pos = $pos + 5;
                $posEnd = strpos($url, ",", $pos);
                $mime_type = substr($url, $pos, $posEnd - $pos);
            }
        }

        //$this->cc_handler->load_media($url, "video/mp4");
        $this->cc_handler->load_media($url, $mime_type, $image_url, $title);
        $this->player_state = ChromecastDevice::PLAY_E;
        $r_play = ok();

        return 0;
    }

    public function scrub($pos, &$r_scrub) {
        echo ">>>[ChromecastDevice::scrub()]\n";
        $r_scrub = unsupported_command();

        return 0;
    }

    public function status(&$r_status) {
        echo ">>>[ChromecastDevice::status()]\n";
        if (isset($this->cc_handler)) {
            if ($this->player_state == ChromecastDevice::PLAY_E) {
                $this->cc_handler->play();
                //$this->cc_handler->status();
            } elseif ($this->player_state == ChromecastDevice::PAUSE_E) {
                $this->cc_handler->pause();
                //$this->cc_handler->status();
            } else {
                $this->cc_handler->status();
            }
            //$this->cc_handler->status();
        }
        
        if ($this->dev->player_state == ChromecastDevice::PLAY_E) {
            $state = "PLAY";
        } elseif ($this->dev->player_state == ChromecastDevice::PAUSE_E) {
            $state = "PAUSE";
        } elseif ($this->dev->player_state == ChromecastDevice::STOP_E) {
            $state = "STOP";
        } else {
            $state = "UNKNOW";
        }
        $url = isset($this->playing_url) ? $this->playing_url : NULL;
        $pos = isset($this->position) ? floatval($this->position) : NULL;
        $dur = isset($this->duration) ? floatval($this->duration) : NULL;
        $r_status = array("status" => 0, "player_state" => $state, "playing_url" => $url, "position" => $pos, "duration" => $dur,);
        var_dump($r_status);
        parent::status($r_status);

        return 0;
    }

    public function notify_status($opt, &$r_notify) {
        echo ">>>[ChromecastDevice::notify_status()]\n";
        $pos = isset($opt->pos) ? floatval($opt->pos) : NULL;
        $dur = isset($opt->dur) ? floatval($opt->dur) : NULL;
        $this->position = $pos;
        $this->duration = $dur;
        if ($dur > 0 and $pos === $dur) {
            $this->completed = 1;
        }
        $r_notify = ok();

        return 0;
    }

}

class ChromecastDiscoverySvc {

    var $sock;
    var $f_on_new_device;

    function init() {
        if (isset($this->sock))
            return 1;
        global $eth_list;

        $socks = array();
        foreach ($eth_list as $eth => $ip) {
            $s = socket_create(AF_INET, SOCK_DGRAM, SOL_UDP);
            socket_bind($s, $ip);
            $socks[$eth] = $s;
        }
        $this->sock = $socks;

        return 0;
    }

    function close() {
        if (!isset($this->sock))
            return 1;
        foreach ($this->sock as $s)
            socket_close($s);
        unset($this->sock);

        return 0;
    }

    function send_ssdp_request() {
        $this->init();
        $req = array("M-SEARCH * HTTP/1.1", "HOST:239.255.255.250:1900", "MAN:\"ssdp:discover\"", "MX:2", "ST:urn:dial-multiscreen-org:service:dial:1", "USER-AGENT:Google Chrome/32.0.1700.76 Windows");
        $req = implode("\r\n", $req) . "\r\n\r\n";

        foreach ($this->sock as $s) {
            $r_send = socket_sendto($s, $req, strlen($req), 0, "239.255.255.250", 1900);
        }

        return 0;
    }

    function on_selected($sock, $param_eth) {
        $this->init();
        $r_recv = socket_recvfrom($sock, $buf, 9999, 0, $name, $port);
        $r_str = strpos($buf, "ST: urn:dial-multiscreen-org:service:dial:1");
        if ($r_str == FALSE)
            return -1;
        $r_match = preg_match("/LOCATION: *(.*)\r\n/", $buf, $m);
        if ($r_match < 1)
            return -1;
        list(, $loc) = $m;
        $cmd = implode(" ", array(CURL_PATH, escapeshellarg($loc), "2>/dev/null"));
        $r_loc = shell_exec($cmd);
        if (!is_string($r_loc))
            return -1;
        $r_xml = simplexml_load_string($r_loc);
        if ($r_xml == FALSE)
            return -1;
        @$modelName = (string) $r_xml->device->modelName;
        if ($modelName !== "Eureka Dongle")
            return -1;
        @$friendlyName = (string) $r_xml->device->friendlyName;
        $r_mac = mac_from_ip($name, $mac, $eth);
        if ($r_mac == FALSE)
            return -1;
        if (isset($this->f_on_new_device)) {
            $f = $this->f_on_new_device;
            $f($name, $mac, $eth, $friendlyName);
            return 0;
        }
    }

}
