<?php

require_once(__DIR__ . "/inc/array_ext.inc.php");

class CommonDevice {
    var $device_type;
    var $device_id;
    var $eth;
    var $bind_ip;
    var $device_desc;
    var $svc_url;
    var $svc_host;
    var $svc_port;
    var $mac, $protovers, $model;
    var $dev_version;
    var $pl_md5 = 0;
    var $playlist;
    var $tick_suspended = FALSE;
    var $load_failed = 0;
    var $loaded = 0;
    var $loaded_time = 0;

    public function keep_alive() {
        return 0;
    }

    public function refresh_status() {
        return 0;
    }

    public function set_airplay_pwd($pwd, &$r) {
        $r = error_function_not_supported();
        return -1;
    }

    public function play($url, &$r = 0) {
        $r = error_function_not_supported();
        return -1;
    }

    public function stop(&$r) {
        $r = error_function_not_supported();
        return -1;
    }

    public function play_rtsp($url, &$r) {
        $r = error_function_not_supported();
        return -1;
    }

    public function setRate($rate, &$r) {
        $r = error_function_not_supported();
        return -1;
    }

    public function scrub($pos, &$r) {
        $r = error_function_not_supported();
        return -1;
    }

    public function notify_status($opt, &$r) {
        $r = error_function_not_supported();
        return -1;
    }

    public function tick() {
        
    }

    function __construct($opt, $dev_version) {
        $this->device_id = $opt["did"];
        $this->device_desc = $opt["name"];
        $this->mac = $opt["mac"];
        $this->model = $opt["model"];
        $this->svc_host = $opt["host"];
        $this->svc_port = $opt["port"];
        $opt_port = $this->svc_port == 80 ? "" : ":{$this->svc_port}";
        $this->svc_url = "http://{$this->svc_host}{$opt_port}";
        $this->device_type = $opt["device_type"];
        $this->eth = $opt["eth"];
        $this->bind_ip = get_eth_ip($this->eth);
        $this->dev_version = $dev_version;
    }

    function reassign_host($opt, $dev_version) {
        $this->device_desc = $opt["name"];
        $opt_port = $this->svc_port == 80 ? "" : ":{$this->svc_port}";
        $this->svc_url = "http://{$this->svc_host}{$opt_port}";
        if ($this->svc_host != $opt["host"] or $this->svc_port != $opt["port"]) {
            $this->close();
            $this->svc_host = $opt["host"];
            $this->svc_port = $opt["port"];
        } $this->dev_version = $dev_version;
    }

    function bind_ip($old_url) {
        $bind_ip = $this->bind_ip;
        $r = parse_url($old_url);
        $host = isset($r['port']) ? "{$bind_ip}:{$r['port']}" : $bind_ip;
        $path = isset($r['path']) ? $r['path'] : "";
        if (isset($r['user'], $r['pass']))
            $user = "{$r['user']}:{$r['pass']}@";
        else if (isset($r['pass']))
            $user = ":{$r['pass']}@";
        else
            $user = "";
        $query = isset($r['query']) ? "?{$r['query']}" : "";
        $frag = isset($r['fragment']) ? "#{$r['fragment']}" : "";
        $url = "{$r['scheme']}://{$user}{$host}{$r['path']}{$query}{$frag}";
        return $url;
    }

    function goto_playlist($pl_md5, $playlist_item_id, &$r_goto = 0) {
        if (!isset($this->pl_md5)) {
            $r_goto = make_resp(-1, "playlist is empty");
            return -1;
        } if ($pl_md5 != $this->pl_md5) {
            $r_goto = make_resp(-2, "playlist md5 not match");
            return -2;
        } foreach ($this->playlist as $k => $pi) {
            if ($pi->id === $playlist_item_id) {
                echo "goto play item found:{$pi->id}\n";
                $this->tick_suspended = FALSE;
                $this->idx_item = $k;
                unset($this->current_item);
                $r_goto = ok();
                return 0;
            }
        } $r_goto = make_resp(-3, "play item not found");
        return -3;
    }

    function get_playlist() {
        if (!isset($this->playlist))
            return array();
        $r = array();
        foreach ($this->playlist as $pl) {
            $r[] = $pl->serialize();
        } return $r;
    }

    function clear_playlist() {
        $n = is_array($this->playlist) ? count($this->playlist) : -1;
        echo "clear_playlist($n)\n";
        if ($n <= 0)
            return 0;
        unset($this->idx_item);
        unset($this->current_item);
        unset($this->pl_md5);
        $this->playlist = array();
        $this->tick_suspended = TRUE;
        $this->stop();
        return 0;
    }

    function tick_playlist() {
        if ($this->tick_suspended)
            return;
        if (isset($this->current_item)) {
            $r = $this->current_item->tick($this);
            if ($r == 1) {
                $this->loaded = $this->load_failed = 0;
                unset($this->current_item, $this->loaded_time);
                $idx_item = $this->idx_item + 1;
                if (count($this->playlist) <= $idx_item) {
                    echo "playlist completed.\n";
                    unset($this->idx_item);
                    $this->tick_suspended = TRUE;
                    $this->stop();
                } else {
                    $this->idx_item = $idx_item;
                }
            }
        } if (isset($this->idx_item) AND ! isset($this->current_item)) {
            echo "starting idx:" . $this->idx_item . "\n";
            @$this->current_item = $this->playlist[$this->idx_item];
            if (isset($this->current_item))
                $this->current_item->start($this);
        }
    }

    public function status(&$r_status) {
        if (isset($this->current_item)) {
            $r_status["playitem_id"] = $this->current_item->id;
        } if (isset($this->current_item))
            $r_status["current_item"] = $this->current_item->id;
        if (isset($this->pl_md5))
            $r_status["pl_md5"] = $this->pl_md5;
        return 0;
    }

    public function playlist_status(&$r_status) {
        $items = $this->get_playlist();
        $pl_md5 = isset($this->pl_md5) ? $this->pl_md5 : NULL;
        $current_item = isset($this->current_item) ? $this->current_item->id : NULL;
        $r_status = array("status" => 0, "pl_md5" => $pl_md5, "items" => $items, "current_item" => $current_item,);
        return 0;
    }

    function set_playlist(array $playlist, $item_to_play, $keep_playing, $replace_bind_ip = 0, &$r_pl = 0) {
        $bind_ip = ($replace_bind_ip) ? $this->bind_ip : "";
        @$old_playing_item = $this->current_item;
        @$old_playitem_item_idx = $this->idx_item;
        @$old_item_id = $keep_playing ? $old_playing_item->id : "_unmatched_item_id_XXsshfhtgcytREFZTJRHRHE";
        if ($replace_bind_ip) {
            foreach ($playlist as $pli) {
                $pli->url = $this->bind_ip($pli->url);
            }
        } $pl_md5 = md5(print_r($playlist, TRUE));
        $i = 0;
        $_playlist = array();
        foreach ($playlist as $_data) {
            $pi = new PlayItem($_data);
            if ($keep_playing and $pi->id == $old_item_id) {
                $pi = $old_playing_item;
                $item_to_play = $old_item_id;
                $skip_reset_playing = 1;
            } $_playlist[$i++] = $pi;
        } if (isset($item_to_play)) {
            echo "looking for {$item_to_play}\n";
            foreach ($_playlist as $pi) {
                if ($pi->id === $item_to_play) {
                    $pi_ready_to_play = $pi;
                    $idx_ready_to_play = $i;
                    break;
                }
            }
        } unset($this->current_item);
        $this->idx_item = 0;
        $this->pl_md5 = $pl_md5;
        $this->playlist = $_playlist;
        if (isset($pi_ready_to_play)) {
            if (isset($skip_reset_playing) and $skip_reset_playing == 1) {
                $this->current_item = $old_playing_item;
                $this->idx_item = $old_playitem_item_idx;
                echo "keep playing...\n";
            } else {
                $this->stop();
                $this->close();
                $this->idx_item = array_search($pi_ready_to_play, $_playlist);
                $this->tick_suspended = FALSE;
                echo "playing new one...\n";
            }
        } else {
            $this->stop();
            $this->close();
            $this->tick_suspended = TRUE;
            echo "stop playing...\n";
        } $r_pl = array("status" => 0, "pl_md5" => $this->pl_md5,);
        return 0;
    }

}
