<?php

require_once(__DIR__ . "/inc/url_parser.inc.php");
require_once(__DIR__ . "/playitem.php");
require_once(__DIR__ . "/CommonDevice.inc.php");

function proc_kill_all($process) {
    $status = proc_get_status($process);
    if ($status['running'] != true)
        return -1;
    $ppid = $status['pid'];
    pid_kill_all($ppid);
}

function pid_kill_all($ppid_to_kill) {
    $proc_items = scandir("/proc");
    $pids = array();
    foreach ($proc_items as $pid) {
        if (!is_numeric($pid))
            continue;
        $status = file_get_contents("/proc/$pid/status");
        if (!is_string($status))
            continue;
        $r = preg_match("/PPid:\s*(\d*)/", $status, $m);
        if ($r != 1)
            continue;
        $ppid = intval($m[1]);
        if ($ppid != $ppid_to_kill)
            continue;
        $pids[] = $pid;
    } $pids[] = $ppid_to_kill;
    foreach ($pids as $pid) {
        posix_kill($pid, SIGINT);
    }
}

function pid_kill_all_by_ps($ppid) {
    $pids = preg_split('/\s+/', `ps -o pid --no-heading --ppid $ppid`);
    foreach ($pids as $pid) {
        if (is_numeric($pid)) {
            echo "Killing $pid\n";
            posix_kill($pid, 9);
        }
    }
}

class RaopDevice extends CommonDevice {

    var $duration = 0.0;
    var $position = 0.0;
    var $connected = 0;
    var $started = 0;
    var $stopped = 1;
    var $completed = 0;
    var $pl_md5;
    var $playlist = array();
    var $current_item;
    var $idx_item;
    var $tick_suspended = FALSE;
    var $last_refresh_status = 0;
    var $ch;
    var $http_status;
    var $airplay_pwd;
    var $auth_confirmed = 0;
    var $use_encoding = 0;
    var $proc;
    var $stderr;
    var $file_left_over;
    var $phase;
    var $connected_time;

    public function stop(&$r_stop = 0) {
        $r = $this->close();
        unset($this->current_item);
        $this->tick_suspended = TRUE;
        $r_stop = ok();
        return 0;
    }

    function __construct($opt, $dev_version) {
        parent::__construct($opt, $dev_version);
        $this->device_id = $opt["did"];
        $this->device_desc = $opt["name"];
        $this->mac = $opt["mac"];
        $this->model = $opt["model"];
        $this->protovers = $opt["protovers"];
        $this->svc_host = $opt["host"];
        $this->svc_port = $opt["port"];
        $this->svc_url = "http://{$this->svc_host}:{$this->svc_port}";
        $this->device_type = $opt["device_type"];
        if ($opt["enc"] == 1)
            $this->use_encoding = 1;
        else
            $this->use_encoding = 0;
    }

    public function close() {
        echo "RaopDevice::close()\n";
        if (isset($this->proc)) {
            if (isset($this->pid_raop_play)) {
                echo "term 0\n";
                pid_kill_all($this->pid_raop_play);
                unset($this->pid_raop_play);
            } $r = proc_terminate($this->proc, SIGINT);
            $r = fclose($this->stderr);
            $r = proc_close($this->proc);
            unset($this->proc, $this->stderr, $this->phase, $this->connected_time);
        } if (isset($this->file_left_over)) {
            @unlink($this->file_left_over);
            unset($this->file_left_over);
        } $this->duration = $this->position = 0.0;
        $this->stopped = 1;
        $this->connected = $this->started = 0;
        $this->completed = 1;
    }

    function refresh_status() {
        if (!isset($this->proc))
            return 1;
        if ($this->phase == 3) {
            $now = microtime(true);
            $diff = $now - $this->connected_time;
            $this->position = $diff;
        } $r_ps = proc_get_status($this->proc);
        if (!$r_ps["running"]) {
            do {
                $line = fgets($this->stderr);
                if ($line === FALSE)
                    break;
                $r = $this->parseStderr(trim($line));
            } while (1);
            $this->close();
            return 1;
        } else {
            do {
                $line = fgets($this->stderr);
                if (!is_string($line)) {
                    break;
                } $this->parseStderr(trim($line));
            } while (1);
            return 0;
        }
    }

    function parseStderr($line) {
        echo "qairtunes.php stderr:$line\n";
        @list($k, $v) = explode(':', $line);
        if (isset($k) and isset($v)) {
            switch ($k) {
                case "phase": if ($v == "transcoding") {
                        $this->phase = 1;
                    } else if ($v == "raop_play") {
                        $this->phase = 2;
                    } else if ($v == "raop_play connected") {
                        $this->phase = 2;
                        $this->connected_time = microtime(true);
                    } else if ($v == "raop_play completed") {
                        unset($this->pid_raop_play);
                    } break;
                case "raop_play pid": $this->pid_raop_play = intval($v);
                    break;
                case "duration": $this->duration = floatval($v);
                    break;
            }
        } return 0;
    }

    public function play_rtsp($url, &$r_rtsp) {
        if (isset($this->proc))
            $this->close();
        if ((substr($opt["u"], 0, 7) == "http://") or substr($opt["u"], 0, 8) == "https://") {
            $query = parse_url($opt["u"]);
            parse_str_ex($query["query"], $q);
            $path = $q["path"];
        } else {
            $path = $url;
        } if ($this->use_encoding)
            $enc_opt = "-e";
        $pcm_fname = MMDIR . "/.qairplay/tmp_raop_{$this->svc_host}.pcm";
        @unlink($pcm_fname);
        $fname = "/share" . urldecode($path);
        $script = __DIR__ . "/script/qairtunes.php";
        $cmd = PHP_PATH . " $script $enc_opt -d {$this->svc_host} -p {$this->svc_port} -s " . escapeshellarg($fname);
        $ds = array(2 => array("pipe", "w"));
        $p = proc_open($cmd, $ds, $pipes);
        if (!is_resource($p)) {
            $r_rtsp = make_resp(-3, "error when exec external raop helper program");
            return -3;
        } $this->file_left_over = $pcm_fname;
        stream_set_blocking($pipes[2], 0);
        $this->position = $this->duration = 0;
        $this->stderr = $pipes[2];
        $this->proc = $p;
        $this->started = 1;
        $this->completed = 0;
        $this->phase = 0;
        $r_rtsp = ok();
        return 0;
    }

    public function status(&$r_status) {
        $this->refresh_status();
        if ($this->http_status == 401) {
            $r_status = error_appletv_need_auth();
            return 0;
        } if ($this->last_refresh_status != 0) {
            $r_status = error_communicating_device();
            return 0;
        } $dur = $this->duration;
        $pos = $this->position;
        $r_status = ok();
        $r_status["duration"] = floatval(sprintf("%.2f", $dur));
        $r_status["position"] = floatval(sprintf("%.2f", $pos));
        parent::status($r_status);
        return 0;
    }

}
