<?php

require_once(__DIR__ . "/inc/airplay.inc.php");
require_once(__DIR__ . "/inc/url_parser.inc.php");
require_once(__DIR__ . "/inc/resp.inc.php");
require_once(__DIR__ . "/inc/socket.inc.php");

function deploy_command($opt, $post_src) {
    $undefined_index = 0;
    $on_undefined_index = function ($errno, $errstr, $errfile, $errline, $errcontext) use (&$undefined_index) {
        if ($errno === 8) {
            $undefined_index = 1;
            return;
        } 
        var_dump($errno, $errstr, $errfile, $errline);
    };
    set_error_handler($on_undefined_index, E_ALL);
    switch ($opt["c"]) {
        case "log": 
            $cmd = array("cmd" => "log", "message" => $opt["s"]);
            break;
        case "hls_play": 
            $seekable = array_key_exists("k", $opt) ? 0 : 1;
            $cmd = array("cmd" => "hls_play", "device_id" => $opt["d"], "source" => $opt["u"], "seekable" => $seekable);
            break;
        case "hls_start": 
            $cmd = array("cmd" => "hls_start", "device_id" => $opt["d"], "source" => $opt["u"]);
            break;
        case "hls_teardown": 
            $cmd = array("cmd" => "hls_teardown", "device_id" => $opt["d"]);
            break;
        case "hls_seek": 
            $cmd = array("cmd" => "hls_seek", "device_id" => $opt["d"], "segment" => $opt["s"]);
            break;
        case "hls_status": 
            $cmd = array("cmd" => "hls_status", "device_id" => $opt["d"]);
            break;
        case "list": 
            $rvar_rescan = ($opt["r"] == "no") ? 0 : 1;
            $cmd = array("cmd" => "list", "rescan" => $rvar_rescan);
            break;
        case "pwd": 
            $cmd = array("cmd" => "pwd", "device_id" => $opt["d"], "pwd" => $opt["w"]);
            break;
        case "status": 
            $cmd = array("cmd" => "status", "device_id" => $opt["d"]);
            break;
        case "play": 
            $cmd = array("cmd" => "play", "device_id" => $opt["d"], "url" => $opt["u"]);
            if(isset($opt["i"])) {
                $cmd["image_url"] = $opt["i"];
            }
            if(isset($opt["t"])) {
                $cmd["title"] = $opt["t"];
            }
            break;
        case "rtsp": 
            $opt["u"] = parse_path_from_fileurl($opt["u"]);
            $cmd = array("cmd" => $opt["c"], "device_id" => $opt["d"], "url" => $opt["u"]);
            break;
        case "photo": 
            if (!in_array($opt["t"], array("none", "left", "right", "dissolve"))) {
                return error_parameter();
            } 
            $opt["u"] = parse_path_from_fileurl($opt["u"]);
            $cmd = array("cmd" => "photo", "device_id" => $opt["d"], "photo" => $opt["u"], "transition" => $opt["t"]);
            break;
        case "stop": 
            $cmd = array("cmd" => "stop", "device_id" => $opt["d"]);
            break;
	case "stop_only": 
            $cmd = array("cmd" => "stop_only", "device_id" => $opt["d"]);
            break;
        case "pause": 
            $cmd = array("cmd" => "rate", "device_id" => $opt["d"], "rate" => 0.0);
            break;
        case "resume": 
            $cmd = array("cmd" => "rate", "device_id" => $opt["d"], "rate" => 1.0);
            break;
        case "scrub": 
            $cmd = array("cmd" => "scrub", "device_id" => $opt["d"], "pos" => $opt["t"]);
            break;
        case "playlist": 
            $rvar_keep_playing = isset($opt["k"]) ? 1 : 0;
            $rvar_bind_ip = isset($opt["b"]) ? 1 : 0;
            $post_data = file_get_contents($post_src);
            $items = json_decode($post_data);
            $cmd = array("cmd" => "playlist", "device_id" => $opt["d"], "items" => $items, "keep_playing" => $rvar_keep_playing, "bind_ip" => $rvar_bind_ip);
            if (isset($opt["i"])) {
                $cmd["play_item"] = $opt["i"];
            } 
            break;
        case "playlist_status": 
            $cmd = array("cmd" => "playlist_status", "device_id" => $opt["d"]);
            break;
        case "playlist_goto": 
            $cmd = array("cmd" => "playlist_goto", "device_id" => $opt["d"], "pl_md5" => $opt["p"], "id" => $opt["i"]);
            break;
    } 
    restore_error_handler();
    unset($on_undefined_index);
    if ($undefined_index) {
        return missing_required_parameter();
    } 
    $r = send_message(AP_SOCK, json_encode($cmd));
    if ($r == "") {
        return array("status" => -1, "message" => "error send_message() to qairplay service");
    } 
    return $r;
}

function deploy_command_ex(&$r_v, $opt, $post_src) {
    @$cmd = $opt["c"];
    
    if (!in_array($cmd, array("smart_play"))) {
        $r_deploy = deploy_command($opt, $post_src);
        if (is_string($r_deploy)) {
            $r_deploy = json_decode($r_deploy, 1);
        } 
        $r_v = $r_deploy;
        return 0;
    } 
    switch ($cmd) {
        case "smart_play": return smart_play($r_v, $opt);
    } 
    return -1;
}

function hls_play(&$r_v, $opt) {
    if (!isset($opt["d"], $opt["u"])) {
        $r_v = missing_required_parameter();
        return -1;
    }
}

function smart_play(&$r_v, $opt) {
    if (!isset($opt["d"], $opt["u"])) {
        $r_v = missing_required_parameter();
        return -1;
    } 
    $opt["u"] = parse_path_from_fileurl($opt["u"]);
    $seekable = array_key_exists("k", $opt) ? 0 : 1;
    if (!is_file($opt["u"])) {
        $r_v = make_resp(-1, "source file not exists");
        return -1;
    } 
    $r_pi = pathinfo($opt["u"]);
    $ext = strtolower($r_pi["extension"]);
    if (in_array($ext, array("jpg", "jpeg", "gif", "bmp", "png", "tif", "tiff"))) {
        $r = make_fileurl($r_url, $opt["u"]);
        $opt = array("c" => "photo", "d" => $opt["d"], "u" => $r_url, "t" => "none");
        $r_v = deploy_command($opt, 0);
        return 0;
    } elseif (in_array($ext, array("m4a", "m4v", "mov", "mp4"))) {
        $r = make_fileurl($r_url, $opt["u"]);
        $opt = array("c" => "play", "d" => $opt["d"], "u" => $r_url);
        $r_v = deploy_command($opt, 0);
        return 0;
    } elseif (in_array($ext, array("rmvb", "avi", "mkv"))) {
        $opt = array("c" => "hls_start", "d" => $opt["d"], "u" => $opt["u"]);
        $r_v = deploy_command($opt, 0);
        if (!isset($r_v, $r_v["status"]) or $r_v["status"] < 0) {
            return -1;
        } 
        $retry_left = 10;
        while ($retry_left > 0) {
            $opt = array("c" => "hls_status", "d" => $opt["d"]);
            $r_v = deploy_command($opt, 0);
            if (!isset($r_v, $r_v["status"]) or $r_v["status"] < 0) {
                return -1;
            } 
            if ($seekable or ( isset($r_v, $r_v["segments"]) and count($r_v["segments"]) > 0))
                break;
            sleep(5);
            $retry_left--;
        } 
        $hls_url = $seekable ? $r_v["seekable_m3u8"] : $r_v["m3u8"];
        $opt = array("c" => "play", "d" => $opt["d"], "u" => $hls_url);
        $r_v = deploy_command($opt, 0);
        if (!isset($r_v, $r_v["status"]) or $r_v["status"] < 0) {
            return -1;
        } 
        return 0;
    } else {
        $r_v = make_resp(-1, "file type not supported");
        return -1;
    } 
    $r_v = ok();
    return 0;
}
