<?php

require_once(__DIR__ . "/inc/array_ext.inc.php");

class ffmpeg_proc extends extern_proc {

    var $duration, $resolution;

    function parse_output($s, $type) {
        $s = trim($s);
        if (!isset($this->duration) and substr($s, 0, 9) == "Duration:") {
            $r = preg_match("/Duration:\s*([0-9]+):([0-9]+):([0-9.]+)/", $s, $m);
            if ($r != 1) {
                echo("No Duration parsed: $s\n");
                $this->duration = 0;
            } else {
                $hour = intval($m[1]);
                $min = intval($m[2]);
                $sec = floatval($m[3]);
                $this->duration = $hour * 3600.0 + $min * 60.0 + $sec;
                echo "ffmpeg reported duration:{$this->duration}\n{$s}\n";
            }
        } if (!isset($this->resolution) and substr($s, 0, 8) == "Stream #" and strstr($s, "Video:")) {
            $r = preg_match("/Stream #.*Video:.*[^0-9]+([0-9]+)x([0-9]+)/", $s, $m);
            if ($r != 1) {
                var_dump($r, $m, $s);
                die("unexpected error!");
            } $w = intval($m[1]);
            $h = intval($m[2]);
            $rate = 30;
            $this->resolution = array($w, $h);
            $this->rate = $rate;
            echo "ffmpeg reported resolution:{$w}x{$h}@$rate fps\n{$s}\n";
        } return 0;
    }

    function close_by_q() {
        if (!isset($this->proc))
            return 1;
        fwrite($this->pipes[0], "q");
        return $this->close(FALSE);
    }

}

class hls_impl {

    var $device_id;
    var $sn = 0;
    var $source, $m3u8_path, $start_seg, $duration;
    var $ff_proc;
    var $target_rate, $target_resolution;
    var $clear_prev_ts = 1;

    function teardown(&$r_v = 0) {
        if ($this->clear_prev_ts) {
            $this->clear_prev_ts = 0;
            $ts_path = __DIR__ . "/api/hls/ts_{$this->device_id}";
            $opt_path = escapeshellarg($ts_path);
            $cmd = "rm -rf $opt_path";
            exec($cmd . " 2>/dev/null");
        } if (!isset($this->source, $this->m3u8_path)) {
            $r_v = array("message" => "not in a HLS session");
            return -1;
        } $this->sn++;
        $this->terminate_ffmpeg();
        $ts_path = dirname($this->m3u8_path);
        $cmd = "rm -rf " . escapeshellarg($ts_path);
        exec($cmd . " 2>/dev/null");
        unset($this->source, $this->m3u8_path, $this->start_seg, $this->duration);
        return 0;
    }

    var $seek_pending = 0;
    var $seek_sn = 0;

    function seek($seg, &$r_v) {
        if (!isset($this->source, $this->m3u8_path)) {
            $r_v = array("status" => -1, "message" => "not in a HLS session");
            return -1;
        } $seg = intval($seg);
        $r_seg = self::get_segment_scope($this->m3u8_path, $seglist);
        $last = end($seglist);
        if ($last !== FALSE) {
            $seglist[] = $last + 1;
            $seglist[] = $last + 2;
        } if (!$this->seek_pending and $this->start_seg !== $seg and ! in_array($seg, $seglist)) {
            $this->seek_pending = 1;
            $sn = $this->seek_sn + 1;
            $source = $this->source;
            $m3u8_path = $this->m3u8_path;
            $dev = $this;
            dispatch(0, function() use($dev, $seg, $source, $m3u8_path, $sn) {
                var_dump("ASKING seg not in scope, performing hls seek.", $seg, $source, $m3u8_path);
                $dev->teardown();
                $dev->invoke_transcoding($seg, $source, $m3u8_path);
                $dev->seek_pending = 0;
                $dev->seek_sn = $sn;
            });
            $r_v = array("status" => 0, "seek_sn" => $sn);
            return 0;
        } var_dump("ASKING seg in scope or seek pending({$this->seek_pending}), skip hls seek.", $seg);
        $r_v = array("status" => 0, "seek_sn" => $this->seek_sn);
        return 1;
    }

    function start($src, &$r_v) {
        $html_path = realpath(__DIR__ . "/api");
        if ($html_path === FALSE) {
            $r_v = make_resp(-1, "web output path cannot be found.");
            return -1;
        } $m3u8_path = "{$html_path}/hls/ts_{$this->device_id}/qaplay.m3u8";
        $dev = $this;
        dispatch(0, function() use($dev, $m3u8_path, $src) {
            $dev->teardown();
            $dev->invoke_transcoding(0, $src, $m3u8_path);
        });
        $r_v = ok();
        return 0;
    }

    function status(&$r_v) {
        if (!isset($this->source, $this->m3u8_path)) {
            $r_v = array("status" => -1, "message" => "not in a HLS session");
            return -1;
        } $r_seg = self::get_segment_scope($this->m3u8_path, $seglist);
        $r_v = array("status" => 0, "source" => $this->source, "m3u8" => DEFAULT_WEB_HOST . "/airplay/hls/ts_{$this->device_id}/qaplay.m3u8", "seekable_m3u8" => DEFAULT_WEB_HOST . "/airplay/hls_m3u8.php?d={$this->device_id}", "start" => $this->start_seg, "segments" => $seglist, "seek_sn" => $this->seek_sn,);
        if (isset($this->duration)) {
            $r_v["duration"] = $this->duration;
        } if (isset($this->target_resolution, $this->target_rate)) {
            $r_v["resolution"] = $this->target_resolution;
            $r_v["rate"] = $this->target_rate;
        } return 0;
    }

    private function terminate_ffmpeg() {
        $this->invoke_sn++;
        if (isset($this->ff_proc)) {
            echo "terminate_ffmpeg():ffmpeg is running.\n";
            $this->ff_proc->close_by_q();
            unset($this->ff_proc);
            return 0;
        } return 1;
    }

    function invoke_transcoding($seg, $source, $m3u8_path) {
        $target_duration = 10;
        $this->start_seg = $seg = intval($seg);
        $this->source = $source;
        $this->m3u8_path = $m3u8_path;
        $this->terminate_ffmpeg();
        $ts_path = dirname($m3u8_path);
        @mkdir($ts_path, 0755, true);
        if (!isset($this->ff_proc)) {
            $opt_aac = "-strict -2 -c:a aac -ac 2 -b:a 96k -ar 44100";
            $opt_h264 = "-c:v libx264 -g 3 -pix_fmt yuv420p -profile:v main -level 32";
            $opt_ss = $opt_hls = $opt_ss_post = "";
            if ($seg !== 0) {
                $opt_hls = "-start_number $seg";
                $opt_ss_post = "-ss {$target_duration}";
                if ($seg > 1) {
                    $opt_ss = "-ss " . ($target_duration * ($seg - 1));
                }
            } $cmd = implode(" ", array(FFMPEG_PATH, "-i", escapeshellarg($this->source)));
            $ff = new ffmpeg_proc();
            $ff->open($cmd);
            $ff->close(0);
            list($w, $h) = $this->source_resolution = $ff->resolution;
            $this->duration = $ff->duration;
            $this->target_rate = $this->rate = $ff->rate;
            $rate = round($ff->rate);
            unset($ff);
            $new_w = 320;
            $new_h = intval(16 * floor((($h * $new_w / $w) / 16.0) + 0.5));
            $new_h = ($new_h > 210) ? 240 : 176;
            $this->target_resolution = array($new_w, $new_h);
            $opt_rate = $opt_venc = $opt_aenc = $opt_hls_wrap = "";
            $opt_hls_size = "-hls_list_size 0";
            if ($this->duration <= 0) {
                $opt_hls_wrap = "-hls_wrap 150";
                $opt_hls_size = "-hls_list_size 15";
                $target_duration = 2;
            } $opt_rate = "-r 30";
            $this->target_rate = 30;
            $opt_venc = "{$opt_h264} -s {$new_w}x{$new_h} {$opt_rate}";
            $opt_aenc = $opt_aac;
            $cmd = array(FFMPEG_PATH, "-y", $opt_ss, "-i", escapeshellarg($this->source), $opt_ss_post, "-f hls -hls_time {$target_duration}", $opt_hls_size, $opt_hls_wrap, $opt_hls, $opt_aenc, $opt_venc, escapeshellarg($this->m3u8_path));
            $cmd = array_remove_null($cmd);
            $_cmd = implode(" ", $cmd);
            echo "transcoding command():\n$_cmd\n";
            $this->ff_proc = new ffmpeg_proc();
            $this->ff_proc->open($_cmd);
            $sn = $this->sn = $this->sn + 1;
            $dev = $this;
            $f = function($self) use($sn, $dev) {
                if (isset($dev->ff_proc)) {
                    if ($sn != $dev->sn)
                        return;
                    $r = $dev->ff_proc->tick();
                    if (isset($dev->ff_proc->rval)) {
                        echo "ffmpeg terminated.\n";
                    } else {
                        dispatch(1, $self);
                    }
                }
            };
            dispatch(1, $f);
        }
    }

    private static function get_segment_scope($m3u8_path, &$r_seglist) {
        @$r_file = file($m3u8_path);
        $r_seglist = array();
        if (!is_array($r_file)) {
            return -1;
        } $pre = "qaplay";
        $pre_len = strlen($pre);
        foreach ($r_file as $_line) {
            $line = trim($_line);
            if (substr($line, 0, $pre_len) == $pre and substr($line, -3, 3) == ".ts") {
                $seg = intval(substr($line, $pre_len, -3));
                $r_seglist[] = $seg;
            }
        } return 0;
    }

}
