<?php

require_once(__DIR__ . "/dns.inc.php");

class mdns_svr {

    var $sock_mdns;
    var $preferred;

    function __construct() {
        $this->preferred = array();
    }

    function setUp() {
        $sock_mdns = stream_socket_server("udp://0.0.0.0:5353", $errno, $errstr, STREAM_SERVER_BIND);
        if (!$sock_mdns) {
            die("$errstr ($errno)");
        } 
        $this->sock_mdns = $sock_mdns;
        return 0;
    }

    function tearDown() {
        unset($this->sock_mdns);
    }

    function select($timeout = 0.1) {
        $sec = intval(floor($timeout));
        $usec = intval(($timeout - $sec) * 1000000);
        $sock_mdns = $this->sock_mdns;
        $rd = array($sock_mdns);
        $wrt = $ex = array();
        $r = stream_select($rd, $wrt, $ex, $sec, $usec);
        if ($r == 0)
            return 1;
        if (!in_array($sock_mdns, $rd)) {
            echo "?";
            return 2;
        } 
        return 0;
    }

    function on_data_arrived() {
        $sock_mdns = $this->sock_mdns;
        $pkt = stream_socket_recvfrom($sock_mdns, 10480, 0, $from);
        $len = strlen($pkt);
        return $this->process_pkt($pkt, $len);
    }

    function process_pkt($pkt, $len) {
        $hdr = unpack('ntid/nflags/nqdcount/nancount/nnscount/narcount', $pkt);
        if ($hdr["flags"] != 0x8400 or $hdr["arcount"] <= 0) {
            return 1;
        } 
        $s = fopen("php://memory", "w+");
        fwrite($s, $pkt);
        fseek($s, 0);
        $r = parse_dns($s);
        fclose($s);
        if (is_array($r)) {
            $this->preferred = array_merge($this->preferred, $r);
            return 0;
        } 
        return 2;
    }

}

function parse_dns($s) {
    $ans = new DNSAnswer($s, 1024);
    if (!is_object($ans) or ! isset($ans->rec_additional) or ! is_array($ans->rec_additional)) {
        return FALSE;
    } 
    $hosts = $prefered = array();
    foreach ($ans->rec_additional as $rr) {
        if ($rr->type == 1) {
            $ip = implode(".", $rr->specific_fields["address"]->labels);
            $ip = $rr->specific_fields["address"]->labels[3] . "." . $rr->specific_fields["address"]->labels[2] . "." . $rr->specific_fields["address"]->labels[1] . "." . $rr->specific_fields["address"]->labels[0];
            if (!isset($hosts[$rr->name->labels[0]]))
                $hosts[$rr->name->labels[0]] = $ip;
            else {
                $old_ip = $hosts[$rr->name->labels[0]];
                if ($old_ip != $ip) {
                    $ok_ip = (substr($old_ip, 0, 4) == "169.") ? $ip : $old_ip;
                    $replace_ip = (substr($old_ip, 0, 4) == "169.") ? $old_ip : $ip;
                    $prefered[$replace_ip] = $ok_ip;
                }
            }
        }
    } 
    return $prefered;
}
