<?php

class MessageServer {

    var $sock_name;
    var $svr_sock;
    var $f_process_message;

    function on_accept($s) {
        $peer = socket_accept($s);
        $recv_buffer = socket_read_all($peer);
        $shutdown_ok = socket_shutdown($peer, 0);
        $svr = $this;
        $f_resp = function($resp) use($svr, $peer) {
            $svr->resp_and_close_peer($peer, $resp);
        };
        $f = $this->f_process_message;
        $f($recv_buffer, $f_resp);
    }

    function resp_and_close_peer($peer, $resp) {
        if (!is_string($resp))
            $resp = json_encode($resp);
        socket_write($peer, $resp);
        socket_shutdown($peer, 2);
        socket_close($peer);
    }

    function setUp($sock_name, $f_process_message) {
        if (isset($this->svr_sock))
            return -1;
        @unlink($sock_name);
        $s = socket_create(AF_UNIX, SOCK_STREAM, 0);
        if (!is_resource($s)) {
            echo "fail socket_create(AF_UNIX, SOCK_STREAM, 0);\n";
            return -1;
        } 
        $old_umask = umask();
        umask(0);
        $bind_ok = socket_bind($s, $sock_name);
        umask($old_umask);
        if (!$bind_ok) {
            echo "fail socket_bind(\"{$sock_name}\");\n";
            return -1;
        } 
        $listen_ok = socket_listen($s, 5);
        $this->svr_sock = $s;
        $this->sock_name = $sock_name;
        $f = $this->f_process_message = $f_process_message;
        
        return 0;
    }

    function tearDown() {
        if (!isset($this->svr_sock))
            return 1;
        socket_close($this->svr_sock);
        unlink($this->sock_name);
        unset($this->svr_sock);
    }

}

class SvcLoop {

    public static $svc;

    public static function dispatch($delay, $w) {
        return self::$svc->dispatch_delayed_work($delay, $w);
    }

    var $delayed_works;
    var $inited = 0;
    var $sock_list, $callback_list;
    var $next_pair_index = 123;
    var $mch;
    var $ch_list;

    function __construct() {
        $this->delayed_works = array();
        $this->callback_list = $this->sock_list = array();
        $this->ch_list = array();
        $this->mch = curl_multi_init();
    }

    function add_callback($sock, $callback, $param) {
        if (is_array($sock)) {
            foreach ($sock as $k => $sock_) {
                $this->add_callback($sock_, $callback, $k);
            } return 0;
        } 
        $r = array_search($sock, $this->sock_list);
        if ($r !== FALSE)
            return -1;
        $idx = $this->next_pair_index++;
        $this->sock_list[$idx] = $sock;
        $this->callback_list[$idx] = $callback;
        $this->param_list[$idx] = $param;
        return 0;
    }

    function dispatch_delayed_work($delay, $w) {
        $t = ($delay > 0) ? time() + $delay : 0;
        $this->delayed_works[$t][] = $w;
        ksort($this->delayed_works);
        return 0;
    }

    function step($timeout = 0.1) {
        if (!$this->inited) {
            $this->inited = 1;
        } 
        $still_running = 0;
        $r_cme = curl_multi_exec($this->mch, $still_running);
        while (1) {
            $r_info = curl_multi_info_read($this->mch, $n_msgs);
            if ($r_info === FALSE)
                break;
            foreach ($this->ch_list as $k => $pair) {
                if ($pair["ch"] === $r_info["handle"]) {
                    curl_multi_remove_handle($this->mch, $pair["ch"]);
                    $pair["f"]();
                    unset($this->ch_list[$k]);
                    break;
                }
            }
        } if (count($this->sock_list) > 0) {
            $read_fd = $this->sock_list;
            $write_fd = $except_fd = NULL;
            $sec = intval(floor($timeout));
            $usec = intval(($timeout - $sec) * 1000000);
            $r_select = @socket_select($read_fd, $write_fd, $except_fd, $sec, $usec);
            foreach ($read_fd as $idx => $s) {
                $callback = $this->callback_list[$idx];
                $param = $this->param_list[$idx];
                $callback($s, $param);
            }
        } else {
            usleep($timeout * 1000000);
        } 
        $now = time();
        foreach ($this->delayed_works as $tm => $bucket) {
            if ($tm !== 0 and $tm > $now)
                break;
            unset($this->delayed_works[$tm]);
            foreach ($bucket as $w) {
                $r = $w($w);
            }
        }
    }

    function curl_exec_ex($ch, $f_on_complete) {
        foreach ($this->ch_list as $k => $pair) {
            if ($pair["ch"] === $ch)
                return -1;
        } 
        $this->ch_list[] = array("ch" => $ch, "f" => $f_on_complete);
        curl_multi_add_handle($this->mch, $ch);
        return 0;
    }

}

function dispatch($delay, $w) {
    return SvcLoop::$svc->dispatch_delayed_work($delay, $w);
}

function curl_exec_ex($ch, $f_on_complete) {
    return SvcLoop::$svc->curl_exec_ex($ch, $f_on_complete);
}
