<?php

list($fn) = get_included_files();
if (fileinode($fn) != fileinode(__FILE__))
    return;
$usage = <<< EOHELP
qairplay 0.1, qairplay service cli interface.
Usage: php qairplay.php [OPTION]

General options:
  -c\t\t\tcommand
  -x\t\t\toutput xml
  -h\t\t\tshow usage

Basic commands: list, status, play, photo, stop, pause, resume, scrub.

list command:
  Query airplay devices, will return airplay device id (device_id) and device name (device_name).
  Example: php qairplay.php -c=list
  -r\t\t\trescan devices (default no)

pwd command:
  Set password needed to authenticate Apple TV device
  -d\t\t\tairplay device id
  -w\t\t\tapple tv password

status command:
  Query airplay device status, returns current playing position and duration.
  Will also returns playlist md5 (pl_md5) and currently playing playlist item id (current_item) if a playlist is presented.
  -d\t\t\tairplay device id

play command:
  Tell an airplay device to play a media file from url (mp4 or m4a).
  -d\t\t\tairplay device id
  -u\t\t\turl of media file
  -i\t\t\turl for thumbnail(just for ChromeCast)
  -t\t\t\ttitle for play resource(just for ChromeCast)

photo command:
  Tell an airplay device to display a image file from url (currently only jpeg are supported).
  -d\t\t\tairplay device id
  -u\t\t\turl of image file
  -t\t\t\ttransition type (left, right, dissolve, none), default is none.
 
stop command:
  Stop current playing media of an airplay device.
  -d\t\t\tairplay device id
 
pause command:
  Pause current playing media of an airplay device.
  -d\t\t\tairplay device id
 
resume command:
  Resume current playing media of an airplay device.
  -d\t\t\tairplay device id
 
scrub command:
  Jump to specific time of current playing media of an airplay device.
  -d\t\t\tairplay device id
  -t\t\t\ttime to jump (in second)
 
rtsp command:
  Transcode a media file then do audio streaming to an airtunes device.
  -d\t\t\tairtunes device id
  -u\t\t\tmedia file path

Playlist commands: playlist, playlist_status, playlist_goto

playlist command:
  Assign new playlist items to an airplay device.
  -d\t\t\tairplay device id
  -i\t\t\tplaylist item id to play after playlist set
  -k\t\t\tkeep playing currently playing item
  -b\t\t\treplace playitem url with bind ip which can reach this airplay device
  stdin\t\t\tplayllist items infomation. Sample json in tests/demo_pl.json.txt

playlist_status command:
  Returns playlist of an airplay device.
  -d\t\t\tairplay device id

playlist_goto command:
  Jump to specific playlist item of an airplay device.
  -d\t\t\tairplay device id
  -p\t\t\t playlist md5 (returned by status or playlist_status command)
  -i\t\t\tplaylist item id

HLS related commands: hls hls_status hls_cancel (testing)

hls_play command:
  play media file or rtsp url, rtsp url will use non-seekable hls.
  -d\t\t\tairplay device id
  -u\t\t\tmedia file path
  -k\t\t\tuse non-seekable hls (default seekable)

hls_start command:
  Initlize transcoding, teardown previous hls if needed
  -d\t\t\tairplay device id
  -u\t\t\tmedia file path

hls_seek command:
  Invoke transcoding to specified segment if not already invoked.
  -d\t\t\tairplay device id
  -u\t\t\tmedia file path
  -s\t\t\tsegment number (default 0)

hls_status command:
  Returns current transcoding source media and duration, segment count, etc.
  -d\t\t\tairplay device id

hls_teardown command:
  -d\t\t\tairplay device id

smart_play command:
  play media file, may use play or hls depends on file type.
  -d\t\t\tairplay device id
  -u\t\t\tmedia file path
  -k\t\t\tuse non-seekable hls (default seekable)

Mail bug reports and suggestions to <lichihwu@qnap.com>. 

EOHELP;
require_once(__DIR__ . "/inc/airplay.inc.php");
require_once(__DIR__ . "/inc/socket.inc.php");
require_once(__DIR__ . "/inc/resp.inc.php");
require_once(__DIR__ . "/api_interface.inc.php");
require_once(__DIR__ . "/api/filelist.php");
detectSystemSettings();
$opt = getopt("c:d:w:p:i:u:t:s:rkhbx");
$defopt = array("list" => array("r" => 0), "photo" => array("t" => "none"),);
@$cmd = $opt["c"];
if (array_key_exists($cmd, $defopt)) {
    foreach ($defopt[$cmd] as $k => $v) {
        if (!array_key_exists($k, $opt)) {
            $opt[$k] = $v;
        }
    }
} 
$output_xml = array_key_exists("x", $opt) ? 1 : 0;
if (count($opt) <= 0 or array_key_exists("h", $opt)) {
    echo($usage);
    exit(0);
} 
if (!array_key_exists("c", $opt)) {
    $r = make_resp(-1, "requires -c parameter");
    $r = $output_xml ? array_to_xml($r) : json_encode($r);
    echo $r;
    exit(-1);
} 
$error_level = 0;
$r = deploy_command_ex($r_deploy, $opt, "php://stdin");
if (is_string($r_deploy)) {
    $resp = json_decode($r_deploy, 1);
    if (!is_array($resp)) {
        $resp = array("status" => -1, "message" => "error deploy_command_ex returned non-JSON", "r_deplay" => $r_deploy);
    }
} else {
    $resp = $r_deploy;
} 
$error_level = isset($resp["status"]) ? $resp["status"] : -1;
$output = $output_xml ? array_to_xml($resp) : (is_string($r_deploy) ? $r_deploy : json_encode($resp));
echo $output . "\n";
exit($error_level);

function array_to_xml($arr, &$xml = 0) 
{
    if ($xml === 0) {
        $xml = new SimpleXMLElement('<?xml version="1.0"?><root/>');
    } 
    foreach ($arr as $key => $value) {
        if (is_array($value)) {
            if (!is_numeric($key)) {
                $subnode = $xml->addChild("$key");
                array_to_xml($value, $subnode);
            } else {
                $subnode = $xml->addChild("item");
                array_to_xml($value, $subnode);
            }
        } else {
            $xml->addChild("$key", "$value");
        }
    } 
    return $xml->asXML();
}
