<?php

require_once(__DIR__ . "/inc/ip_helper.inc.php");
require_once(__DIR__ . "/inc/airplay.inc.php");
require_once(__DIR__ . "/inc/socket_svr.inc.php");
require_once(__DIR__ . "/inc/resp.inc.php");
require_once(__DIR__ . "/inc/mdns_svr.inc.php");
require_once(__DIR__ . "/inc/msgsvr.inc.php");
require_once(__DIR__ . "/inc/extern_proc.inc.php");
require_once(__DIR__ . "/CommonDevice.inc.php");
require_once(__DIR__ . "/AirPlayDevice.php");
require_once(__DIR__ . "/RaopDevice.php");
//require_once(__DIR__ . "/Chromecast.php");

$debug = file_exists(__DIR__ . "/debug");
$vm_debug = file_exists(__DIR__ . "/vm_debug");
$cast_support = 0;

if (file_exists(__DIR__ . "/bin/cc_handler")) {
    $cast_support = 1;
    define("CC_HANDLER", __DIR__ . "/bin/cc_handler");
} elseif (file_exists(__DIR__ . "/cc_root/bin/cc_handler")) {
    $cast_support = 1;
    define("CC_HANDLER", "/usr/sbin/chroot " . __DIR__ . "/cc_root /bin/cc_handler");
} 

@detectSystemSettings();
if (!defined("DEFAULT_WEB_HOST")) {
    die("fail detecting DEFAULT_WEB_HOST");
} 
//if (!defined("FFMPEG_PATH")) {
//    die("fail detecting FFMPEG_PATH");
//} 
if (!defined("PHP_PATH")) {
    die("fail detecting PHP_PATH");
} 
if (!defined("CURL_PATH")) {
    die("fail detecting CURL_PATH");
} 

$fn = __DIR__ . "/key";
if (!file_exists($fn)) {
    $key = random_string(64);
    umask(022);
    file_put_contents($fn, $key);
} 

unset($fn, $key);
umask(0);
$dev_version = 0;
$devices = array();
$mdns = new mdns_svr();
$mdns->setUp();
if ($debug) {
    
} 

$preferred_ip = $mdns->preferred;
$msg_svr = new MessageServer();
$r_setup = $msg_svr->setUp(AP_SOCK, "handle_incoming_message");
if ($r_setup !== 0)
    die("fail msg_svr->setUp()\n");
SvcLoop::$svc = new SvcLoop();
SvcLoop::$svc->add_callback($msg_svr->svr_sock, 
                            function($s) use($msg_svr) {
                                $msg_svr->on_accept($s);
                            }, 0);
                            
$avahi_cache_option = "";
//SvcLoop::$svc->dispatch_delayed_work(2, function() {handle_list_cmd(NULL);});
dispatch(2, function() {handle_list_cmd(NULL);});


$reset_cache_option = function() use (&$reset_cache_option) {
    global $avahi_cache_option;
    $avahi_cache_option = "";
    dispatch(30, $reset_cache_option);
};
//SvcLoop::$svc->dispatch_delayed_work(30, $reset_cache_option);
dispatch(30, $reset_cache_option);

echo "airplay service started...\n";

while (1) {
    SvcLoop::$svc->step(1);
    foreach ($devices as $d) {
        $d->tick_playlist();
        $d->tick();
    }
} 
$msg_svr->tearDown();
$mdns->tearDown();
$finder->tearDown();
exit;

function find_device(&$r_error, $opt) 
                {
    if (!isset($opt->device_id)) {
        $r_error = missing_required_parameter();
        return FALSE;
    } 
    global $devices;
    foreach ($devices as $d) {
        if ($d->device_id === $opt->device_id) {
            return $d;
        }
    } 
    $r_error = device_not_found();
    return FALSE;
}

function start_finder() 
{
    global $finder;
    $finder = new AppleTvFinder();
    $finder->setUp();
}

function handle_list_cmd($_buf) 
{
    global $devices, $dev_version, $preferred_ip, $next_force_rescan_time;
    $k_preferred_ip = array_keys($preferred_ip);
    $rescan = (isset($_buf->rescan) and $_buf->rescan);
    
    if (!isset($next_force_rescan_time) or time() >= $next_force_rescan_time) {
        $next_force_rescan_time = time() + 60;
        $rescan = 1;
    } 
    if ($rescan) {
        global $cast_support, $send_chromecast_discovery_request;
    
        if ($cast_support) {
            $send_chromecast_discovery_request = 1;
        } 
        $dev_version++;
        $r_devices = device_discovery();
        $new_devices = array();
        foreach ($r_devices as $opt) {
            $old_ip = $opt["host"];
            if (in_array($old_ip, $k_preferred_ip)) {
                $ip = $preferred_ip[$old_ip];
                echo "found preferred ip $ip of $old_ip\n";
                $opt["host"] = $ip;
            } 
            $_dev_id = $opt["did"];
            $found = 0;
            foreach ($devices as $d) {
                if ($d->device_id === $_dev_id) {
                    $d->reassign_host($opt, $dev_version);
                    $new_devices[] = $d;
                    $found = 1;
                    break;
                }
            } 
            if ($found === 0) {
                if ($opt["device_type"] == "airplay") {
                    $new_devices[] = new AirPlayDevice($opt, $dev_version);
                } else if ($opt["device_type"] == "raop") {
                    $new_devices[] = new RaopDevice($opt, $dev_version);
                } else if ($opt["device_type"] == "chromecast") {
                    //$new_devices[] = $cast = new ChromecastDevice($opt, $dev_version);
                }
            }
        } 
        $devices = $new_devices;
        unset($new_devices);
        $msg = array();
        foreach ($devices as $d) {
            $msg[] = array("device_desc" => $d->device_desc, "mac" => $d->mac, "host" => $d->svc_host, "eth" => $d->eth, "device_id" => $d->device_id, "device_type" => $d->device_type);
        } 
        var_dump($msg);
    } 
    $r = ok();
    $r["devices"] = array();
    foreach ($devices as $d) {
        $_d = array("device_id" => $d->device_id, "device_name" => $d->device_desc, "type" => $d->device_type, "svc_url" => $d->svc_url, "ip" => $d->svc_host, "mac" => $d->mac, "model" => $d->model, "eth" => $d->eth,);
        if (isset($d->pl_md5))
            $_d["pl_md5"] = $d->pl_md5;
        $r["devices"][] = $_d;
    } 
    return $r;
}

function device_discovery() {
    global $debug, $cast_support, $preferred_ip;
    $dids = $opt = array();
    get_eth_ip("eth0");
    $avahi_path = file_exists("/usr/sbin/avahi-browse") ? "/usr/sbin/avahi-browse" : "avahi-browse";
    global $avahi_cache_option;
    $avahi_cmd = "$avahi_path -rkpt {$avahi_cache_option}";
    var_dump($avahi_cmd);
    exec("$avahi_cmd _airplay._tcp", $r_airplay, $rval);
    exec("$avahi_cmd _raop._tcp", $r_raop, $rval);
    exec("$avahi_cmd _googlecast._tcp", $r_googlecast, $rval);
    $avahi_cache_option = "-c";
    $r_exec = array_merge($r_airplay, $r_raop, $r_googlecast);
    global $eth_list;
    $ifaces = array_keys($eth_list);
    foreach ($r_exec as $line) {
        @list($head, $eth, $ipver, $dev_name, $service_type,,, $ip, $port, $_txt) = explode(";", $line);
        if (!($head == "=" and $ipver == "IPv4"))
            continue;
        if (!in_array($eth, $ifaces)) {
            rescan_iface();
            $ifaces = array_keys($eth_list);
        } 
        $dev_name = decode_avahi($dev_name);
        $_txt = explode('" ', $_txt);
        $txt = array();
        foreach ($_txt as $txt_item) {
            $r = trim($txt_item, '"');
            @list($k, $v) = explode("=", $r, 2);
            if (!isset($k, $v))
                continue;
            $txt[$k] = $v;
        } 
        if (isset($preferred_ip[$ip])) {
            echo "change ip from $ip to preferred ip  {$preferred_ip[$ip]}\n";
            $ip = $preferred_ip[$ip];
        } 
        if ($service_type == "_googlecast._tcp") {
            $mac_addr = "";
            $did = $txt["id"];
            if ($did === 0)
                continue;
            if (in_array($did, $dids)) {
                continue;
            } 
            $dev = array("device_type" => "chromecast", "did" => $did, "name" => $dev_name, "mac" => $mac_addr, "model" => $txt["md"], "host" => $ip, "port" => 8008, "eth" => $eth);
            $dids[] = $did;
            $opt[] = $dev;
        } else if ($service_type == "_airplay._tcp") {
            @$deviceid = strtolower(str_replace(':', '', $txt["deviceid"]));
            if (in_array($deviceid, $dids) or $deviceid == "") {
                continue;
            } 
            $mac = implode(":", str_split($deviceid, 2));
            $dev = array("did" => $deviceid, "eth" => $eth, "name" => $dev_name, "host" => $ip, "port" => $port, "model" => $txt["model"], "mac" => $mac, "protovers" => $txt["srcvers"], "device_type" => "airplay");
            $dids[] = $deviceid;
            $opt[] = $dev;
        } else if ($service_type == "_raop._tcp") {
            list($deviceid, $dev_name) = explode("@", $dev_name, 2);
            $deviceid = strtolower($deviceid);
            if (in_array($deviceid, $dids)) {
                continue;
            } 
            $mac = implode(":", str_split($deviceid, 2));
            $enc = isset($txt['am']) ? 0 : 1;
            @$dev = array("did" => $deviceid, "eth" => $eth, "name" => $dev_name, "host" => $ip, "port" => $port, "model" => $txt["am"], "enc" => $enc, "mac" => $mac, "protovers" => $txt["vs"], "device_type" => "raop");
            $dids[] = $deviceid;
            $opt[] = $dev;
        }
    } 
    return $opt;
}

function decode_avahi($s) {
    $r = "";
    $i = 0;
    do {
        $c = substr($s, $i, 1);
        if (!is_string($c))
            break;
        if ($c == '\\') {
            $n = substr($s, $i + 1, 3);
            if (is_numeric($n)) {
                $i += 4;
                $r .= chr($n);
            } else {
                $i++;
            }
        } else {
            $i++;
            $r.= $c;
        }
    } while (1);
    
    return $r;
}

function random_string($len) 
{
    $validCharacters = "abcdefghijklmnopqrstuxyvwzABCDEFGHIJKLMNOPQRSTUXYVWZ0123456789!@#$%^&*()-=_+`~";
    $max = strlen($validCharacters) - 1;
    $key = "";
    mt_srand();
    
    for ($i = 0; $i < $len; $i++) {
        $pos = mt_rand(0, $max);
        $key .= $validCharacters[$pos];
    } 
    
    return $key;
}

function handle_log_cmd($_buf) 
{
    echo "LOG:\n";
    var_dump($_buf->message);
    
    return ok();
}

function handle_status_cmd($_buf) 
{
    $d = find_device($r_error, $_buf);
    if ($d == FALSE)
        return $r_error;
    $r = $d->status($r_status);

    return $r_status;
}

function handle_stop_cmd($_buf) 
{
    $d = find_device($r_error, $_buf);
    
    if ($d == FALSE)
        return $r_error;
    $d->stop();
    $d->close();

    return ok();
}

function handle_stop_only_cmd($_buf) 
{
    $d = find_device($r_error, $_buf);
    
    if ($d == FALSE)
        return $r_error;
    $d->stop();
	//Chromecast won't close when stop.
	//echo "device type is $d->device_type\n";
	//if(strcmp($d->device_type, 'chromecast') != 0) {
	//	echo "device close\n";
	//	$d->close();
	//}    

    return ok();
}

function handle_hls_status_cmd($_buf) 
{
    $d = find_device($r_error, $_buf);
    
    if ($d == FALSE)
        return $r_error;
    $r = $d->hls_status($r_status);
    return $r_status;
}

function handle_hls_seek_cmd($_buf) 
{
    $d = find_device($r_error, $_buf);
    
    if ($d == FALSE)
        return $r_error;
    if (!isset($_buf->segment))
        return missing_required_parameter();
    $r = $d->hls_seek($_buf->segment, $r_seek);

    return $r_seek;
}

function handle_hls_play_cmd($_buf) 
{
    $d = find_device($r_error, $_buf);
    
    if ($d == FALSE)
        return $r_error;
    if (!isset($_buf->source, $_buf->seekable))
        return missing_required_parameter();
    $d->hls_play($_buf->source, $_buf->seekable);

    return ok();
}

function handle_hls_start_cmd($_buf) 
{
    $d = find_device($r_error, $_buf);

    if ($d == FALSE)
        return $r_error;
    if (!isset($_buf->source))
        return missing_required_parameter();
    $d->hls_start($_buf->source);
    
    return ok();
}

function handle_hls_teardown_cmd($_buf) 
{
    $d = find_device($r_error, $_buf);

    if ($d == FALSE)
        return $r_error;
    $d->hls_teardown();
    
    return ok();
}

function handle_playlist_status_cmd($_buf) 
{
    $d = find_device($r_error, $_buf);

    if ($d == FALSE)
        return $r_error;
    $r = $d->playlist_status($r_status);
    
    return $r_status;
}

function handle_clear_playlist_cmd($_buf) 
{
    $d = find_device($r_error, $_buf);
    
    if ($d == FALSE)
        return $r_error;
    $d->clear_playlist();

    return ok();
}

function handle_playlist_cmd($_buf) 
{
    var_dump($_buf);

    if (!isset($_buf->device_id, $_buf->items) or ! is_array($_buf->items))
        return missing_required_parameter();
    $d = find_device($r_error, $_buf);
    if ($d == FALSE)
        return $r_error;
    @$keep_playing = (isset($_buf->keep_playing) and $_buf->keep_playing) ? 1 : 0;
    @$play_item = (isset($_buf->play_item)) ? $_buf->play_item : "_unmatched_item_md5_XXsshfhtgcytREFZTJRHRHE";
    $bind_ip = (isset($_buf->bind_ip) and $_buf->bind_ip) ? 1 : 0;
    $r = $d->set_playlist($_buf->items, $play_item, $keep_playing, $bind_ip, $r_pl);
    
    return $r_pl;
}

function handle_playlist_goto_cmd($_buf) 
{
    if (!isset($_buf->pl_md5, $_buf->device_id, $_buf->id))
        return missing_required_parameter();
    $d = find_device($r_error, $_buf);
    if ($d == FALSE)
        return $r_error;
    if ($d->pl_md5 !== $_buf->pl_md5)
        return error_parameter();
    $r = $d->goto_playlist($_buf->pl_md5, $_buf->id, $r_goto);

    return $r_goto;
}

function handle_play_cmd($_buf, $f_resp) 
{
    $d = find_device($r_error, $_buf);

    if ($d == FALSE)
        return $r_error;
    if (!isset($_buf->url))
        return missing_required_parameter();
    $d->clear_playlist();
    $r_play = 0;
    $r = $d->play($_buf->url, $r_play, $f_resp, $_buf->image_url, $_buf->title);
    if ($r_play !== 0)
        $f_resp(json_encode($r_play));
    
    return 0;
}

function handle_rtsp_cmd($_buf) 
{
    $d = find_device($r_error, $_buf);

    if ($d == FALSE)
        return $r_error;
    if (!isset($_buf->url))
        return missing_required_parameter();
    $d->clear_playlist();
    $d->play_rtsp($_buf->url, $r_rtsp);
    return $r_rtsp;
    
    if ($d->http_status == 401)
        return error_appletv_need_auth();
    
    return ok();
}

function handle_rate_cmd($_buf) 
{
    $d = find_device($r_error, $_buf);
    if ($d == FALSE)
        return $r_error;
    if (!isset($_buf->rate))
        return missing_required_parameter();
    $r = $d->setRate($_buf->rate, $r_rate);

    return $r_rate;
}

function handle_scrub_cmd($_buf) 
{
    $d = find_device($r_error, $_buf);
    if ($d == FALSE)
        return $r_error;
    if (!isset($_buf->pos))
        return missing_required_parameter();
    $r = $d->scrub($_buf->pos, $r_scrub);
    return $r_scrub;
}

function handle_photo_cmd($_buf) 
{
    print_r($_buf);
    $d = find_device($r_error, $_buf);
    if ($d == FALSE)
        return $r_error;
    if (!isset($_buf->url) and ! isset($_buf->photo))
        return missing_required_parameter();
    @$trans = $_buf->transition;
    @$url = $_buf->url;
    @$photo_path = $_buf->photo;
    dispatch(0, 
            function($self) use($d, $url, $photo_path, $trans) {
                $d->clear_playlist();
                $r = ok();
                if (isset($url))
                    $d->play_photo_url($url, $trans);
                else if (isset($photo_path))
                    $d->play_photo($photo_path, $trans);
            });
            
    return ok();
}

function handle_cast_notify_status_cmd($opt) 
{
    $d = find_device($r_error, $opt);
    if ($d === FALSE)
        return $r_error;
    $r = $d->notify_status($opt, $r_notify);

    return $r_notify;
}

function handle_pwd_cmd($_buf) 
{
    $d = find_device($r_error, $opt);
    
    if ($d === FALSE)
        return $r_error;
    if (!isset($_buf->pwd))
        return missing_required_parameter();
    $r = $d->set_airplay_pwd($pwd, $r_pwd);

    return $r_pwd;
}

function handle_incoming_message($message, $f_resp) 
{
    $opt = json_decode($message);
    
    if ($opt === FALSE) {
        fprintf(STDERR, "fail json_decode()\n");
        return $f_resp("");
    } 
    
    @$cmd = $opt->cmd;
    $func_name = "handle_{$cmd}_cmd";
    if (function_exists($func_name)) {
        $r = $func_name($opt, $f_resp);
    } else {
        var_dump($message);
        echo "unknown command '{$cmd}', function '{$func_name}' not found\n";
        $r = unsupported_command();
    } if ($r !== 0)
        $f_resp(json_encode($r));
}
