#!/bin/sh
# this script pipe VATranscode stdout to ffmpeg segment

# variable
FFMPEG_BIN="/mnt/ext/opt/medialibrary/bin/ffmpeg"
FFMPEG_FILE_ATTACHED_NAME="%04d.ts"
FFMPEG_PLAYLIST_ATTACHED_NAME=".m3u"

# init var
FFMPEG_SEG_TIME=""
FFMPEG_SEG_PLAYLIST=""
FFMPEG_FILE_DEST=""
FFMPEG_CMD=""
FFMPEG_DEBUG="0"

usage()
{
cat >&2 << HELP

usage:
  -segtime:     video segment duration in second, ex: -segtime 60
  -seglist:     real path of video play list, the attached name will be auto added, ex: -seglist /tmp/myPlaylist
  -dest:        real path of video destination, the attached name and sequence number will be auto added, ex: -dest /tmp/myVideo
  -debug:       debug mode, 1: show cmd only, 2: show ffmpeg execution debug message
  -h/-help:     show VASegmenter usage
HELP
}

#start parse option
currOpt=""
while [ $# -ge 1 ]; do	
	case "$1" in
		-h|-help)		
			usage && exit 0
			;;
		-segtime)
			shift
			currOpt="segtime"
			;;
		-seglist)
			shift
			currOpt="seglist"
			;;
		-dest)
			shift
			currOpt="dest"
			;;
		-debug)
			shift
			currOpt="debug"
			;;
		*)
			case "$currOpt" in									
				segtime)
					FFMPEG_SEG_TIME="$1"
					shift
					;;
				
				seglist)
					FFMPEG_SEG_PLAYLIST="$1"
					shift
					;;
				
				dest)
					FFMPEG_FILE_DEST="$1"
					shift
					;;
				debug)
					FFMPEG_DEBUG="$1"
					shift
					;;
				*)
					shift
					;;
			esac
			currOpt=""
			;;		
	esac
done

# check necessary parameters
if [ "$FFMPEG_SEG_TIME" = "" ];then
	cat >&2 <<ERR_MSG
$0: need to specify -segtime
ERR_MSG
	usage && exit 1
fi

if [ "$FFMPEG_SEG_PLAYLIST" = "" ];then
	cat >&2 <<ERR_MSG
$0: need to specify -seglist
ERR_MSG
	usage && exit 1
else
	TMP="$FFMPEG_SEG_PLAYLIST"
	FFMPEG_SEG_PLAYLIST="$TMP""$FFMPEG_PLAYLIST_ATTACHED_NAME"
fi
echo "ffmpeg_seg_playlist="$FFMPEG_SEG_PLAYLIST""

if [ "$FFMPEG_FILE_DEST" = "" ];then
	cat >&2 <<ERR_MSG
$0: need to specify -dest
ERR_MSG
	usage && exit 1
else
	TMP="$FFMPEG_FILE_DEST"
	FFMPEG_FILE_DEST="$TMP""$FFMPEG_FILE_ATTACHED_NAME"
fi

# generate cmd
FFMPEG_CMD=""$FFMPEG_BIN" -y \
-i - -strict -2 -c:a copy -c:v copy \
-bsf h264_mp4toannexb \
-flags -global_header -map 0 -f segment \
-segment_format mpegts \
-segment_time "$FFMPEG_SEG_TIME" \
-segment_list "$FFMPEG_SEG_PLAYLIST"  \
"$FFMPEG_FILE_DEST" "

# show command
if [ "$FFMPEG_DEBUG" -ge 1 ];then
cat >&2 <<FFMPEG
========================execute ffmpeg cmd====================================

$FFMPEG_CMD

==============================================================================
FFMPEG
fi
if [ "$FFMPEG_DEBUG" -eq 1 ];then
exit 0
fi

# execute command
if [ "$FFMPEG_DEBUG" -eq 2 ];then
exec $FFMPEG_CMD #show ffmpeg execution debug message
else
exec $FFMPEG_CMD 2>/dev/null
fi

exit $?