#!/bin/sh
HAL_EVENT="/sbin/hal_event"
GPHOTOFS="/usr/bin/gphotofs"
CONFIG_PATH="/etc/config/mtp"
CONFIG_STATUS="${CONFIG_PATH}/status"
LOCK_FILE="/tmp/mtp.lock"
PIC_RAW="/sbin/pic_raw"
HAL_APP="/sbin/hal_app"

#Check device
if [ ${DEVTYPE} != "usb_device" ]; then
    exit 0
fi

#Check environment
if [ ! -d ${CONFIG_PATH} ]; then
    /bin/mkdir ${CONFIG_PATH}
fi

SYSPATH="/sys${DEVPATH}"
PRODUCT=`/bin/cat ${SYSPATH}/product | /usr/bin/xargs /bin/echo`
MANUFACTURER=`/bin/cat ${SYSPATH}/manufacturer | /usr/bin/xargs /bin/echo`
IDVENDER=`/bin/cat ${SYSPATH}/idVendor | /usr/bin/xargs /bin/echo`
IDPRODUCT=`/bin/cat ${SYSPATH}/idProduct | /usr/bin/xargs /bin/echo`
SERIAL=`/bin/cat ${SYSPATH}/serial | /usr/bin/xargs /bin/echo`
MOUNTPATH="/share/external/${PRODUCT}"
TIMESTAMP="${CONFIG_PATH}/timestamp/${IDVENDER}/${IDPRODUCT}/${SERIAL}"
METADATA_D="${CONFIG_PATH}/metadata/${IDVENDER}/${IDPRODUCT}"
METADATA="${METADATA_D}/${SERIAL}"

_usb_led_on()
{
	SMART_IMPORT=`/sbin/getcfg "usb copy" "Camera Import Enable" -d "FALSE" -f /etc/config/uLinux.conf`
	if [ "${SMART_IMPORT}" != "TRUE" ]; then
		return 1;
	fi
	if [ -x ${HAL_APP} ]; then
		${HAL_APP} --se_sys_set_usb_led enc_sys_id=root,mode=1
	else
		${PIC_RAW} 96
	fi
}

_usb_led_off()
{
	SMART_IMPORT=`/sbin/getcfg "usb copy" "Camera Import Enable" -d "FALSE" -f /etc/config/uLinux.conf`
	if [ "${SMART_IMPORT}" != "TRUE" ]; then
		return 1;
	fi
	if [ -x ${HAL_APP} ]; then
		${HAL_APP} --se_sys_set_usb_led enc_sys_id=root,mode=0
	else
		${PIC_RAW} 98
	fi
}

_usb_led_blink()
{
	SMART_IMPORT=`/sbin/getcfg "usb copy" "Camera Import Enable" -d "FALSE" -f /etc/config/uLinux.conf`
	if [ "${SMART_IMPORT}" != "TRUE" ]; then
		return 1;
	fi
	if [ -x ${HAL_APP} ]; then
		${HAL_APP} --se_sys_set_usb_led enc_sys_id=root,mode=2
	else
		${PIC_RAW} 97
	fi
}

if [ -z "${PRODUCT}" ]; then
    PRODUCT="MTP_UNKNOWN"
    MOUNTPATH="/share/external/${PRODUCT}"
fi

if [ -z "${SERIAL}" ]; then
    SERIAL="unknown"
    TIMESTAMP="${CONFIG_PATH}/timestamp/${IDVENDER}/${IDPRODUCT}/${SERIAL}"
    METADATA="${METADATA_D}/${SERIAL}"
fi

if [ ${ACTION} == "add" ]; then
    #Add actions
    # 1. lock and check no other camera is mounted
    # 2. mount it
    # 3. create timestamp folder
    # 4. update metadata
    # 5. update status
    # 6. notify daemon and remove lock

    #exit immediately when
    # 1. IDVENDER or IDPRODUCT is empty
    # 2. bInterfaceClass is 8 (mass storage) [Workaround: MTP will let USB JBOD detection be very slow]
    if [ -z ${IDVENDER} ] || [ -z ${IDPRODUCT} ]; then
        exit 0
    else
        INTERFACE_DEVICE=`find "${SYSPATH}" -name bInterfaceClass`
        bInterfaceClass=`cat "${INTERFACE_DEVICE}"`
        if [ "${bInterfaceClass}" == "08" ] || [ "${bInterfaceClass}" == "8" ]; then
            exit 0
        fi
    fi

    #lock and check no other camera is mounted
    while [ -f "${LOCK_FILE}" ]; do
        LOCK_DEVICE=`/bin/cat "${LOCK_FILE}" | /usr/bin/xargs /bin/echo`
        if [ "${LOCK_DEVICE}" == "${DEVPATH}" ]; then
            exit 0
        fi
        /bin/sleep 1
    done

	_usb_led_blink
    /bin/touch "${LOCK_FILE}"
    /bin/echo "${DEVPATH}" > "${LOCK_FILE}"

    if [ -f "${CONFIG_STATUS}" ]; then
        /bin/rm -f "${LOCK_FILE}"
        _usb_led_off
        exit 0
    fi

    #mount it
    if [ ! -d "${MOUNTPATH}" ]; then
        /bin/mkdir -p "${MOUNTPATH}"
    fi
    /bin/umount "${MOUNTPATH}"
    LD_LIBRARY_PATH="/usr/lib/libfuse/" ${GPHOTOFS} "${MOUNTPATH}"

    /bin/sleep 5
    /bin/df "${MOUNTPATH}"

    if [ $? != 0 ]; then
        #this device doesn't support mtp/ptp
        /bin/umount "${MOUNTPATH}"
        /bin/rm -rf "${MOUNTPATH}"
        /bin/rm -f "${LOCK_FILE}"
        _usb_led_off
        exit 0
    fi

    #create timestamp folder
    if [ ! -d "${TIMESTAMP}" ]; then
        /bin/mkdir -p "${TIMESTAMP}"
    fi

    #update metadata
    if [ ! -d "${METADATA_D}" ]; then
        /bin/mkdir -p "${METADATA_D}"
    fi

    /bin/echo "product: ${PRODUCT}" > ${METADATA}
    /bin/echo "manufacturer: ${MANUFACTURER}" >> ${METADATA}
    /bin/echo "idVendor: ${IDVENDER}" >> ${METADATA}
    /bin/echo "idProduct: ${IDPRODUCT}" >> ${METADATA}
    /bin/echo "devpath: ${DEVPATH}" >> ${METADATA}
    /bin/echo "serial: ${SERIAL}" >> ${METADATA}
    /bin/echo "mountpath: ${MOUNTPATH}" >> ${METADATA}
    /bin/echo "timestamp_path: ${TIMESTAMP}" >> ${METADATA}

    #update status
    /bin/echo "${DEVPATH}" > ${CONFIG_STATUS}
    /bin/echo "${SERIAL}" >> ${CONFIG_STATUS}
    /bin/echo "${IDVENDER}" >> ${CONFIG_STATUS}
    /bin/echo "${IDPRODUCT}" >> ${CONFIG_STATUS}
    /bin/echo "${MOUNTPATH}" >> ${CONFIG_STATUS}

    #notify daemon and remove lock
    if [ -f "${HAL_EVENT}" ]; then
        #HAL mode
        ${HAL_EVENT} --mtp_add
    else
        #Normal mode
        SMART_IMPORT=`/sbin/getcfg "usb copy" "Camera Import Enable" -d "FALSE" -f /etc/config/uLinux.conf`
        SMART_IMPORT_BUTTON=`/sbin/getcfg "usb copy" "Camera Import Button" -d "TRUE" -f /etc/config/uLinux.conf`

        if [ "${SMART_IMPORT}" == "TRUE" ] && [ "${SMART_IMPORT_BUTTON}" != "TRUE" ]; then
            /sbin/camera_import mtp 1>>/dev/null 2>>/dev/null &
        fi
    fi
    /bin/rm -f "${LOCK_FILE}"
    _usb_led_on

elif [ ${ACTION} == "remove" ]; then
    #Remove action
    # 1. umount it and remove the mount point
    # 2. update status
    # 3. notify daemon

    devicepath=`/usr/bin/head -n 1 ${CONFIG_STATUS}`
    if [ "${devicepath}" == "${DEVPATH}" ]; then
        #umount it and remove the mount point
        mount_point=`/usr/bin/tail -n 1 ${CONFIG_STATUS}`
        /bin/umount "${mount_point}"
        /bin/rm -rf "${mount_point}"

        #update status
        /bin/rm -f ${CONFIG_STATUS}

        #notify daemon
        if [ -f "${HAL_EVENT}" ]; then
            #HAL mode
            ${HAL_EVENT} --mtp_remove
        else
            #Normal mode
            SMART_IMPORT=`/sbin/getcfg "usb copy" "Camera Import Enable" -d "FALSE" -f /etc/config/uLinux.conf`

            if [ "${SMART_IMPORT}" == "TRUE" ]; then
                /sbin/camera_import stop 1>>/dev/null 2>>/dev/null &
            fi
        fi
        _usb_led_off
    fi
fi

exit 0
