#/bin/sh

AD_DOMAIN=$2	# fred.local
AD_USER=$3	# Administrator
AD_PASSWD=$4	# Admin!@#$%
AD_SERVER=$5	# PRIMARY-DC-DNS
AD_WG=$6	# FRED
AD_OU=$7

#
# Redundant DC used
#

AD_SERVER2=$8
SRV1=
SRV2=
KDC1=
KDC2=

#
# Debug Setting
#

QTS_DBG_ENABLE=0
QTS_DBG_FILE=/tmp/qts_dc_dbg

if [ "x$1" = "xupdate_krb5_ticket" ]; then
	AD_PASSWD=$2
fi

if [ "x$1" = "xADM" ]; then
DEBUG_FILE=/var/log/setup_smb.debug
/bin/rm -f /home/httpd/cgi-bin/setup_smb.debug
/bin/ln -sf ${DEBUG_FILE} /home/httpd/cgi-bin/setup_smb.debug
else
DEBUG_FILE=/dev/null
fi

# constants
SUCCESS=0
USAGE_ERROR=1
UNKNOWN_ERROR=2
KDC_SRV_ERROR=3
KDC_REALM_ERROR=4
KDC_TIME_DIFF_ERROR=5
AD_JOIN_ERROR=6
NTD_JOIN_ERROR=7
KDC_SRV_ADNAME_ERROR=8
ALL_DC_DEAD=9

# utility 
SED=/bin/sed
UPPERCASE=/usr/bin/uppercase
LOWERCASE=/usr/bin/lowercase
GETKDC=/usr/bin/getKDC
KINIT=/usr/bin/kinit
GREP=/bin/grep
CUT=/bin/cut
SETCFG=/sbin/setcfg
GETCFG=/sbin/getcfg
NET=/usr/local/samba/bin/net
WBINFO=/usr/bin/wbinfo

# config file
KRB5_TMPL=/etc/krb5_tmpl.conf
# Redundant Domain Controller Only
KRB5_TMPL_RDDC=/etc/krb5_tmpl_rddc.conf
KRB5_CONF=/etc/config/krb5.conf
SMB_CONF=/etc/config/smb.conf
IDMAP_CONF=/etc/config/idmap.conf
PROGRESS_FILE=/tmp/join_ad_progress.conf

# functions

# progress
#   configuration	10%
#   synchronization	20%
#   authentication	40%
#   joining_ad		60%
#   starting_service	80%
#   finish	       100% 	    
write_progress(){
	state=$1
	/bin/touch $PROGRESS_FILE
	/sbin/rmcfg ad -f $PROGRESS_FILE
	case ${state} in
	configuration)
		$SETCFG ad "progress" "10" -f $PROGRESS_FILE
		$SETCFG ad "state" "$state" -f $PROGRESS_FILE
	;;
	synchronization)
		$SETCFG ad "progress" "20" -f $PROGRESS_FILE
		$SETCFG ad "state" "$state" -f $PROGRESS_FILE
	;;
	authentication)
		$SETCFG ad "progress" "40" -f $PROGRESS_FILE
		$SETCFG ad "state" "$state" -f $PROGRESS_FILE
	;;
	joining_ad)
		$SETCFG ad "progress" "60" -f $PROGRESS_FILE
		$SETCFG ad "state" "$state" -f $PROGRESS_FILE
	;;
	starting_service)
		$SETCFG ad "progress" "80" -f $PROGRESS_FILE
		$SETCFG ad "state" "$state" -f $PROGRESS_FILE
	;;
	finish)
		$SETCFG ad "progress" "100" -f $PROGRESS_FILE
		$SETCFG ad "state" "$state" -f $PROGRESS_FILE
	;;
	*)
		$SETCFG ad "state" "$state" -f $PROGRESS_FILE
	;;
	esac
}

function backup_conf_file()
{
	SMB_TMP=`mktemp $SMB_CONF.XXXXXX`
	if [ -f $KRB5_CONF ]; then
		KRB5_TMP=`mktemp $KRB5_CONF.XXXXXX` 
		cp -af $KRB5_CONF $KRB5_TMP
		dbg "backup $KRB5_CONF to $KRB5_TMP"
	fi
	cp -af $SMB_CONF $SMB_TMP

	dbg "backup $SMB_CONF to $SMB_TMP"
}

function restore_conf_file()
{
	dbg "restore config file"
	dbg "$KRB5_TMP to $KRB5_CONF"
	dbg "$SMB_TMP to $SMB_CONF"
	[ ! -f "$KRB5_TMP" ] || mv $KRB5_TMP $KRB5_CONF
	mv $SMB_TMP $SMB_CONF

}

function clean_backup_file()
{
	dbg "clean backup file $KRB5_TMP and $SMB_TMP"

	#echo "enter clean_backup_file"
	[ ! -f "$KRB5_TMP" ] || rm -f $KRB5_TMP
	rm -f $SMB_TMP
}

add_printer_admin_main(){
	printers_support=`$GETCFG Printers Support`
        printers_enable=`$GETCFG Printers Enable`
        if [ "x${printers_enable}" = "xTRUE" ]; then
		echo $printers_support > /tmp/log
		echo $printers_enable >> /tmp/log
        	add_printer_admin
                add_printer_admin
                /etc/init.d/smb.sh restart > /dev/null
        fi
}
add_printer_admin(){
	user=""
        for x in `wbinfo -g | grep "Domain Users"`; do
                if [ x"$x" != x"Users" ]; then
                        user=$user"@\""$x\ Users\",
                fi
        done
        user=$user@everyone,guest
        /sbin/setcfg Printers "printer admin" "$user" -f $LOCAL_SMB_CONF -c
        return
}

function clean_smb_conf()
{
	dbg "clean samba config: $1"

	LOCAL_SMB_CONF=$1	
	local TMP_FILE=/tmp/tmp.backend
	$SETCFG global 'character set' -f $LOCAL_SMB_CONF -e
	$SETCFG global 'name resolve order' -f $LOCAL_SMB_CONF -e
	$SETCFG global 'realm' -f $LOCAL_SMB_CONF -e
	$SETCFG global 'ldap timeout' -f $LOCAL_SMB_CONF -e
	$SETCFG global 'password server' -f $LOCAL_SMB_CONF -e
	$SETCFG global 'pam password change' -f $LOCAL_SMB_CONF -e
##	$SETCFG global 'winbind separator' -f $LOCAL_SMB_CONF -e
	$SETCFG global 'idmap uid' -f $LOCAL_SMB_CONF -e
	$SETCFG global 'idmap gid' -f $LOCAL_SMB_CONF -e
	$SETCFG global 'winbind enum users' -f $LOCAL_SMB_CONF -e
	$SETCFG global 'winbind enum groups' -f $LOCAL_SMB_CONF -e
	$SETCFG global 'winbind cache time' -f $LOCAL_SMB_CONF -e
	$SETCFG Printers 'printer admin' -f $LOCAL_SMB_CONF -e

	/bin/grep ": backend" $LOCAL_SMB_CONF > $TMP_FILE
	while read idmap_backend; do
	    [ "$idmap_backend" ] && $SETCFG global "${idmap_backend% =*}" -f $LOCAL_SMB_CONF -e
	done < $TMP_FILE

	/bin/grep ": range" $LOCAL_SMB_CONF > $TMP_FILE
	while read idmap_range; do
	    [ "$idmap_range" ] && $SETCFG global "${idmap_range% =*}" -f $LOCAL_SMB_CONF -e
	done < $TMP_FILE

	/etc/init.d/hostname.sh
}

function clean_winbind_cache()
{
	dbg "clean winbind cache"

	volume_test=`/sbin/getcfg Public path -f /etc/smb.conf | cut -d '/' -f 3`
	[ "x${volume_test}" = "x" ] || volume=${volume_test}
	if [ -d /share/${volume}/.locks ]; then
		cd /share/${volume}/.locks
		/bin/rm -f /share/${volume}/.locks/* -rf
	fi
}

clean_domain_db() {
	/bin/rm -f /mnt/HDA_ROOT/.domain_user
	/bin/rm -f /mnt/HDA_ROOT/.domain_group
}

alone_set_smb_conf() {
	LOCAL_SMB_CONF=$1
	$SETCFG global workgroup "$AD_DOMAIN" -f $LOCAL_SMB_CONF
	$SETCFG global security USER -f $LOCAL_SMB_CONF
	$SETCFG System Workgroup "$AD_DOMAIN"
}

join_ntd() {
	LOCAL_SMB_CONF=$1
	domain_name=`${LOWERCASE} $2`
	wg_name=`${UPPERCASE} ${domain_name}`
	user_name=$3
	passwd=$4
	_winbind_separator=`${GETCFG} Samba "winbind separator" -d "\\\\"`

	$SETCFG global 'security' DOMAIN -f $LOCAL_SMB_CONF
	$SETCFG global 'workgroup' $wg_name -f $LOCAL_SMB_CONF
	$SETCFG global 'realm' $domain_name -f $LOCAL_SMB_CONF
	$SETCFG global 'password server' '*' -f $LOCAL_SMB_CONF
	$SETCFG global 'pam password change' yes -f $LOCAL_SMB_CONF
	if [ "x${_winbind_separator}" != "x+" ]; then
		$SETCFG -e global 'winbind separator' -f $LOCAL_SMB_CONF
	else
		$SETCFG global 'winbind separator' ${_winbind_separator} -f $LOCAL_SMB_CONF
	fi
	$SETCFG global 'idmap uid' '30001-300000' -f $LOCAL_SMB_CONF
	$SETCFG global 'idmap gid' '30001-300000' -f $LOCAL_SMB_CONF
	$SETCFG global 'winbind enum users' yes -f $LOCAL_SMB_CONF
	$SETCFG global 'winbind enum groups' yes -f $LOCAL_SMB_CONF
	$SETCFG global 'winbind cache time' '3600' -f $LOCAL_SMB_CONF
	$SETCFG System Workgroup $wg_name

	/bin/echo "[command] ${NET} rpc join -U \"$user_name%********\" -s $SMB_CONF" >> $DEBUG_FILE
	${NET} rpc join -U "$user_name%$passwd" -s $SMB_CONF >> $DEBUG_FILE 2>&1
	if [ "$?" != "0" ]; then 
		return 2
	fi
	return 0
}

#
# function log, show log message to $DEBUG_FILE
#
# $1 = log message
#

function log()
{
	echo "$1" >> $DEBUG_FILE
}

#
# function dbg, show debug message to $QTS_DBG_FILE
#

function dbg()
{
	if [ "$#" == "0" ]; then
		return 1
	fi

	if [ -z "$QTS_DBG_ENABLE" ] || [ "$QTS_DBG_ENABLE" == "0" ]; then
		return 2
	fi

	if [ -z "$QTS_DBG_FILE" ]; then
		return 3
	fi

	local DATE=`date|awk -F " " '{print $4}'`

	echo "$DATE $1" >> $QTS_DBG_FILE

	return 0
}

function dbg_start()
{
	if [ -z "$QTS_DBG_ENABLE" ] || [ "$QTS_DBG_ENABLE" == "0" ]; then
		return 1
	fi

	echo "=== QTS DEBUG START ===" > $QTS_DBG_FILE

	return 0
}

function dbg_end()
{
	if [ -z "$QTS_DBG_ENABLE" ] || [ "$QTS_DBG_ENABLE" == "0" ]; then
		return 1
	fi

	echo "=== QTS DEBUG END ===" >> $QTS_DBG_FILE

	return 0
}

#
# function ads_join_ad, join AD via ADS protocol.
#
# $1 = $AD_SERVER_NAME
# $2 = $AD_USER
# $3 = $AD_PASSWD
# $4 = $AD_OU(option)
#

function ads_join_ad()
{
	#
	# command definition
	#

	[ $# -lt 3 ] || [ $# -gt 4 ] && return 1

	ads_cmd="${NET} ads join"

	if [ "$4" != "" ]; then
		ads_cmd="${ads_cmd} createcomputer=$4"
	fi

	if [ "$1" != "" ]; then
		ads_cmd="${ads_cmd} -S ${1}"
	fi

	#
	# write debug log and execute command
	#

	dbg "${ads_cmd} -U \"${2}%*******\" -s $SMB_CONF"
	log "[command] ${ads_cmd} -U \"${2}%********\" -s $SMB_CONF"
	${ads_cmd} -U "$2%$3" -s $SMB_CONF >> $DEBUG_FILE 2>&1
}

#
# function rpc_join_ad, join AD via RPC protocol.
#
# $1 = $AD_SERVER_NAME
# $2 = $AD_USER
# $3 = $AD_PASSWD
# $4 = $AD_OU(option)
#

function rpc_join_ad()
{
	#
	# command definition
	#

	[ $# -lt 3 ] || [ $# -gt 4 ] && return 1

	rpc_cmd="${NET} rpc join"

	if [ "$4" != "" ]; then
		rpc_cmd="${rpc_cmd} createcomputer=$4"
	fi

	if [ "$1" != "" ]; then
		rpc_cmd="${rpc_cmd} -S ${1}"
	fi

	#
	# write debug log and execute command
	#

	dbg "${rpc_cmd} -U \"${2}%*******\" -s $SMB_CONF"
	log "[command] ${rpc_cmd} -U \"${2}%********\" -s $SMB_CONF"
	${rpc_cmd} -U "$2%$3" -s $SMB_CONF >> $DEBUG_FILE 2>&1
}

#
# function __join_ad, most important here.
# NAS join to particular DC or hava redundant DC
#
# No parameter
#

function __join_ad()
{
	dbg "__join_ad"
	if [ ! -z "${AD_OU}" ]; then
		dbg "join DC with OU parameter: \"${AD_OU}\""
		local OU_PARAM="${AD_OU}"
	fi

	local srv=$(string_split ${AD_SERVER})

	for i in ${srv}
	do
		local kdc=${i}.${AD_DOMAIN}
		dbg "srv=${i} kdc=${kdc}"

		ads_join_ad ${i} "${AD_USER}" "${AD_PASSWD}" "${OU_PARAM}"
		[ $? -eq 0 ] && return 0

		ads_join_ad ${kdc} "${AD_USER}" "${AD_PASSWD}" "${OU_PARAM}"
		[ $? -eq 0 ] && return 0
	done

	ads_join_ad "" "${AD_USER}" "${AD_PASSWD}" "${OU_PARAM}"
	[ $? -eq 0 ] && return 0

	for i in ${srv}
	do
		local kdc=${i}.${AD_DOMAIN}

		rpc_join_ad ${i} "${AD_USER}" "${AD_PASSWD}" "${OU_PARAM}"
		[ $? -eq 0 ] && return 0

		rpc_join_ad ${kdc} "${AD_USER}" "${AD_PASSWD}" "${OU_PARAM}"
		[ $? -eq 0 ] && return 0
	done

	rpc_join_ad "" "${AD_USER}" "${AD_PASSWD}" "${OU_PARAM}"

#	ads_join_ad ${SRV1} ${AD_USER} ${AD_PASSWD} ${OU_PARAM}

#	if [ "$?" != "0" ]; then
#		ads_join_ad ${KDC1} ${AD_USER} ${AD_PASSWD} ${OU_PARAM}
#	fi
#	if [ "$?" != "0" ] && [ ! -z "${SRV2}" ]; then
#		ads_join_ad ${SRV2} ${AD_USER} ${AD_PASSWD} ${OU_PARAM}
#	fi
#	if [ "$?" != "0" ] && [ ! -z "${KDC2}" ]; then
#		ads_join_ad ${KDC2} ${AD_USER} ${AD_PASSWD} ${OU_PARAM}
#	fi
#	if [ "$?" != "0" ]; then
#		ads_join_ad "" ${AD_USER} ${AD_PASSWD} ${OU_PARAM}
#	fi
#	if [ "$?" != "0" ]; then
#		rpc_join_ad ${SRV1} ${AD_USER} ${AD_PASSWD} ${OU_PARAM}
#	fi
#	if [ "$?" != "0" ]; then
#		rpc_join_ad ${KDC1} ${AD_USER} ${AD_PASSWD} ${OU_PARAM}
#	fi
#	if [ "$?" != "0" ] && [ ! -z "${SRV2}" ]; then
#		rpc_join_ad ${SRV2} ${AD_USER} ${AD_PASSWD} ${OU_PARAM}
#	fi
#	if [ "$?" != "0" ] && [ ! -z "${KDC2}" ]; then
#		rpc_join_ad ${KDC2} ${AD_USER} ${AD_PASSWD} ${OU_PARAM}
#	fi
#	if [ "$?" != "0" ]; then
#		rpc_join_ad "" ${AD_USER} ${AD_PASSWD} ${OU_PARAM}
#	fi
}

#
# function join_ad, all join preparation here
# including samba config and kerberos config
#
# $1 = Samba config, typically /etc/config/smb.conf
# $2 = $AD_DOMAIN = Domain Name
# $3 = $AD_USER   = User Name
# $4 = $AD_PASSWD = Password
# $5 = $AD_SERVER = Server Name
#

function join_ad()
{
	dbg "join_ad start"

	LOCAL_SMB_CONF=$1
	domain_name=`${LOWERCASE} ${AD_DOMAIN}`
	if [ "x${AD_WG}" != "x" ]; then
		wg_name=`${UPPERCASE} ${AD_WG}`
		log "Specify WORKGROUP = ${wg_name}"
	else
		wg_name=`${UPPERCASE} ${domain_name} | ${CUT} -d . -f 1`
		log "Domain prefix WORKGROUP = ${wg_name}"
	fi
	
	user_name=${AD_USER}
	passwd=${AD_PASSWD}
	_winbind_separator=`${GETCFG} Samba "winbind separator" -d "\\\\"`

	if [ "x${AD_SERVER}" != "x" ]; then
		kdc=$(get_multi_kdc)
	else
		log "[command] ${GETKDC} ${lc_name}"
		${GETKDC} ${lc_name} >> $DEBUG_FILE
		if [ "$?" != "0" ]; then
			echo "get KDC failed!"
			return 1
		fi
		kdc=`${GETKDC} ${lc_name}`
	fi

	#
	# Set samba config item, example:
	#	security  = ADS
	#	workgroup = FRED from "fred".local uppercase
	#	realm     = fred.local
	#	password server     = primary-dc-dns.fred.local
	#	pam password change = yes
	#	winbind separator   = (empty)
	#

	$SETCFG global 'security' ADS -f $LOCAL_SMB_CONF
	$SETCFG global 'workgroup' $wg_name -f $LOCAL_SMB_CONF
	$SETCFG global 'realm' $domain_name -f $LOCAL_SMB_CONF
	$SETCFG global 'ldap timeout' 5 -f $LOCAL_SMB_CONF

	#
	# create target DC to password server option.
	#

	dbg "create dc list to $LOCAL_SMB_CONF"

	if [ "${AD_SERVER}" == "*" ]; then
		$SETCFG global 'password server' "*" -f $LOCAL_SMB_CONF
		dbg "create wildcard password server"
	else
		$SETCFG global 'password server' "${kdc}" -f $LOCAL_SMB_CONF
		dbg "create particular password server $kdc"
	fi

	$SETCFG global 'pam password change' yes -f $LOCAL_SMB_CONF
	if [ "x${_winbind_separator}" != "x+" ]; then
		$SETCFG -e global 'winbind separator' -f $LOCAL_SMB_CONF
	else
		$SETCFG global 'winbind separator' ${_winbind_separator} -f $LOCAL_SMB_CONF
	fi

	#
	# Set samba config item, example:
	#	winbind enum users  = yes
	#	winbind enum groups = yes
	#	winbind cache time  = 3600
	#
	# Set uLinux.config
	#	Workgroup = FRED
	#

	$SETCFG global 'winbind enum users' yes -f $LOCAL_SMB_CONF
	$SETCFG global 'winbind enum groups' yes -f $LOCAL_SMB_CONF
	$SETCFG global 'winbind cache time' '3600' -f $LOCAL_SMB_CONF
	$SETCFG System Workgroup $wg_name

	/etc/init.d/hostname.sh
	/bin/cp /etc/config/smb.conf /home/httpd/cgi-bin/smb.conf.debug

	#
	# Join AD without Server Name
	#

	if [ "x${AD_SERVER}" = "x*" ]; then
		dbg "join ad without server name"
		if [ "x${AD_OU}" != "x" ]; then
			local OU_PARAM="\"${AD_OU}\""
		fi

		ads_join_ad "" "${AD_USER}" "${AD_PASSWD}" "${OU_PARAM}"

		if [ "$?" != "0" ]; then
			rpc_join_ad "" "${AD_USER}" "${AD_PASSWD}" "${OU_PARAM}"
		fi

		return 0
	fi

	#
	# Join AD with Server Name
	#

	AD_SERVER_NAME=`/sbin/getcfg global "password server" -f /etc/config/smb.conf  | /bin/cut -d. -f1`

	__join_ad

	if [ "$?" != "0" ]; then
		dbg "join fail"
		return 2
	fi

	dbg "join success, prepare samba config"

	local idmap_rid=`${GETCFG} Samba "idmap_rid" -d "FALSE"`
	local default_range=0
	local smb_version=`/usr/local/samba/sbin/smbd -V`
	if [ "${idmap_rid}" != "FALSE" ]; then
		if [ -f "${IDMAP_CONF}" ]; then
			while read LINE
			do
				str=`/bin/echo "$LINE" | tr -d ' '`
				_netbios_name=${str%%=*}
				range=${str##*=}
				if [ "$_netbios_name" == "" -o "$_netbios_name" == "" ]; then
					continue
				fi
				if [ "$_netbios_name" == "system_default_range" ]; then
					default_range=1
					if [ ! "$smb_version" \< "Version 3.6.0" ]; then
						$SETCFG global "idmap config * : backend" 'tdb' -f $LOCAL_SMB_CONF
						$SETCFG global "idmap config * : range" "$system_rid_range" -f $LOCAL_SMB_CONF
					else
						$SETCFG global "idmap uid" "$range" -f $LOCAL_SMB_CONF
						$SETCFG global "idmap gid" "$range" -f $LOCAL_SMB_CONF
					fi
				else
					$SETCFG global "idmap config ${_netbios_name} : backend" 'rid' -f $LOCAL_SMB_CONF
					$SETCFG global "idmap config ${_netbios_name} : range" "$range" -f $LOCAL_SMB_CONF
				fi
			done < "$IDMAP_CONF"
			if [ "$default_range" == "0" ]; then
				$SETCFG global 'idmap uid' '400001-500000' -f $LOCAL_SMB_CONF
				$SETCFG global 'idmap gid' '400001-500000' -f $LOCAL_SMB_CONF
			fi
		else
		#  domain mapping table
		# +---------------------+
		# |  30001  ~   300000  |  main domain
		# +---------------------+
		# | 300001  ~   400000  |  reserve 
		# +---------------------+
		# | 400001  ~   500000  |  default assignment
		# +---------------------+  -->trust_domain_user_index
		# | max_user_per_domain |  \ 
		# |         .           |   max_trust_domain 
		# |         .           |  /
		# +---------------------+
		local max_trust_domain=`$GETCFG Samba "Max_Trust_Domain" -d 5`
		local max_user_per_domain=`$GETCFG Samba "Max_User_Per_Trust_Domain" -d 100000`
		local trust_domain_user_index=`$GETCFG Samba "Trust_Domain_User_Index" -d 500001`
		local system_rid_range=`$GETCFG Samba "System_Rid_Range" -d '400001-500000'`
		local main_domain_rid_range=`$GETCFG Samba "Main_Domain_Rid_Range" -d '30001-300000'`
		local i=0
		local start_uid=0
		local end_uid=0
		local first_loop=1
		local _trusted_netbios=/tmp/.trusted_netbios
		##if [ $max_user_per_domain -gt 200000 ]; then
		##	max_user_per_domain=200000
		##	echo "assign max= $max_user_per_domain"
		##fi
		if [ ! "$smb_version" \< "Version 3.6.0" ]; then
			$SETCFG global "idmap config * : backend" 'tdb' -f $LOCAL_SMB_CONF
			$SETCFG global "idmap config * : range" "$system_rid_range" -f $LOCAL_SMB_CONF
		else
			$SETCFG global 'idmap uid' "$system_rid_range" -f $LOCAL_SMB_CONF
			$SETCFG global 'idmap gid' "$system_rid_range" -f $LOCAL_SMB_CONF
		fi
		$SETCFG global "idmap config $wg_name : backend" 'rid' -f $LOCAL_SMB_CONF
		$SETCFG global "idmap config $wg_name : range" "$main_domain_rid_range" -f $LOCAL_SMB_CONF
		##$SETCFG global "idmap config $wg_name : base_rid" '100' -f $LOCAL_SMB_CONF
		##$SETCFG global "idmap backend" 'rid' -f $LOCAL_SMB_CONF
		/bin/rm /tmp/.trusted_domain
                /bin/rm /tmp/.trusted_netbios
		/etc/init.d/winbind restart
		/sbin/get_trusted_domain  | /bin/cut -f 3 -d '[' | /bin/cut -f 1 -d ']' > ${_trusted_netbios}
		while read _netbios_name && [ "$i" -le "$max_trust_domain" ] ; do
			if [ "$first_loop" = 1 ]; then
				first_loop=2
				continue
			fi
			let "start_uid=trust_domain_user_index+max_user_per_domain*i"
			let "i++"
			let "end_uid=trust_domain_user_index+max_user_per_domain*i-1"
			$SETCFG global "idmap config ${_netbios_name} : backend" 'rid' -f $LOCAL_SMB_CONF
			$SETCFG global "idmap config ${_netbios_name} : range" "$start_uid-$end_uid" -f $LOCAL_SMB_CONF
		done < ${_trusted_netbios}
		fi
	else
		$SETCFG global 'idmap uid' '30001-500000' -f $LOCAL_SMB_CONF
		$SETCFG global 'idmap gid' '30001-500000' -f $LOCAL_SMB_CONF
	fi

	dbg "join_ad end"

	return 0
}

#
# function find_dc_ip, find FQDN's IP using /bin/host
#
# $1 = FQDN
#

function find_dc_ip()
{
	if [ -z "$1" ]; then
		echo "0.0.0.0"
		return
	fi

	local TMP_FILE=/tmp/find_dc_ip
	/bin/host $1|head -1|grep "has address" > ${TMP_FILE} 2>&1

	if [ "$?" != "0" ]; then
		echo "0.0.0.0"
	else
		echo `cat ${TMP_FILE}|cut -d ' ' -f4`
	fi

	rm -rf $TMP_FILE
}

#
# function net_sync_time, sync system time with DC
#
# $1 = DC's FQDN
#

function net_sync_time()
{
	if [ -z "$1" ]; then
		return
	fi

	dbg "time sync to ${1}..."
	log "${NET} time set -S ${1}"
	${NET} time set -S ${1} > /dev/null 2>&1

	if [ "$?" != "0" ]; then
		dbg "time sync to ${1} fail, try to find ${1}'s IP"
		log "Sync time with domain name fail, try to sync time with IP"
		local kdc_ip=$(find_dc_ip ${1})
		if [ "$kdc_ip" == "0.0.0.0" ]; then
			# Cannot find DC's IP
			dbg "cannot find ${1}'s IP, time sync fail"
			log "Cannot find ${1}'s IP"
			echo "1"
		else
			dbg "time sync to ${kdc_ip}..."
			log "${NET} time set -S ${kdc_ip}"
			${NET} time set -S ${kdc_ip} > /dev/null 2>&1
			if [ "$?" != "0" ]; then
				# Sync time with DC'S IP fail
				dbg "time sync to ${kdc_ip} fail"
				log "Sync time fail with IP"
				echo "2"
			else
				# Sync time with DC's IP success
				dbg "time sync to ${kdc_ip} success"
				echo "0"
			fi
		fi
	else
		# Sync time with DC's FQDN success
		dbg "time sync to ${1} success"
		echo "0"
	fi
}

#
# function swap, swap two variables
#
# $1 = variable 1
# $2 = variable 2
#

function swap()
{
	eval "$1=${!2} $2=${!1}"
}

function string_split()
{
	local IFS=,
	set $1

	for ((i=1; i<= $#; ++i));
	do
		CMD="$`echo $i`"
		eval echo $CMD
	done
}

function get_multi_kdc()
{
	local AD_SRV_ARRAY=$(string_split ${AD_SERVER})

	for i in ${AD_SRV_ARRAY}
	do
		echo -n "${i}.${AD_DOMAIN} "
	done
}

#
# function multi_ad_server
# return:
#    true    setup_smb.sh revice two or more DC server
#   false    setup_smb.sh revice only one DC server
#

function multi_ad_server()
{
	local flag=`echo $AD_SERVER|awk '{print match($0,",")}'`

	if [ $flag -gt 0 ]; then
		echo "true";
	else
		echo "false";
	fi
}

#
# function setup_krb5
#	1. find alive DNS and change order into /etc/resolv.conf
#	2. find alive DC and change order for join
#	3. sync time
#	4. create kerberos config
#	5. kinit test
#

function setup_krb5()
{
	# lc_name   = fred.local    = ${AD_DOMAIN}
	# uc_name   = FRED.LOCAL
	# user_name = Administrator = ${AD_USER}
	# passwd    = Admin!@#$%    = ${AD_PASSWD}
	# AD_SERVER = PRIMARY-DC-DNS

	dbg "setup kerberos 5 start"

	lc_name=`${LOWERCASE} ${AD_DOMAIN}`
	uc_name=`${UPPERCASE} ${lc_name}`
	user_name=${AD_USER}
	passwd=${AD_PASSWD}
	write_progress synchronization 
	if [ "x${AD_SERVER}" != "x" ]; then
		if [ "x${AD_SERVER}" = "x*" ]; then
			kdc=${AD_DOMAIN}
		elif [ "$(multi_ad_server)" == "false" ]; then
			kdc=${AD_SERVER}.${AD_DOMAIN}
		else
			kdc=$(get_multi_kdc)
		fi
	else
		echo "[command] ${GETKDC} ${lc_name}" >> $DEBUG_FILE
		${GETKDC} ${lc_name} >> $DEBUG_FILE
		if [ "$?" != "0" ]; then
			echo "get KDC failed!"
			return 1
		fi
		kdc=`${GETKDC} ${lc_name}`
	fi

#	if [ ! -z "$AD_SERVER2" ]; then
#		kdc2=${AD_SERVER2}.${AD_DOMAIN}
#	else
#		kdc2=""
#	fi

#	dbg "original KDC order"
#	dbg "SRV1 = $AD_SERVER"
#	dbg "KDC1 = $kdc"
#	dbg "SRV2 = $AD_SERVER2"
#	dbg "KDC2 = $kdc2"

	dbg "Get KDC from AD_SERVER and AD_DOMAIN"
	dbg "KDC may single or multi FQDN"
	dbg "KDC = $kdc"

	#
	# DNS alive detection
	#

#	conf_dns_type=`$GETCFG Network "DNS type" -d unknown`
#	dbg "DNS type = $conf_dns_type"

#	if [ "$conf_dns_type" != "auto" ]; then
#		conf_dns1=`$GETCFG Network "Domain Name Server 1" -d 0.0.0.0`
#		conf_dns2=`$GETCFG Network "Domain Name Server 2" -d 0.0.0.0`
#		conf_dns3=`$GETCFG Network "Domain Name Server 3" -d 0.0.0.0`

#		dbg "original DNS order"
#		dbg "DNS1 = $conf_dns1"
#		dbg "DNS2 = $conf_dns2"
#		dbg "DNS3 = $conf_dns3"

#		nslookup ${kdc} ${conf_dns1}
#		if [ "$?" != "0" ]; then
#			swap conf_dns1 conf_dns3
#			swap conf_dns1 conf_dns2

#			nslookup ${kdc} ${conf_dns1}
#			if [ "$?" != "0" ]; then
#				swap conf_dns1 conf_dns3
#				swap conf_dns1 conf_dns2
#			fi
#		fi

#		$SETCFG Network "Domain Name Server 1" $conf_dns1
#		$SETCFG Network "Domain Name Server 2" $conf_dns2
#		$SETCFG Network "Domain Name Server 3" $conf_dns3

#		/etc/init.d/network.sh writedns > /dev/null 2>&1

#		dbg "new DNS order"
#		dbg "DNS1 = $conf_dns1"
#		dbg "DNS2 = $conf_dns2"
#		dbg "DNS3 = $conf_dns3"
#	fi

	#
	# Change DC to correct order
	# alive DC be kdc, dead DC be kdc2
	#

#	if [ ! -z "${kdc2}" ]; then
#		/usr/local/samba/bin/net ads ping ${kdc}
#		if [ "$?" != "0" ]; then
#			swap kdc kdc2
#			/usr/local/samba/bin/net ads ping ${kdc}
#			if [ "$?" != "0" ]; then
#				dbg "DC ping failed: all DC dead"
#				log "DC ping failed: all DC dead"
#				return 9
#			fi
#		fi
#	fi

#	KDC1=${kdc}
#	SRV1=`echo $KDC1|cut -d '.' -f 1`

#	if [ ! -z "${kdc2}" ]; then
#		KDC2=${kdc2}
#		SRV2=`echo $KDC2|cut -d '.' -f 1`
#	fi

#	dbg "new KDC order"
#	dbg "SRV1 = $SRV1"
#	dbg "KDC1 = $KDC1"
#	dbg "SRV2 = $SRV2"
#	dbg "KDC2 = $KDC2"

	#
	# Sync time.
	# 1. Select DC if we have redundant DC
	# 2. Try DC's FQDN first. If FQDN no response, try DC's IP.
	#

	dbg "time sync process start"

#	if [ -z "${kdc2}" ]; then
#		# only one DC used
#		net_sync_time ${kdc}
#	else
#		# Redundant DC used
#		for i in ${kdc} ${kdc2}
#		do
#			sync_res=$(net_sync_time ${i})
#			if [ "$sync_res" == "0" ]; then
#				break;
#			fi
#		done
#	fi

	for i in ${kdc}
	do
		sync_res=$(net_sync_time ${i})
		if [ "$sync_res" == "0" ]; then
			break;
		fi
	done

	dbg "time sync process end"

	#
	# Create /etc/krb5.conf
	# 
	# Skelton file is /etc/krb5_tmpl.conf or /etc/krb5_tmpl_rddc.conf
	#	AD_SERVER2 = Slave DC's hostname
	#	kdc        = Master DC's FQDN
	#	kdc2       = Slave DC's FQDN
	#	uc_name    = Domain Name(uppercase)
	#	lc_name    = Domain Name(lowercase)
	#

	if [ -f ${KRB5_CONF} ]; then
		# Delete old config
		dbg "remove kerberos config ${KRB5_CONF}"
		rm -rf ${KRB5_CONF}
	fi

	if [ "$(multi_ad_server)" == "false" ]; then
		# Single DC
		dbg "create kerberos config with only one DC"
		${SED} "s/##QNAP_REALM##/${uc_name}/g" ${KRB5_TMPL} | \
		${SED} "s/##QNAP_DOMAIN##/${lc_name}/g" | \
		${SED} "s/##QNAP_KDC##/${kdc}/g" > ${KRB5_CONF}
	else
		# Redundant DC
		dbg "create kerberos config with dual DCs"
		echo "[libdefaults]" > /etc/krb5.conf
		echo " default_realm = ${uc_name}" >> ${KRB5_CONF}
		echo "" >> ${KRB5_CONF}
		echo "[realms]" >> ${KRB5_CONF}
		echo " ${uc_name} = {" >> ${KRB5_CONF}
		for i in ${kdc}
		do
			echo "  kdc = $i" >> ${KRB5_CONF}
		done
		echo " }" >> ${KRB5_CONF}
		echo "" >> ${KRB5_CONF}
		echo "[domain_realms]" >> ${KRB5_CONF}
		for i in ${kdc}
		do
			echo " .$i = ${uc_name}" >> ${KRB5_CONF}
		done
	fi

	unset kdc

	#
	# kinit test
	#

	write_progress authentication
	dbg "kinit test start"
	log "[command] echo ******** | ${KINIT} \"${user_name}@${uc_name}\""
	echo ${passwd} | ${KINIT} "${user_name}@${uc_name}" >> $DEBUG_FILE 2>&1
	case $? in
	0)
		dbg "kinit test success"
		echo "setup krb5 success"
		;;
	127)
		if [ $# = 4 ]; then
			dbg "kinit failed, check DNS and AD server name"
			echo "kinit failed: please check DNS and AD server name"
			return 4
		fi
		;;
	*)
		log "[command] echo ******** | ${KINIT} \"${user_name}@${uc_name}\""
		tmp=`echo ${passwd} | ${KINIT} "${user_name}@${uc_name}" 2>&1 1>>$DEBUG_FILE` 
		clk=`echo ${tmp} | ${GREP} -qc "Clock skew too great"`
		if [ "${clk}" = "0" ]; then
			dbg "kinit failed, can't find JDC realm"
			echo "kinit failed: can't find KDC realm!"
			return 2
		else
			dbg "kinit failed: too much time difference"
			echo "kinit failed: too much time difference!"
			return 3
		fi
	esac

	dbg "kinit test end"
	dbg "setup kerberos 5 end"

	return 0
}

update_ad_user_group() {
	_sed="/bin/sed"
	_netbios_name=`$GETCFG global workgroup -f $SMB_CONF`
	_security=`$GETCFG global security -d user -f $SMB_CONF`
	_smbd_locks="/usr/local/samba/var/locks"
	if [ "x${_security}" = "xADS" ]; then
			_trusted_netbios=/tmp/.trusted_netbios
			/bin/rm /tmp/.trusted_domain
	                /bin/rm /tmp/.trusted_netbios
			/sbin/get_trusted_domain  | /bin/cut -f 3 -d '[' | /bin/cut -f 1 -d ']' > ${_trusted_netbios}
			while read _netbios_name; do
				if [ "x$1" = "x\\" ]; then
					_from=${_netbios_name}\\\\
				else
					_from=${_netbios_name}$1
				fi
				if [ "x$2" = "x\\" ]; then
					_to=${_netbios_name}\\\\
				else
					_to=${_netbios_name}$2
				fi
				${_sed} -i "s/${_from}/${_to}/g" /etc/config/smb.conf
			done < ${_trusted_netbios}
		_winbind_separator=`${GETCFG} Samba "winbind separator" -d "\\\\"`
		if [ "x${_winbind_separator}" != "x+" ]; then
			$SETCFG -e global 'winbind separator' -f $SMB_CONF
		else
			$SETCFG global 'winbind separator' ${_winbind_separator} -f $SMB_CONF
		fi
	fi
}

update_password_server()
{
	return 0
	_tmpDC=/tmp/.tmpDC
	_password_server=
	/bin/grep "security = ADS" $SMB_CONF 2>/dev/null 1>/dev/null
	if [ $? = 0 ]; then
		_realm=`$GETCFG global realm -d user -f $SMB_CONF`
		/usr/bin/getDomainServers k ${_realm} | grep target > ${_tmpDC}
		if [ $? = 0 ]; then
			while read LINE; do
				_dc=`/bin/echo $LINE | /bin/cut -d = -f 2`
				_target=`/usr/bin/uppercase ${_dc} | /bin/cut -d . -f 1`
				_password_server=${_password_server}${_target}.${_realm}' '
			done < ${_tmpDC}
			/bin/echo "Update AD password server to ${_password_server}"
			$SETCFG global "password server" "${_password_server}" -f ${SMB_CONF}
		fi
	fi
}

update_krb5_ticket()
{
	/bin/grep "security = ADS" $SMB_CONF 2>/dev/null 1>/dev/null
	if [ $? = 0 ]; then
		${KINIT} -R
		if [ $? != 0 ]; then
			_kdc=`$GETCFG realms kdc -d 127.0.0.1 -f /etc/krb5.conf`
			${NET} time set -S ${_kdc}
			if [ $? != 0 ]; then
				TMP_FILE=/tmp/tmp.adserver
				/bin/ping -q -c 1 $_kdc > ${TMP_FILE}
				ADSERVER_IP=`/bin/cat ${TMP_FILE} | /bin/tr -cs '[0-9\.]' '\012' | /bin/awk -F'.' 'NF==4 && $1>0 && $1<256 && $2<256 && $3<256 && $4<256 && !/\.\./' | /bin/uniq`
				${NET} time set -S ${ADSERVER_IP} 2>/dev/null 1>/dev/null
			fi
			_passwd=${AD_PASSWD}
			_user_name=`$GETCFG Samba User -f /etc/config/uLinux.conf`
			_realm=`$GETCFG global realm -f ${SMB_CONF}`
			_dc_name=`/usr/bin/uppercase ${_realm}`
			echo ${_passwd} | ${KINIT} "${_user_name}@${_dc_name}"
			if [ $? = 0 ]; then
				echo "update krb5 ticket successfully"
			else
				echo "update krb5 ticket fail"
			fi
		else
			echo "renew krb5 ticket successfully"
		fi
	fi
}

#import_remote_accounts()
#{
#	echo "import remote accounts!"
#	# so far, we only import remote users
#	$WBINFO --import-users >/dev/null 2>&1 &
#}

#clean_imported_accounts()
#{
#	echo "kill imported remote accounts!"
#	# so far, we only import remote users
#	$WBINFO --kill-remote-users >/dev/null 2>&1 &
#}

#set_sync_account_schedule()
#{
#	echo "setting schedule to synchronize remote accounts"
#	# so far, only sync remote users
#	clear_sync_account_schedule
#	echo "0 6 * * * root /usr/bin/sync_remote_accounts.sh user>/dev/null 2>&1" >> /etc/crontab
#}

#clear_sync_account_schedule()
#{
#	cp /etc/crontab /tmp/crontab
#	${SED} -e '/sync_remote_accounts/d' /tmp/crontab > /etc/crontab
#	rm /tmp/crontab -f
#}

# main program 

auth_user=$3
auth_pwd=$4
enauth_pwd=`/sbin/get_encstr $auth_pwd e`
echo "======== DEBUG START =======" > $DEBUG_FILE

#
# kill redundant DC monitor first
#

#killall rddcmon

case "$1" in
	"ADM") 		#AD member
		write_progress configuration

		if [ $# -ne 6 ] && [ $# -ne 7 ]; then
			echo -n "USAGE $0 $1 [DOMAIN] [USER] [PASSWD] "
			echo "[WORKGROUP] [OU]"
			exit $USAGE_ERROR
		fi

		dbg_start
		dbg "AD_SERVER=${AD_SERVER}"
		dbg "AD_DOMAIN=${AD_DOMAIN}"
		dbg "samba stop"
		/etc/init.d/smb.sh stop > /dev/null
		
		LOCAL_SID=`/usr/local/samba/bin/net getlocalsid | /bin/cut -d ':' -f 2`
		dbg "Backup LOCAL_SID ${LOCAL_SID}"
		
		backup_conf_file
		[ ! -f /etc/config/secrets.tdb ] || /bin/mv /etc/config/secrets.tdb /etc/config/secrets.tdb_bak

		dbg "backup secrets.tdb to secrets.tdb_bak"

		#
		# setup_krb5
		#
		# 1. create /etc/config/krb5.conf
		# 2. time sync by net command
		# 3. kinit test
		#

		setup_krb5 $2 $3 $4 $5 > /dev/null
		case "$?" in
			"0")
				RETVAL=$SUCCESS
				;;
			"1")
				RETVAL=$KDC_SRV_ERROR
				;;
			"2")
				RETVAL=$KDC_REALM_ERROR
				;;
			"3")
				RETVAL=$KDC_TIME_DIFF_ERROR
				;;
			"4")
				RETVAL=$KDC_SRV_ADNAME_ERROR
				;;
			"5")
				RETVAL=$ALL_DC_DEAD
				;;
			*)
				RETVAL=$KNOWN_ERROR
		esac

		#
		# if setup_krb5 not success, start samba only
		#

		if [ $RETVAL -ne $SUCCESS ]; then
			restore_conf_file
			dbg "setup_krb5 fail, restore config and samba restart"
			dbg "Local SID Restoration : setlocalsid ${LOCAL_SID}"
			/usr/local/samba/bin/net setlocalsid ${LOCAL_SID}
			/etc/init.d/smb.sh start > /dev/null
			exit $RETVAL
		fi

		#
		# here, kinit success. join now
		#

		clean_smb_conf $SMB_CONF
		clean_winbind_cache
		write_progress joining_ad

		#
		# join AD
		# join_ad [sambaconfig] [domain] [user] [passwd] [server]
		#

		join_ad $SMB_CONF $2 $3 $4 $5
		if [ "$?" = "0" ]; then 
			clean_backup_file
			/etc/init.d/ads_register_dns.sh
			RETVAL=$SUCCESS
		else
			restore_conf_file
			RETVAL=$AD_JOIN_ERROR
		fi
		
		dbg "Local SID Restoration : setlocalsid ${LOCAL_SID}"
		/usr/local/samba/bin/net setlocalsid ${LOCAL_SID}
		
		write_progress starting_service
		$SETCFG "Samba" "User" "$auth_user"
		$SETCFG "Samba" "Password" "$enauth_pwd"
		$SETCFG "Samba" "OU" "$AD_OU"
		$0 update_password_server > /dev/null

		dbg "samba start"

		/etc/init.d/smb.sh start > /dev/null
		add_printer_admin_main
		clean_domain_db
		idmap_rid=`${GETCFG} Samba "idmap_rid" -d "FALSE"`
		if [ $RETVAL = $SUCCESS -a "$idmap_rid" = "FALSE" ]; then
			/bin/cat /etc/config/crontab | /bin/grep idmap.sh > /dev/null
			if [ $? -ne 0 ]; then
				echo "0 3 * * 0 /etc/init.d/idmap.sh dump" >> /etc/config/crontab
				/usr/bin/crontab /etc/config/crontab
			fi
		else
			/bin/sed -i '/idmap.sh/d' /etc/config/crontab
			/usr/bin/crontab /etc/config/crontab
		fi
#		import_remote_accounts
#		set_sync_account_schedule
		write_progress finish

		#
		# Redundant DC status monitor
		#

#		if [ $RETVAL = $SUCCESS ] && \
#		   [ ! -z "${KDC1}" ] && \
#		   [ ! -z "${KDC2}" ]; then
#			dbg "enable redundant DC monitor"
#			/usr/bin/rddcmon dynamic ${KDC1} ${KDC2} &
#		fi
		dbg_end
		exit $RETVAL
		;;
	"NTM")		#NT member
		[ "$#" = "4" ] || ( echo "USAGE: $0 $1 NT_DOMAIN ADM_ACCOUNT ADM_PASSWD!"; exit $USAGE_ERROR )
		/etc/init.d/smb.sh stop > /dev/null
		backup_conf_file
		clean_smb_conf $SMB_CONF
		clean_winbind_cache
		join_ntd $SMB_CONF $2 $3 $4
		if [ "$?" = "0" ]; then 
			clean_backup_file
			[ ! -f ${KRB5_CONF} ] || rm -f ${KRB5_CONF}
			RETVAL=$SUCCESS
		else
			restore_conf_file
			RETVAL=$NTD_JOIN_ERROR
		fi
		$SETCFG "Samba" "User" "$auth_user"
		$SETCFG "Samba" "Password" "$enauth_pwd"
		/etc/init.d/smb.sh start > /dev/null
		sleep 3
                add_printer_admin_main
		clean_domain_db

#		import_remote_accounts
#		set_sync_account_schedule
		exit $RETVAL
		;;
#	"PDC")		#NT PDC
#		;;
#	"BDC")		#NT BDC
#		;;
	"ALONE")	#Stand Alone Server

		#
		# $1 = ALONE
		# $2 = WORKGROUP(ex: FRED)
		#

		[ "$#" = "2" ] || ( echo "USAGE: $0 $1 WORKGROUP_NAME!"; exit $USAGE_ERROR )
		/etc/init.d/idmap.sh leave_dump
		/bin/sed -i '/idmap.sh/d' /etc/config/crontab
		/usr/bin/crontab /etc/config/crontab
		/etc/init.d/smb.sh stop > /dev/null
#		if [ `$GETCFG global security -f $SMB_CONF -u -d ADS` = ADS ]; then
#			$NET ads leave
#		fi
		backup_conf_file
		clean_smb_conf $SMB_CONF
		alone_set_smb_conf $SMB_CONF
		add_printer_admin_main
#		clean_imported_accounts
		/etc/init.d/smb.sh start > /dev/null
		clean_backup_file
		clean_domain_db
#		clear_sync_account_schedule
		[ ! -f ${KRB5_CONF} ] || rm -f ${KRB5_CONF}
		;;
	"update_winbind_separator")
		update_ad_user_group $2 $3
		;;
	"update_password_server")
		update_password_server
		;;
	"update_krb5_ticket")
		update_krb5_ticket
		;;
	*)
		echo "USAGE: $0 [ADM|NTM|ALONE|update_winbind_separator|update_password_server|update_krb5_ticket] [arguments]!"
		exit $USAGE_ERROR
		;;
esac
