#!/usr/local/apache/bin/php
<?php
setlocale(LC_CTYPE, "en_US.UTF-8");

define('DEBUG', 'true');
define('LOG_PATH', '/tmp/php_dmc_log');
define('APP_TYPE', 'FileStation');

class MediaController {
    private static $CMD = '/usr/local/medialibrary/bin/qRPlayerAgent';
    private static $err = array();

    /*

        
        General Query functions


    */

    public static function searchDevices(){
        return self::simpleExec(self::$CMD.' -c Search');
    }

    public static function listRenderers() {
        $ret = array();
        $data = self::simpleExec(self::$CMD.' -c ListRender');
        if($data['status'] == 0){
            if(isset($data['device']['deviceName'])){
                $deviceType = $data['device']['@attributes']['Type'];
                $deviceSubtype = $data['device']['@attributes']['Subtype'];
                $data['device']['deviceType'] = $deviceType;
                $data['device']['deviceSubtype'] = $deviceSubtype;
                $data['device']['installed'] = 'true';
                if (empty($data['device']['type'])) {
                    $data['device']['type'] = 'video,photo,audio';
                }
                unset($data['device']['@attributes']);
                $ret[]['device'] = $data['device'];
            }else if(is_array($data['device'])) {
                foreach ($data['device'] as &$device) {
                    $deviceType = $device['@attributes']['Type'];
                    $deviceSubtype = $device['@attributes']['Subtype'];
                    $device['deviceType'] = $deviceType;
                    $device['deviceSubtype'] = $deviceSubtype;
                    $device['installed'] = 'true';
                    if (empty($device['type'])) {
                        $device['type'] = 'video,photo,audio';
                    }
                    unset($device['@attributes']);
                    $ret[]['device'] = $device;
                }
            }
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }

        $data = self::simpleExec(self::$CMD.' -c ListUnknowRender');
        if($data['status'] == 0){
            if(isset($data['device']['deviceName'])){
                $deviceType = $data['device']['@attributes']['Type'];
                $data['device']['deviceType'] = $deviceType;
                $data['device']['installed'] = 'false';
                unset($data['device']['@attributes']);
                $ret[]['device'] = $data['device'];
            }else if(is_array($data['device'])) {
                foreach ($data['device'] as &$device) {
                    $deviceType = $device['@attributes']['Type'];
                    $device['deviceType'] = $deviceType;
                    $device['installed'] = 'false';
                    unset($device['@attributes']);
                    $ret[]['device'] = $device;
                }
            }
        }

        return $ret;
    }

    public static function listPlayers() {
        $ret = array();
        $data = self::simpleExec(self::$CMD.' -c ListPlayer');
        if($data['status'] == 0){
            if(isset($data['player']['renderID'])){
                $ret[]['player'] = $data['player'];
            }else if(is_array($data['player'])){
                foreach ($data['player'] as &$player) {
                    $ret[]['player'] = $player;
                }
            }
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    /*

        
        Player functions


    */

    public static function play($dev) {
        $ret = array();
        $dev = escapeshellarg($dev);
        $data = self::simpleExec(self::$CMD." -c Play --dev $dev");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function pause($dev) {
        $ret = array();
        $dev = escapeshellarg($dev);        
        $data = self::simpleExec(self::$CMD." -c Pause --dev $dev");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function stop($dev) {
        $ret = array();
        $dev = escapeshellarg($dev);
        $data = self::simpleExec(self::$CMD." -c Stop --dev $dev");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function next($dev) {
        $ret = array();
        $dev = escapeshellarg($dev);
        $data = self::simpleExec(self::$CMD." -c Next --dev $dev");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function prev($dev) {
        $ret = array();
        $dev = escapeshellarg($dev);
        $data = self::simpleExec(self::$CMD." -c Previous --dev $dev");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function seek($dev, $time=0) {
        $ret = array();
        $dev = escapeshellarg($dev);
        $time = escapeshellarg($time);
        $data = self::simpleExec(self::$CMD." -c Seek --dev $dev --position $time");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function setTrack($dev, $track=1){
        $ret = array();
        $dev = escapeshellarg($dev);
        $track = escapeshellarg($track);
        $data = self::simpleExec(self::$CMD." -c SetTrack --dev $dev --track $track");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function setPlayMode($dev, $mode='NORMAL'){
        $ret = array();
        /*Mode: NORMAL|REPEAT|SHUFFLE|SHUFFLE_REPEAT|REPEAT_ONE_E*/
        $dev = escapeshellarg($dev);
        $mode = escapeshellarg($mode);
        $data = self::simpleExec(self::$CMD." -c SetPlayMode --dev $dev --playMode $mode");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function setSlideshowDelay($dev, $delay){
        $ret = array();
        $dev = escapeshellarg($dev);
        $delay = escapeshellarg($delay);
        $data = self::simpleExec(self::$CMD." -c SetSlideShowDelay --dev $dev --delay $delay");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function getPlayerStatus($dev){
        $ret = array();
        $dev = escapeshellarg($dev);
        $data = self::simpleExec(self::$CMD." -c GetPlayerStatus --dev $dev");
        if($data['status'] == 0){
            if ($data['trackContentType'] == 'file') {
                $data['file_path'] = $data['trackContent'];
                $data['file_title'] = pathinfo($data['trackContent'], PATHINFO_BASENAME);
            } else {
                $dbhandle = new mysqli('127.0.0.1', 'read', 'read', 's01', 3310, '/tmp/mysql_mediadb.sock');
                if ($dbhandle) {
                    $dbhandle->set_charset("utf8");
                    switch ($data['trackType']) {
                        case 'video':
                            $sql = 'SELECT dir.cFullPath, t.cFileName, t.cVideoTitle FROM videoTable as t LEFT OUTER JOIN dirTable as dir ON dir.iDirId = t.iDirId WHERE t.iVideoId=' . $data['trackContent'];
                            $result = $dbhandle->query($sql);
                            $row = $result->fetch_row();
                            $result->close();
                            $data['file_path'] = realpath('/share/' . $row[0] . $row[1]);
                            $data['file_title'] = $row[2];
                            break;

                        case 'photo':
                            $sql = 'SELECT dir.cFullPath, t.cFileName, t.cPictureTitle FROM pictureTable as t LEFT OUTER JOIN dirTable as dir ON dir.iDirId = t.iDirId WHERE t.iPictureId=' . $data['trackContent'];
                            $result = $dbhandle->query($sql);
                            $row = $result->fetch_row();
                            $result->close();
                            $data['file_path'] = realpath('/share/' . $row[0] . $row[1]);
                            $data['file_title'] = $row[2];
                            break;

                        case 'audio':
                            $sql = 'SELECT dir.cFullPath, t.cFileName, t.cSongTitle, t.cArtistName FROM songTable as t LEFT OUTER JOIN dirTable as dir ON dir.iDirId = t.iDirId WHERE t.iSongId=' . $data['trackContent'];
                            $result = $dbhandle->query($sql);
                            $row = $result->fetch_row();
                            $result->close();
                            $data['file_path'] = realpath('/share/' . $row[0] . $row[1]);
                            $data['file_title'] = $row[3] . ' - ' .$row[2];
                            break;
                    }
                    $dbhandle->close();
                } else {
                    $data['file_path'] = 'unknown';
                    $data['file_title'] = 'unknown';
                }
            }

            if (isset($data['device']['@attributes'])) {
                $data['deviceType'] = $data['device']['@attributes']['Type'];
                $data['deviceSubtype'] = $data['device']['@attributes']['Subtype'];
                unset($data['device']);
            }

            $ret = $data;
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function getPlayerVolume($dev){
        $ret = array();
        $dev = escapeshellarg($dev);        
        $data = self::simpleExec(self::$CMD." -c GetVolume --dev $dev");
        if($data['status'] == 0){
            $ret = $data;
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function setPlayerVolume($dev, $vol){
        $ret = array();
        $dev = escapeshellarg($dev);
        $vol = escapeshellarg($vol);
        $data = self::simpleExec(self::$CMD." -c SetVolume --dev $dev --vol $vol");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function getPlayerMute($dev){
        $ret = array();
        $dev = escapeshellarg($dev);
        $data = self::simpleExec(self::$CMD." -c GetMute --dev $dev");
        if($data['status'] == 0){
            $ret = $data;
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function setPlayerMute($dev, $mute){
        $ret = array();
        $dev = escapeshellarg($dev);
        $mute = escapeshellarg($mute);
        $data = self::simpleExec(self::$CMD." -c SetMute --dev $dev --mute $mute");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function setPlaylist($dev, $title, $app){
        $ret = array();

        $dev = escapeshellarg($dev);
        $title = escapeshellarg($title);
        $app = escapeshellarg($app);

        $data = self::simpleExec(self::$CMD." -c SetPlaylist --dev $dev --playlist $title --appType ".APP_TYPE);
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    /*

        
        Playlist functions


    */

    public static function listPlaylists($app, $offset=0, $count=100){
        $ret = array();
        $app = escapeshellarg($app);
        $offset = escapeshellarg($offset);
        $count = escapeshellarg($count);
        $data = self::simpleExec(self::$CMD." -c ListPlaylist --appType $app --start $offset --count $count");
        if($data['status'] == 0){
            if(isset($data['playlist']['playlistID'])){
                    $ret[]['list'] = $data['playlist'];
            }else if(is_array($data['playlist'])){
                foreach ($data['playlist'] as $playlist) {
                    $ret[]['list'] = $playlist;
                }
            }
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function getPlaylistCount($app){ 
        $app = escapeshellarg($app);  
        return self::simpleExec(self::$CMD." -c SizeOfPlaylist --appType $app");
    }

    public static function listPlaylistContent($title, $app, $offset=0, $count=100){
        $ret = array();
        $title = escapeshellarg($title);
        $app = escapeshellarg($app);
        $offset = escapeshellarg($offset);
        $count = escapeshellarg($count);
        $data = self::simpleExec(self::$CMD." -c ListContentOfPlaylist --playlist $title --appType $app --start $offset --count $count");
        if($data['status'] == 0){
            if(isset($data['item']['playlistID'])){
                    $ret[]['item'] = $data['item'];
            }else if(is_array($data['item'])) {
                foreach ($data['item'] as $item) {
                    $ret[]['item'] = $item;
                }
            }
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function getPlayListContentCount($title, $app){
        $title = escapeshellarg($title);
        $app = escapeshellarg($app);
        return self::simpleExec(self::$CMD." -c SizeOfListContent --playlist $title --appType $app");
    }

    public static function removePlaylist($title, $app){
        $ret = array();
        $title = escapeshellarg($title);
        $app = escapeshellarg($app);
        $data = self::simpleExec(self::$CMD." -c DelPlaylist --playlist $title --appType $app");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    public static function removeContent($title, $app, $contentId){
        $title = escapeshellarg($title);
        $app = escapeshellarg($app);
        $contentId = escapeshellarg($contentId);
        return self::simpleExec(self::$CMD." -c DelContent --playlist $title --appType $app --uuid $contentId");
    }

    public static function createPlaylist($title, $app, $type='photo', $owner='guest', $tag='all'){
        $ret = array();

        $title = escapeshellarg($title);
        $type = escapeshellarg($type);
        $app = escapeshellarg($app);
        $owner = escapeshellarg($owner);
        $tag = escapeshellarg($tag);

        $data =  self::simpleExec(self::$CMD." -c CreatePlaylist --playlist $title --playlistType $type --appType $app --owner $owner --tag $tag");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;

    }

    public static function appendPlaylist($title, $app, $mediaType, $mediaId){
        $title = escapeshellarg($title);
        $app = escapeshellarg($app);
        $mediaType = escapeshellarg($mediaType);
        $mediaId = escapeshellarg($mediaId);

        $data = self::simpleExec(self::$CMD." -c AddContent --playlist $title --appType $app --contentType 'dbID' --mediaType $mediaType --content $mediaId ");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }

        return $ret;
    }

    public static function appendFile($title, $app, $mediaType, $file){
        $title = escapeshellarg($title);
        $app = escapeshellarg($app);
        $mediaType = escapeshellarg($mediaType);
        $file = escapeshellarg($file);

        $data = self::simpleExec(self::$CMD." -c AddContent --playlist $title --appType $app --contentType 'file' --mediaType $mediaType --content $file ");
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }

        return $ret;
    }

    public static function commitPlaylist($title){
        $data = self::simpleExec(self::$CMD." -c BatchAddPlaylist --filePath ".escapeshellarg($title));
        if($data['status'] == 0){
            $ret['msg'] = 'success';
        }else{
            $ret['error'] = $data['status'];
            $ret['msg'] = self::getErrorMsg($data['status']);
        }
        return $ret;
    }

    private static function getErrorMsg($status){
        /*
        SYSTEM_ERR                  -1
        IPC_FORMAT_ERR              -2
        PARAMTER_NOT_FOUND_ERR      -3
        RENDER_NOT_EXIST_ERR        -4
        CREATE_PLAYER_FAIL_ERR      -5
        QDMCD_NOT_EXIST_ERR         -6
        COMMAND_NOT_FOUND_ERR       -7
        UNKNOW_PLAY_MODE_ERR        -8
        RETURN_FORMAT_ERR           -9
        PLAYLIST_NOT_FOUND_ERR      -10
        PLAYQUEUE_DB_QUERY_ERR      -11
        PLAYQUEUE_DB_ACCESS_ERR     -12
        FILE_NOT_IN_DB_ERR          -13
        PLAYLIST_NOT_EXIST_ERR      -14
        FILE_OPEN_FAIL_ERR          -15
        XML_FILE_FORMAT_ERR         -16
        INTERNAL_XML_FORMAT_ERR     -17
        */
        switch ($status) {
            case '-1':
                return 'System error';
            case '-2':
                return 'Invalid IPC format';
            case '-3':
                return 'Missing parameters';
            case '-4':
                return 'Renderer not found';
            case '-5':
                return 'Fail to create player';
            case '-6':
                return 'QDMCD does not exist';
            case '-7':
                return 'Command not found';
            case '-8':
                return 'Unknown play mode';
            case '-9':
                return 'Return format error';
            case '-10':
                return 'Playlist not found';
            case '-11':
                return 'Playquery query error';
            case '-12':
                return 'Playqueue access error';
            case '-13':
                return 'No matching database entry'; 
            case '-14':
                return 'Playlist does not exist';
            case '-15':
                return 'Fail to open file';
            case '-16':
                return 'XML file format error';
            case '-17':
                return 'Internal XML format error'; 
            case '-99':
                return 'Unable to execute command'; 
            default:
                return 'Unknown error';
        }

    }

    private static function simpleExec($cmd){
        exec($cmd . ' 2>/dev/null', $output, $status);

        if(DEBUG == 'true'){
            if(@filesize(LOG_PATH)>32*1024){
                rename(LOG_PATH, LOG_PATH.'.1');
            }
            if ($status!=0) {
               exec('echo "[error]['.$status.'] '.$cmd.'" >> '.LOG_PATH);
            } else {
               exec('echo "[success] '.$cmd.'" >> '.LOG_PATH);
            }
        }

        if($status!=0){
            return array('status'=>'-99');
        }else{
            $xml = simplexml_load_string(implode('', $output));
            if($xml)
                $xml_array = unserialize(serialize(json_decode(json_encode((array) $xml), 1)));
            else
                return array('status'=>'-999');
            return $xml_array;
        }
    }

}

function printOutput($input){
    echo json_encode($input, JSON_UNESCAPED_UNICODE)."\n";
}

function getCommandInput($input='', $default=''){
    global $argv;

    if(empty($input)) return '';

    $ret = array();

    foreach ($argv as $key=>$value) {
        if($value == '-'.$input){
            $ret[] = $argv[$key+1];
        }
    }

    $count = count($ret);
    if($count==1){
        return $ret[0];
    }else if($count>1){
        return $ret;
    }

    return '';
}


/********************************************************
*********************************************************
    API endpoints
*********************************************************
*********************************************************/

$action = $argv[1];

switch ($action) {

    case 'search':
        MediaController::searchDevices();
        printOutput(array('status'=>'0'));
        break;

    case 'listRenderers':
        $playerList = array();
        $players = MediaController::listPlayers();
        if(is_array($players)){
            foreach ($players as &$player) {
                $playerList[] = $player['player']['renderID'];
            }
        }

        $renderers = MediaController::listRenderers();   

        if(isset($renderers['error'])){
            printOutput(array('status'=>$renderers['error'], 'msg'=>$renderers['msg']));
        }else{
            $list = array();   
            if(is_array($renderers)){
                foreach ($renderers as &$renderer) {
                    if(in_array($renderer['device']['deviceId'], $playerList)){ 
                        $renderer['device']['active'] = 'true';
                        $info = MediaController::getPlayerStatus($renderer['device']['deviceId']);
                        unset($info['status']);
                        $renderer['device']['playerStatus'] = $info;
                    }else{
                        $renderer['device']['active'] = 'false';
                        $renderer['device']['playerStatus'] = '';
                    }
                    
                    $list[] = $renderer;
                }
            }
            printOutput(array('status'=>'0', 'deviceCount'=>count($list), 'devices'=>$list));
        }
        break;

    case 'listPlayers':
        $ret = MediaController::listPlayers();
        if(isset($ret['error']))
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        else
            printOutput(array('status'=>'0', 'playerCount'=>count($ret), 'players'=>$ret));
        break;

    case 'listPlayersStatus':
        $playerList = array();
        $ret = MediaController::listPlayers();

        if ( isset($ret['error']) ) {
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        } else {
            if (is_array($ret)) {
                foreach ($ret as &$player) {
                    $player['player']['status'] = MediaController::getPlayerStatus($player['player']['renderID']);
                    $playerList[] = $player;
                }
            }
            printOutput(array('status'=>'0', 'playerCount'=>count($playerList), 'players'=>$playerList));
        }
        break;

    case 'listContents':
        $title = getCommandInput('t');
        $deviceId = getCommandInput('d');
        $page = getCommandInput('p', 1);
        $count = getCommandInput('c', 100);
        if(empty($title)){          
            $playerStatus = MediaController::getPlayerStatus($deviceId);
            $title = $playerStatus['playlistTitle'];
        }
        $ret = MediaController::listPlaylistContent($title, APP_TYPE, ($page-1)*$count, $count);
        if (isset($ret['error'])) {
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        } else {
            printOutput(array('status'=>'0', 'itemCount'=>count($ret), 'items'=>$ret));
        }
        //clears status cache       
        $ret = MediaController::listPlayers();
        foreach ($ret as $player) {
            $playerStatus = MediaController::getPlayerStatus($player['player']['renderID']);
            if ($playerStatus['playlistTitle'] == $title) {
                flushStatus($player['player']['renderID']);
            }
        }
        break;

    case 'listPlaylists':
        $ret = MediaController::listPlaylists(APP_TYPE);
        if(isset($ret['error']))
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        else
            printOutput(array('status'=>'0', 'listCount'=>count($ret), 'lists'=>$ret));
        break;

    case 'getPlayerStatus':
        $deviceId = getCommandInput('d');
        $ret = fetchStatus($deviceId);
        if(isset($ret['error']))
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        else
            printOutput($ret);
        break;

    case 'play':
        $deviceId = getCommandInput('d');
        if(isOutOfSync($deviceId)){
            printOutput(array('status'=>'-2', 'msg'=>'Other application is using the device'));
        }else{
            $ret = MediaController::play($deviceId);
            $status = fetchStatus($deviceId, false);
            if ($status['trackType'] != 'photo') { //do not wait for photo because it doesn't have a playing time
                $status = fetchStatus($deviceId);
            } else {
                flushStatus($deviceId);
            }
            if(isset($ret['error']))
                printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
            else
                printOutput(array('status'=>'0', 'time'=>$status['currTime']));
        }
        break;

    case 'stop':
        $deviceId = getCommandInput('d');
        $ret = MediaController::stop($deviceId);
        if(isset($ret['error']))
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        else
            printOutput(array('status'=>'0'));
        break;

    case 'pause':
        $deviceId = getCommandInput('d');
        if(isOutOfSync($deviceId)){
            printOutput(array('status'=>'-2', 'msg'=>'Other application is using the device'));
        }else{
            $ret = MediaController::pause($deviceId);
            if(isset($ret['error']))
                printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
            else
                printOutput(array('status'=>'0'));
        }
        break;

    case 'next':
        $deviceId = getCommandInput('d');
        if(isOutOfSync($deviceId)){
            printOutput(array('status'=>'-2', 'msg'=>'Other application is using the device'));
        }else{
            $ret = MediaController::next($deviceId);
            $status = fetchStatus($deviceId, false);
            if ($status['trackType'] != 'photo') { //do not wait for photo because it doesn't have a playing time
                $status = fetchStatus($deviceId);
            } else {
                flushStatus($deviceId);
            }
            if(isset($ret['error']))
                printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
            else
                printOutput(array('status'=>'0', 'time'=>$status['currTime']));
        }
        break;

    case 'prev':
        $deviceId = getCommandInput('d');
        if(isOutOfSync($deviceId)){
            printOutput(array('status'=>'-2', 'msg'=>'Other application is using the device'));
        }else{
            $ret = MediaController::prev($deviceId);
            $status = fetchStatus($deviceId, false);
            if ($status['trackType'] != 'photo') { //do not wait for photo because it doesn't have a playing time
                $status = fetchStatus($deviceId);
            } else {
                flushStatus($deviceId);
            }
            if(isset($ret['error']))
                printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
            else
                printOutput(array('status'=>'0', 'time'=>$status['currTime']));
        }
        break;

    case 'seek':
        $deviceId = getCommandInput('d');
        $seconds = getCommandInput('s');
        if(isOutOfSync($deviceId)){
            printOutput(array('status'=>'-2', 'msg'=>'Other application is using the device'));
        }else{
            $ret = MediaController::seek($deviceId, $seconds);
            $status = fetchStatus($deviceId, false);
            if ($status['trackType'] != 'photo') { //do not wait for photo because it doesn't have a playing time
                $status = fetchStatus($deviceId);
            } else {
                flushStatus($deviceId);
            }
            if(isset($ret['error']))
                printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
            else
                printOutput(array('status'=>'0', 'time'=>$status['currTime']));
        }
        break;

    case 'jump':
        $deviceId = getCommandInput('d');
        $track = getCommandInput('o');
        if(isOutOfSync($deviceId)){
            printOutput(array('status'=>'-2', 'msg'=>'Other application is using the device'));
        }else{
            $ret = MediaController::setTrack($deviceId, $track);
            $status = fetchStatus($deviceId, false);
            if ($status['trackType'] != 'photo') { //do not wait for photo because it doesn't have a playing time
                $status = fetchStatus($deviceId);
            } else {
                flushStatus($deviceId);
            }
            if(isset($ret['error']))
                printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
            else
                printOutput(array('status'=>'0', 'time'=>$status['currTime']));
        }
        break;

    case 'setPlayMode':
        $deviceId = getCommandInput('d');
        $mode = getCommandInput('m');
        if(isOutOfSync($deviceId)){
            printOutput(array('status'=>'-2', 'msg'=>'Other application is using the device'));
        }else{
            $ret = MediaController::setPlayMode($deviceId, $mode);
            if(isset($ret['error']))
                printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
            else
                printOutput(array('status'=>'0'));
        }
        break;

    case 'setSlideshowInterval':
        $deviceId = getCommandInput('d');
        $seconds = getCommandInput('i');
        if(isOutOfSync($deviceId)){
            printOutput(array('status'=>'-2', 'msg'=>'Other application is using the device'));
        }else{
            $ret = MediaController::setSlideshowDelay($deviceId, $seconds);
            if(isset($ret['error']))
                printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
            else
                printOutput(array('status'=>'0'));
        }
        break;

    case 'setVolume':
        $deviceId = getCommandInput('d');
        $volume = getCommandInput('v');
        $ret = MediaController::setPlayerVolume($deviceId, $volume);
        if(isset($ret['error']))
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        else
            printOutput(array('status'=>'0'));
        break;

    case 'setMute':
        $deviceId = getCommandInput('d');
        $mute = getCommandInput('m');
        $ret = MediaController::setPlayerMute($deviceId, $mute);
        if(isset($ret['error']))
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        else
            printOutput(array('status'=>'0'));
        break;

    case 'getVolume':
        $deviceId = getCommandInput('d');
        $ret = MediaController::getPlayerVolume($deviceId);
        if(isset($ret['error']))
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        else
            printOutput($ret);
        break;

    case 'getMute':
        $deviceId = getCommandInput('d');
        $ret = MediaController::getPlayerMute($deviceId);
        if(isset($ret['error']))
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        else
            printOutput($ret);
        break;

    case 'setPlaylist':
        $deviceId = getCommandInput('d');
        $playlistId = getCommandInput('t');

        MediaController::stop($deviceId);
        MediaController::setPlaylist($deviceId, $playlistId, APP_TYPE);
        $ret = MediaController::play($deviceId);
        if(isset($ret['error']))
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        else
            printOutput(array('status'=>'0'));
        break;

    case 'createPlaylist':
        $title = getCommandInput('t',uniqid('ps'));
        $mediaType = getCommandInput('m', 'photo');
        $owner = getCommandInput('o','guest');
        $tag = getCommandInput('k','ALL');

        $ret = MediaController::createPlaylist($title, APP_TYPE, $mediaType, $owner, $tag);
        if(isset($ret['error']))
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        else
            printOutput(array('status'=>'0', 'title'=>$title));
        break;

    case 'removePlaylist':  
        $title = getCommandInput('t');
        $ret = MediaController::removePlaylist($title, APP_TYPE);
        if(isset($ret['error']))
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        else
            printOutput(array('status'=>'0'));
        break;

    case 'addPhoto':
        $title = getCommandInput('t');
        $filepath = getCommandInput('f');
        $ret = MediaController::appendFile($title, APP_TYPE, 'photo', realpath($filepath));
        if(isset($ret['error']))
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        else
            printOutput(array('status'=>'0'));
        break;

    case 'addVideo':
        $title = getCommandInput('t');
        $filepath = getCommandInput('f');
        $ret = MediaController::appendFile($title, APP_TYPE, 'video', realpath($filepath));
        if(isset($ret['error']))
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        else
            printOutput(array('status'=>'0'));
        break;

    case 'addMusic':
        $title = getCommandInput('t');
        $filepath = getCommandInput('f');
        $ret = MediaController::appendFile($title, APP_TYPE, 'audio', realpath($filepath));
        if(isset($ret['error']))
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        else
            printOutput(array('status'=>'0'));
        break;

    case 'createPlaylistBatch':
        // $appType, $playlistTitle, $owner, $list, $tmpDir='/tmp'
        $title = getCommandInput('t');
        $playlistTitle = getCommandInput('f');

        $commandPath = tempnam('/tmp', 'dmc_');

        $content = '<root>';
        $content .= '<playlist playlistType="video">'.$playlistTitle.'</playlist>';

        $ret = file_put_contents($commandPath, $content);        

        if ($ret) {
            printOutput(array('status'=>'0'));
        } else {
            printOutput(array('status'=>'1'));
        }

        break;

    case 'addPhotoBatch':
        $title = getCommandInput('t');
        $filepath = getCommandInput('f');

        $content = '<item mediaType="photo" contentType="file">'.$filepath.'</item>';
        $ret = file_put_contents($title, $content, FILE_APPEND);

        if ($ret) {
            printOutput(array('status'=>'0'));
        } else {
            printOutput(array('status'=>'1'));
        }
        break;

    case 'addVideoBatch':
        $title = getCommandInput('t');
        $filepath = getCommandInput('f');
        
        $content = '<item mediaType="video" contentType="file">'.$filepath.'</item>';
        $ret = file_put_contents($title, $content, FILE_APPEND);

        if ($ret) {
            printOutput(array('status'=>'0'));
        } else {
            printOutput(array('status'=>'1'));
        }
        break;

    case 'addMusicBatch':
        $title = getCommandInput('t');
        $filepath = getCommandInput('f');
        
        $content = '<item mediaType="audio" contentType="file">'.$filepath.'</item>';
        $ret = file_put_contents($title, $content, FILE_APPEND);

        if ($ret) {
            printOutput(array('status'=>'0'));
        } else {
            printOutput(array('status'=>'1'));
        }
        break;

    case 'commitPlaylistBatch':
        $title = getCommandInput('t');
        $owner = getCommandInput('o','guest');

        $content = '<appType>'.APP_TYPE.'</appType>';
        $content .= '<owner>'.$owner.'</owner>';
        $content .= '</root>';

        $ret = file_put_contents($title, $content, FILE_APPEND);

        $ret = MediaController::commitPlaylist($title);
        if (isset($ret['error'])) {
            printOutput(array('status'=>$ret['error'], 'msg'=>$ret['msg']));
        } else {
            printOutput(array('status'=>'0'));
        }
        break;

    default:
        ?>
  Usage:
  <?php echo $argv[0]; ?> <action> 
  
  [Query actions]

  List renderers: 
    <?php echo $argv[0]; ?> listRenderers

  List players:
    <?php echo $argv[0]; ?> listPlayers

  List playlist contents:
    <?php echo $argv[0]; ?> listContents

  List playLists:
    <?php echo $argv[0]; ?> listContents


  [Device Actions]

  getPlayerStatus:
    <?php echo $argv[0]; ?> getPlayerStatus -d <device id>

  Play:
    <?php echo $argv[0]; ?> play -d <device id>
  Stop:
    <?php echo $argv[0]; ?> stop -d <device id>

  Pause:
    <?php echo $argv[0]; ?> pause -d <device id>

  Next:
    <?php echo $argv[0]; ?> next -d <device id>

  Prev:
    <?php echo $argv[0]; ?> prev -d <device id>

  Get mute flag:
    <?php echo $argv[0]; ?> getMute -d <device id>
  Get player volume:

    <?php echo $argv[0]; ?> getVolume -d <device id>

  Jump:
    <?php echo $argv[0]; ?> jump -d <device id> -o <track number>

  Seek:
    <?php echo $argv[0]; ?> seek -d <device id> -s <time in seconds>

  Set player mute flag:
    <?php echo $argv[0]; ?> setMute -d <device id> -m <on|off>

  setVolume:
    <?php echo $argv[0]; ?> setVolume -d <device id> -v <1-100>

  set player mode:
    <?php echo $argv[0]; ?> setPlayMode -d <device id> -m <NORMAL|REPEAT|SHUFFLE|SHUFFLE_REPEAT|REPEAT_ONE_E>

  setSlideshowInterval:
    <?php echo $argv[0]; ?> setSlideshowInterval -d <device id> -i <time in seconds>


  [Playlist Actions]

  createPlaylist:
    <?php echo $argv[0]; ?> createPlaylist -t <playlist title> -m <playlist type> -o <owner> -k <tags>

  removePlaylist:
    <?php echo $argv[0]; ?> removePlaylist -t <playlist title>

  setPlaylist:
    <?php echo $argv[0]; ?> setPlaylist -t <playlist title> -d <device id>

  Add photo to playlist:
    <?php echo $argv[0]; ?> addPhoto -t <playlist title> -f <file path>

  Add video to playlist:
    <?php echo $argv[0]; ?> addVideo -t <playlist title> -f <file path>

  Add music to playlist:
    <?php echo $argv[0]; ?> addMusic -t <playlist title> -f <file path>

  [Create playlist in a batch]

  Initialize a playlist batch:
    <?php echo $argv[0]; ?> createPlaylistBatch -t <playlist file> -f <playlist name>

  Add photo to playlist:
    <?php echo $argv[0]; ?> addPhotoBatch -t <playlist file> -f <file path>

  Add video to playlist:
    <?php echo $argv[0]; ?> addVideoBatch -t <playlist file> -f <file path>

  Add music to playlist:
    <?php echo $argv[0]; ?> addMusicBatch -t <playlist file> -f <file path>

  Commit playlist:
    <?php echo $argv[0]; ?> commitPlaylistBatch -t <playlist file> -o <owner>


        <?php
        echo "\n";
}

function isOutOfSync($deviceId){
    $playerStatus = MediaController::getPlayerStatus($deviceId);
    if($playerStatus['appType'] == APP_TYPE){
        return false;
    }
    return true;
}

function fetchStatus($deviceId, $buffer=true){

    $statusFile = '/tmp/FS-DMR-'.$deviceId;
    $queryCount = 0;
    $queryLimit = 30;
    $doItAgain = true;

    if (file_exists($statusFile)) {
        $oldStatus = json_decode(file_get_contents($statusFile), true);
    } else {
        $oldStatus = '';
    }

    do {
        $doItAgain = $buffer;
        $ret = MediaController::getPlayerStatus($deviceId);

        if ( $ret['trackType'] == 'photo' ){
            $ret['currTime'] = '00:00:00';
            $ret['totalTime'] = '00:00:00';
        }

        //if there are no changes since the last query, keep querying for 30 seconds            
        if (is_array($oldStatus) && $queryCount < $queryLimit) {
            foreach ($oldStatus as $key => $value) {
                if ($value != $ret[$key]) {
                    $doItAgain = false;                     
                    break;
                }
            }
        } else {
            $doItAgain = false;
        }

        //if the video/music is ending in 10 seconds, wait until it's ending
        $timeDiff = timeToSeconds($ret['totalTime']) - timeToSeconds($ret['currTime']);
        if ( !empty($oldStatus) && !$doItAgain && $timeDiff > 0 && $timeDiff < 10 ) {
            //sleep through the time diff
            sleep($timeDiff);
            $doItAgain = true;
        }

        //only query again if there were no errors
        if ($ret['status'] == 0 && $doItAgain) {
            $queryCount++;
            sleep(1);
        } else {
            $doItAgain = false;
        }     

    } while ($doItAgain);

    //if doItAgain is true at this point, that means nothing has changed for the last 30 seconds
    //and if that happends and the current state is 'PLAY', change the state to 'PAUSE'
    if ( $doItAgain && $ret['playerState'] == 'PLAY' ) {
        $ret['playerState'] = 'PAUSE';
    }

    if ( $ret['playerState'] == 'PLAY' ) {
        //if there is a status cache in the beginning but now there isn't, don't resave it.
        if (empty($oldStatus) || file_exists($statusFile)) {
            file_put_contents($statusFile, json_encode($ret, JSON_UNESCAPED_UNICODE));
        }
    } else {
        flushStatus($deviceId);
    }

    if ( empty($oldStatus) && file_exists($statusFile) ) {
        chown($statusFile, 99); //httpdusr
        chmod($statusFile, 0666);
    }

    $info = MediaController::getPlayerVolume($deviceId);
    $mute = MediaController::getPlayerMute($deviceId);
    $ret['volume'] = is_numeric($info['volume']) ? $info['volume'] : '-1';
    $ret['mute'] = $mute['mute'];

    return $ret;
}

function timeToSeconds($input='00:00:00'){
    if (is_array($input)) {
        return 0;
    }
    $tmp = array_map('intval', explode(':', $input));
    return 60*60*$tmp[0] + 60*$tmp[1] + $tmp[2];
}

function flushStatus($deviceId){
    if ( empty($deviceId) ) {
        return false;
    }
    $statusFile = '/tmp/FS-DMR-'.$deviceId;
    @unlink($statusFile);
}
?>
