<?php
/*
    $Id: subtitle_cli.php,v 1.3 2014/09/26 01:51:03 johnsonlee Exp $
    Copyright 2004-2014 QNAP Systems,Inc. ALL Rights Reserved.
*/
namespace Captioning;
use Captioning\Format\SubripFile;
use Captioning\Format\WebvttFile;
use Captioning\Format\SubstationalphaFile;
use Captioning\Format\MicrodvdFile;

setlocale(LC_CTYPE, "en_US.UTF-8");

include('CueInterface.php');
include('Cue.php');
include('FileInterface.php');
include('File.php');
include('Converter.php');
include('Format/SubripCue.php');
include('Format/SubripFile.php');
include('Format/WebvttCue.php');
include('Format/WebvttFile.php');
include('Format/SubstationalphaCue.php');
include('Format/SubstationalphaFile.php');
include('Format/MicrodvdCue.php');
include('Format/MicrodvdFile.php');

class Subtitle {
    /*
    @param $inputFile   Full path to the subtitle text
    @param $outputFile  Full path to the converted subtitle
    @param $offset      Subtitle offset in ms
    */
    public static function convert($inputFile, $outputFile='', $offset=0){

        if(!$inputFile) return false;
            
        //check for file encoding
        $oldOutput = file_get_contents($inputFile);
        if( !mb_check_encoding( $oldOutput, 'UTF-8' ) ){
            $encoding = mb_detect_encoding($oldOutput);
            if(!$encoding){
                //assumes it's UCS-2 if we can't detect it.
                $newOutput = mb_convert_encoding( $oldOutput, 'UTF-8', 'UCS-2' ); 
            }else{
                $newOutput = mb_convert_encoding( $oldOutput, 'UTF-8', $encoding ); 
            }
            $inputFile .= '.utf8.srt';
            file_put_contents($inputFile, $newOutput);            
        }

        $pathInfo = pathinfo($inputFile);

        switch ($pathInfo['extension']) {
            case 'sub':
                $subtitle = new MicrodvdFile($inputFile);
                break;

            case 'srt':
                $subtitle = new SubripFile($inputFile);
                break;

            case 'vtt':
                $subtitle = new WebvttFile($inputFile);
                break;

            case 'ssa':
                $subtitle = new SubstationalphaFile($inputFile);
                break;
            
            default:
                exit('unknown format');
                break;
        }    

        if (empty($outputFile)) {            
            $outputFile = $pathInfo['dirname'].'/'.$pathInfo['filename'].'.srt';
        }

        if ($offset>0) {
            $cues = $subtitle->getCues();
            foreach ($cues as $index => $cue) {  
                $start = $cue->getStartMS();
                $stop = $cue->getStopMS();
                if ($start < $offset && $stop < $offset) {      
                    $subtitle->removeCue($index);
                }
            }
            $subtitle->sortCues();
            $subtitle->shift($offset*-1);
        }

        if (strtolower($pathInfo['extension']) == 'srt') {
            $subtitle->build()->save($outputFile);
        }else{
            $subtitle->convertTo('subrip')->build()->save($outputFile);
        }

        return true;

    }
}

function getCommandInput($input='', $default=''){
    global $argv;

    if(empty($input)) return '';

    $ret = array();

    foreach ($argv as $key=>$value) {
        if($value == '-'.$input){
            $ret[] = $argv[$key+1];
        }
    }

    $count = count($ret);
    if($count==1){
        return $ret[0];
    }else if($count>1){
        return $ret;
    }

    return '';
}

$offset = getCommandInput('offset', 0); // in ms
$outputFile = getCommandInput('output', '');
$inputFile = getCommandInput('input', '');

Subtitle::convert($inputFile, $outputFile, $offset);


